
/*
 * Copyright (C) Narf Industries <info@narfindustries.com>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/


#ifndef MIX_CHEM
#define MIX_CHEM

const static int N_FORMULAS = 3000;
typedef struct chem_formula {
	char *compound_name;
	double mole;
} chem_formula_t;

static chem_formula_t chem_formulas[N_FORMULAS] = {
	{ "[CH2-].CN1C2=C(C=CC3=C2C(=NC(=C)C3=C)[O-])[N-]C1=N.[Rb+].[Y]", 0.688187645035 },
	{ "(C)N(CC(=O)OC)C(=O)C=CC1=CC=CC(=C1)C#N", 1.98716677345 },
	{ "WVGDXNDJTJGGGO-UHFFFAOYSA-M", 1.26072600093 },
	{ "BWAXEZEPDEQDF-UHFFFAOYSA-N", 0.402192937932 },
	{ "[2-amino-3-(1-carbamoylimidazol-4-yl)propanoyl] dodec", 1.53462936647 },
	{ "1=C(C=CC(=C1)C=CC2=NC(=C3N2CCCC3)C4=CC=CC=C4)[N+]5=CNC(=C5", 2.3372847241 },
	{ "2-[2-(2-diphenylphosphinophenyl)-45-dihydrooxazol-4-yl]-2-prop", 2.3228368645 },
	{ "4-butan-2-yl-1-phenacylpiperidine-2-carboxamid", 2.15830805831 },
	{ "3-iodanyl-7-(trifluoromethyl)-1H-ind", 1.11842585446 },
	{ "hI=1S/C7H16N2/c1-5-4-8-7(3)9-6(5)2/h5-9H4H21-3H3", 2.01968122843 },
	{ "69H125N2O25", 1.95054368547 },
	{ "LARADLMQDLEQTL-UHFFFAOYSA-N", 0.23982585707 },
	{ "12H24O7", 0.645160935792 },
	{ "FDGAXXDROKQIPL-UHFFFAOYSA-N", 2.23212371696 },
	{ "19H26Cl2O3", 2.34471708972 },
	{ "1(C(=O)N(C(=NC1(C)C2=C(C=CC(=C2)CC3CC4CC3CC4O)F)N", 0.123698109962 },
	{ "hI=1S/C11H22/c1-8(2)10(9-6-7-9)11(34)5/h8-10H6-7H21-5H3", 1.7103790715 },
	{ "AGSAQMZLPYSPQ-UHFFFAOYSA-N", 1.24310035665 },
	{ "WJPOAPVMSBMLEO-UHFFFAOYSA-N", 0.11493544672 },
	{ "N1C=CN=C1CNC(=O)C2=C3N(C4CCCCC4N3C5C(C2=O)CC(C(N5)N6CCOCC6)F", 0.755655059599 },
	{ "HZZKURGSKSEPNO-UHFFFAOYSA-N", 1.99202351885 },
	{ "13H5Cl2F6N3O2S", 1.34360094756 },
	{ "24H28FN7O2", 0.333570088783 },
	{ "51H76O6Si", 0.539486641554 },
	{ "LJKWKNHAMYDWIF-UHFFFAOYSA-O", 1.98980820694 },
	{ "QIFKMVYSFROYRH-UHFFFAOYSA-N", 2.33371674261 },
	{ "39H56O4", 2.14892189234 },
	{ "hyl N-tert-butyl-N-(33-dimethylbutylideneamino)carbam", 1.38837382985 },
	{ "RZEIWTSOPIIPRM-UHFFFAOYSA-N", 0.19231744749 },
	{ "19H18N4O9", 0.954210227463 },
	{ "N-[(4-bromo-2-methyl-3-pyridyl)methyleneamino]methanami", 0.378919751058 },
	{ "QIKDMLMOWKRDKU-UHFFFAOYSA-N", 1.28757529568 },
	{ "ZGCRVGYARCBSAL-UHFFFAOYSA-N", 0.812116023355 },
	{ "ZPRRVWMRNNCJSR-UHFFFAOYSA-N", 1.33566142665 },
	{ "3-cyano-2-propene-1", 0.680560304629 },
	{ "(CN1C=CC=N1)N2C=NC3=C(C2=O)C=C4C(=C3)C(=O)N(N=N4)C5CC5", 1.98045718192 },
	{ "20H26F2O2", 0.571297797914 },
	{ "24H17F5", 1.50221220635 },
	{ "23H40N2O4S", 1.21170627564 },
	{ "WIYKGDNEEQTTEF-UHFFFAOYSA-N", 2.35834138178 },
	{ "2-amino-33-dimethyl-1-[4-(2-methylpropyl)-1-piperazinyl]-1-b", 1.1379015029 },
	{ "KQLTFXIGGVXPR-UHFFFAOYSA-N", 1.64384604698 },
	{ "2-[(6-bromo-2-naphthyl)oxy]-N&apos;-hydroxy-propanamidi", 1.96403807229 },
	{ "hI=1S/C14H24.Y/c1-9-12(6)13(10(2)3)14(78)11(4)5;/h1-8H3;/q-2;", 1.97135646765 },
	{ "2-azanyl-1-(2-ethylmorpholin-4-yl)-33-dimethyl-butan-1", 0.825713566485 },
	{ "2-[[morpholin-2-yl(phenyl)methyl]thio]nicotinonitri", 2.32461770914 },
	{ "(CNCC[NH-])N.[Pd]", 1.60689243742 },
	{ "WWRVCDUACSHVST-UHFFFAOYSA-N", 1.34422364868 },
	{ "(=CCCCCCCC(=O)C1C(CC2C1CC(=C2)CCCCC(=O)O)O", 0.609006522616 },
	{ "HFAXQJBFLJPDBL-UHFFFAOYSA-N", 1.19692810137 },
	{ "XOUFOZKXTIZZIJ-UHFFFAOYSA-N", 1.31768782522 },
	{ "JMLCTLOYUNJVAB-UHFFFAOYSA-N", 1.54932757763 },
	{ "N-[(4-but-2-enoxy-35-dimethyl-phenyl)methyl]ethanami", 0.292495538773 },
	{ "29H37N5O6", 2.00693847444 },
	{ "(C(=CC(=O)C(C)(C)C)O[Sn])O", 1.91961979107 },
	{ "UUIPMSJHNDINX-UHFFFAOYSA-N", 0.563551690287 },
	{ "7H11FmNO4", 0.77506999171 },
	{ "OC1=CC=CC2=C1N=C(N3C2=NC(=N3)CN4CCCC5=C4N=CC=C5)N", 0.906295256535 },
	{ "OCCOCC#[C-].[Mg+2].[Br-]", 0.336427808945 },
	{ "FGPRITAHWOTNKJ-UHFFFAOYSA-N", 1.82228690153 },
	{ "17H22N4", 2.34432236009 },
	{ "(C1=CC2=CC=CC=C2C=C1)SC3=NNC4N3C5CCCCC5C(=O)N4CCCO", 0.30293760965 },
	{ "TYNGMNNIWIJIW-UHFFFAOYSA-N", 1.51135046317 },
	{ "2-oxo-2-pyridin-4-yl-N-(4-sulfamoylanilino)ethanimidoyl cyanid", 2.35733758023 },
	{ "2-pyrazolidin-4-ylquinoli", 0.280658251448 },
	{ "2-(23-dichlorophenoxy)-N&apos;-hydroxy-6-methyl-nicotinamidi", 0.896276620705 },
	{ "57H77N9O8S2", 0.814777178604 },
	{ "4-amino-5-(26-diamino-1-oxohexoxy)-5-oxopentanoic acid", 1.26417942398 },
	{ "HFCGAIZALANJOA-UHFFFAOYSA-N", 0.954785722557 },
	{ "38H46N4O6", 0.663853172529 },
	{ "2-amino-4-(methylthio)-1-(4-phenoxypiperidino)butan-1", 0.872872169041 },
	{ "(C(=S)N)NC(=O)COC1=C(C=C(C=C1", 0.85255529481 },
	{ "YREQMSPSCVSGMQ-UHFFFAOYSA-N", 2.25600632519 },
	{ "1=C(C=C(C=C1F)C(C)NC(=O)C2=COC(=N2)OC3=CC(=CC=C3)Br)F", 1.80844914121 },
	{ "FUSGPTUCEPKVPP-UHFFFAOYSA-N", 2.32399640341 },
	{ "GEPUFNMSXFQJFA-UHFFFAOYSA-N", 2.3635099295 },
	{ "1=CC=C(C=C1)C(C)C(=O)C2CNN3C2=NC(CC3(C)C)C4=CC=CC=C4", 0.605615674794 },
	{ "THGBCFELOAQCM-UHFFFAOYSA-N", 1.73201988553 },
	{ "N&apos;-hydroxy-3-(hydroxymethyl)cyclopentane-1-carboximidamid", 2.09632376106 },
	{ "KGFOCKUSOKFJFU-UHFFFAOYSA-N", 1.61085449592 },
	{ "YPNBMUVYSSXROU-UHFFFAOYSA-N", 0.456801469533 },
	{ "KFKOEZHATRMACP-UHFFFAOYSA-N", 1.29929084955 },
	{ "7-[5-(4-phenoxyphenyl)tetrazol-2-yl]heptane-123457-hex", 0.371090648896 },
	{ "2-(3-ethylpyridin-1-ium-1-yl)-2-phenyleth", 2.04711663408 },
	{ "40H68O5Si2", 1.91107287411 },
	{ "35H49O2", 1.2222724641 },
	{ "RMRBRLZXVNKDQ-UHFFFAOYSA-N", 2.36575701408 },
	{ "XKOYUACCTAWFNW-UHFFFAOYSA-N", 1.11051833738 },
	{ "iridium(3+);1-methyl-45-diphenyl-imidaz", 1.04084819508 },
	{ "[2-(25-difluorophenyl)-2-oxo-ethyl] 2-tert-butyl-4-oxo-p", 0.896575031447 },
	{ "OZYUEUIGKBVIE-UHFFFAOYSA-N", 1.43033983694 },
	{ "BIXTXPPGVBRCGJ-UHFFFAOYSA-L", 1.60166358996 },
	{ "N1C2=NC=[N+](C2C(=O)N(C1=O)C)CC(=O)NN", 1.73191794137 },
	{ "4-prop-1-enyl-2-(4-prop-1-enyl-2-pyridyl)pyridi", 1.21253726502 },
	{ "OC1=CN=C2C=CC(=O)N(C2=C1)CCN3CCC(C(C3)O)NCC4=NC=C5C(=C4)CCCO5", 0.403556336652 },
	{ "AJPHZZZPDZROTB-UHFFFAOYSA-N", 1.88275531597 },
	{ "hI=1S/C10H9NOS/c11-6-9(12)7-1-2-10-8(5-7)3-4-13-10/h1-5H611H2", 1.91226708837 },
	{ "12H16FN5O3", 1.94797758487 },
	{ "1=NN(C(=C1C=CC(=O)O)N(CC(=O)N)C(", 0.904710584937 },
	{ "22H16N8O", 1.40098646706 },
	{ "MVHLLOKIHAUEK-UHFFFAOYSA-N", 0.35099844933 },
	{ ".CCCC.CCCC.CC(=NNC(=O)N)C1=CC=C(C=C1)C2=CC=CC=C2.[Sn+4]", 0.677578612913 },
	{ "methyl 2-(22-dimethylpropanoyl)-5-phenyl-pent-4", 0.934033495444 },
	{ "WPDTVGXDMGMFQN-UHFFFAOYSA-N", 2.36227554916 },
	{ "3-(2-methylpyridin-4-yl)-5-nitro-1-(triphenylmethyl)indaz", 2.34468998045 },
	{ "13H24O4S2Y-2", 1.75303099094 },
	{ "1=C(C(=CC2=NC(=O)C(N=C21)C(=O)NC(=O)N", 0.415705659945 },
	{ "6-fluoro-2-methyl-3-prop-1-enyl-quinoxali", 0.984782649694 },
	{ "34H49F2N4O16PS2", 1.80644099169 },
	{ "2-azanyl-3-(4-ethylphenyl)prop", 0.288469604697 },
	{ "SYRRJHUHNUCRY-UHFFFAOYSA-N", 0.384184634817 },
	{ "XDHNVSQAFGZEFH-UHFFFAOYSA-N", 0.143359596561 },
	{ "N-[1-[4-(hydroxycarbamoyl)phenyl]propyl]-4-phenylbenzamid", 0.577344039855 },
	{ "UXUUDISFFMSPJO-UHFFFAOYSA-N", 0.934286083835 },
	{ "1C2C(C(=O)N2C(=C1COC3=CC=CC4=C3C=CC=C4CN=C(N)N)C(=O)[O-])C(C)O", 1.67753841566 },
	{ "2-(1-butoxycarbonyl-2-carbethoxy-4-piperidyl)acetonitrilium", 1.63133232654 },
	{ "12CCC3(C(C1CC(=CF)C2(C(=O)CO)O)C(C(C4=CC(=O)C(=O)CC43C)F)Cl)F", 1.88449382197 },
	{ "2-amino-4-methylpentanoic acid phenylmethoxycarbonyl ester", 1.95007589824 },
	{ "(C)CC(C(=O)O)N(CC(=O)O)C(=O)C(C)CSC(=O", 2.29109390193 },
	{ "dimethyl-(3-methyl-6-quinolyl)ami", 0.425236644769 },
	{ "1CN(C(=O)NC1=O)C2C(C(C(O2)C(C)CP(=O)(C)C)O)OCC(=O)N", 0.459835913799 },
	{ "1CC(CN(C1)CCCN2CCN(C(C2=O)C)C(=O)C=CC3=CC(=C(C=C3)Cl)Cl)O", 0.724161163434 },
	{ "1CCC2C(C1)C3C(CCNC3NC2=O)C#CC4=CC=CC5=CC=CC=C54", 2.01265445087 },
	{ "1=CC(=CC=C1)N2C3=NC(=NC(=O)C3C=N2)CC4CCCC4", 2.10204734903 },
	{ "OTGZTCBCQXTBSL-UHFFFAOYSA-N", 1.6345512364 },
	{ "11H13NO2S", 0.224317407188 },
	{ "MIERRJLVIXLDHQ-UHFFFAOYSA-N", 1.39029027033 },
	{ "LFQIBFVVDLOMMM-UHFFFAOYSA-N", 0.399543257447 },
	{ "30H36BrNO4", 0.791155294981 },
	{ "NMSSIBIAFOEQPI-UHFFFAOYSA-N", 2.37257930134 },
	{ "QBPROCFNSHOFLT-UHFFFAOYSA-N", 0.923114918876 },
	{ "N1C=C(C=N1)C2=C(C=C(C(=N2)NCCC3=CC(=CC=C3)F)C(=O)NCC(CO)N)F", 1.43570990687 },
	{ "rt-butyl N-(4-ethyl-2-hydroxy-3-methylcyclopentyl)carbam", 0.245032523443 },
	{ "N-decylidene-2-methyl-2-propanesulfinamid", 0.842998577803 },
	{ "GFZAXQWCBAOTQV-UHFFFAOYSA-N", 0.243945974163 },
	{ "KUJUSTBBOKWYSG-UHFFFAOYSA-N", 1.85075411096 },
	{ "JIUFGXHCBVLJPW-UHFFFAOYSA-N", 1.30842492182 },
	{ "1=CC(=CC=C1C(=O)OCCCOC(=O)C(CC2=CC(=C(C=C2)O)O)N)F", 1.70865063506 },
	{ "(C)C1=NC(=CS1)CN(C)C(=O)NC(COC(C)(C)C)C(=O)O", 0.774596748864 },
	{ "OIAXWNIEUIBPLW-UHFFFAOYSA-N", 0.798244327042 },
	{ "ZVSOCLICFZULJC-UHFFFAOYSA-N", 2.03771534946 },
	{ "JTLNPXACZUPTTC-UHFFFAOYSA-N", 1.91183488588 },
	{ "[3-[33-bis(fluoranyl)cyclohexyl]cyclohexyl]benz", 1.35266594571 },
	{ "3-[3-fluoro-4-(phenylmethoxymethyl)phenyl]prop-2-enoic acid", 1.42005256555 },
	{ "4-[6-(2-aminopyrrolidin-1-yl)sulfonyl-2-pyridyl]benzonitri", 1.26411138853 },
	{ "37H67N3O16", 0.119835405453 },
	{ "1CCC2=C(C1)SC3C2C(=O)NC(N3)C(C)SC4=NC5=CC=CC=C5O4", 2.28051098554 },
	{ "WHJSQJPOYIKQGS-UHFFFAOYSA-N", 1.12629009272 },
	{ "XFFNCKSCANWAST-UHFFFAOYSA-N", 0.513175758968 },
	{ "(C)(C)CO[C-]=O.[Y]", 1.17528167124 },
	{ "28H53N2O6", 1.46442753398 },
	{ "7-azanyl-2-(4-methoxyphenyl)-3aH-pyrazolo[43-c]quinolin-3", 0.808323356749 },
	{ "UPBDSGBIBDCNOF-UHFFFAOYSA-N", 0.721793573308 },
	{ "(CN(C)S(=O)(=O)C1=CC(=C(C=C1)F)F)C(=NO)N", 0.873549485726 },
	{ "UCTOUAHHUOINPC-UHFFFAOYSA-N", 1.91219915247 },
	{ "1=[C-]C2=CC=CC=C2C(=O)O1.[Y]", 0.496998636108 },
	{ "GBFAPUUQXHRDMR-UHFFFAOYSA-N", 1.70103493064 },
	{ "JYLWRLKCLTWXBW-GDNBJRDFSA-N", 0.152769285883 },
	{ "SYIDXKYCRYEHMK-UHFFFAOYSA-N", 0.439443556163 },
	{ "5H9N3S3", 2.3081264665 },
	{ "GMJJQHUDGLPRNN-UHFFFAOYSA-N", 1.256086028 },
	{ "1-pyrrolidino-3-(135-trimethylpyrazol-4-yl)prop-2-en-1", 0.686513511784 },
	{ "FSILKBWVFHUHIK-UHFFFAOYSA-N", 1.48819486785 },
	{ "1C2C1CC(C2)C3=CN=C(N3)C4=CC=C(C=C4)Br", 1.59307321361 },
	{ "YMFRDDMLXSEN-UHFFFAOYSA-N", 0.580340277508 },
	{ "KKNNWSYBTXHHJ-UHFFFAOYSA-N", 1.51857112355 },
	{ "YRNCLRVUGVRAHN-UHFFFAOYSA-N", 1.59292600505 },
	{ "(C)NC(=O)CCNC(=O)COC1=C(C=C(C=C1)C=CC)O", 2.37693110139 },
	{ "RYOZCEIBXGXLQM-UHFFFAOYSA-N", 1.55992381689 },
	{ "12CCCCC1CCC3C2CCC4(C3CCC4OC5=CC=C(C=C5)N6C=CC=C6", 1.64631111983 },
	{ "N1C2=CC=CC=C2SC1=NNC(=O)C3=CC(=NC=C3)N4C=NC=N4", 0.487123050381 },
	{ "2-azanyl-4-methyl-1-oxidanylidene-pentane-1-sulfi", 1.99933686546 },
	{ "N(CCF)CC1=CC2=NC(=O)C3=C(C2C(=C1)OC)NCCC3", 2.25530213729 },
	{ "1=CC(=CC=C1CN2C(NC(=O)N(C2=O)CCO)NC3=CC(=C(C=C3)OC(F)F", 0.262791226567 },
	{ "42H48N10O6S2", 1.31850431089 },
	{ "26H31FN4O4", 0.144526097269 },
	{ "WVYRALBPFRYZBR-UHFFFAOYSA-N", 0.414664464575 },
	{ "BNQVULMMJREPLY-UHFFFAOYSA-N", 2.13472605588 },
	{ "(C)C1=NN(N=N1)C2=CC(=C(C=C2)OC)C=CCCCCC3=CSC(=N3)C(", 2.01376280495 },
	{ "NZPFVXPZMOTAGA-UHFFFAOYSA-N", 1.08910881416 },
	{ "(7-fluoranyl-4-methyl-23-dihydro-14-benzoxazin-2-yl)meth", 0.474735172778 },
	{ "18H14FN3O4", 2.2715419008 },
	{ "iridium;2-(methylthio)-12H-imidazo[12-f]phenanthridin-12-id", 1.17404281389 },
	{ "N(C1=CC=CC=C1CON=C(C)C(=NOC)C2=CC=C(C=C2)F)C(=O)O", 0.757759829828 },
	{ "4-[6-[2-methyl-4-(1-naphthalenyl)phenyl]-1-pyrenyl]dibenzofur", 0.284882067206 },
	{ "OC1CN(CCC1CNC(=O)C2=C3C(=C(C=C2)OC)OCCO3)CCCC(=O)N", 0.819177653001 },
	{ "methyl 7-[2-[4-(1-acetoxyhexyl)phenyl]-3-oxo-cyclopentyl]hept-5", 0.205707845459 },
	{ "8-chloro-2-cyclohexyl-7-(22-dimethoxyethoxy)-8H-quinolin-4", 1.90348660265 },
	{ "WVUWTKFSROIUHE-UHFFFAOYSA-N", 1.21220595888 },
	{ "hyl 5-chloranyl-1H-pyrrolo[23-c]pyridine-7-carboxy", 1.63085415904 },
	{ "MMOUABQLLDOMDT-UHFFFAOYSA-N", 2.3012024134 },
	{ "KEIJZGDPFVRVDH-UHFFFAOYSA-J", 1.15038749466 },
	{ "FQDIRMURFRIHM-UHFFFAOYSA-N", 1.8182190706 },
	{ "UEUCBIOEUSPFIJ-UHFFFAOYSA-N", 1.07096411945 },
	{ "WJDTBRKVYHLCA-UHFFFAOYSA-N", 1.56734320185 },
	{ "2-[(2-keto-1-methyl-butyl)thio]hexahydropyrimidin-4", 1.05526222425 },
	{ "HYMLBRBWOMVJFX-UHFFFAOYSA-N", 1.9387127909 },
	{ "(C)C(CCNC(=O)NC)C1=C(C=C(C=C1F)F)F", 2.278009723 },
	{ "28H38N2O8S", 2.14861284347 },
	{ "SCCC(=O)CC1C2CCC(C1C=CC(C3=CC=CC=C3)O)O2", 1.39705078952 },
	{ "OHNSJXOHFPXAET-UHFFFAOYSA-N", 1.39592616203 },
	{ "1=CC(=C(C=C1)NC(=O)C2CC(NC(=O)C2)O)O", 1.97266783664 },
	{ "ARVPBWBJPPDJMK-UHFFFAOYSA-N", 0.614649854651 },
	{ "11H13NO5", 1.57517691287 },
	{ "21H24O7S", 2.00870644283 },
	{ "(C)(C)C(C(=O)OC)NCCCC#", 1.74357470293 },
	{ "N1CC(CC2C1CCC2)O", 2.31397592749 },
	{ "1-methyl-4-(methylamino)-2-(oxidanylmethylidene)pyrrol-3", 0.659710731457 },
	{ "(5-isopropylpyridazin-3-yl)-dimethyl-ami", 1.14758684009 },
	{ "AECBJPQQYDQJP-UHFFFAOYSA-T", 1.9136121432 },
	{ "QDUBNTIVXGYJRB-UHFFFAOYSA-N", 0.715832993866 },
	{ "26-di(phenyl)pyridine;gold;triphenylphosphanium", 1.32877303261 },
	{ "ZDWPJWFOPZBSTP-UHFFFAOYSA-N", 0.54177890919 },
	{ "1CC2C(CC1C=C3C4=CC=CC=C4NC3=O)NNC2C5=CC6=CC=CC=C6N=C5", 1.77213483111 },
	{ "N-(2-amino-2-hydroxyiminoethyl)-N-methyl-3-isoxazolecarboxamid", 0.782580817363 },
	{ "JVIIZSWVJAAZQB-UHFFFAOYSA-N", 0.225087612638 },
	{ "NRBRRRGLNFFWMR-UHFFFAOYSA-N", 1.88725949055 },
	{ "FHFMHBHKTXFFBL-RVDMUPIBSA-N", 0.617252745467 },
	{ "(6Z)-24-dichloro-6-(1-hydrazinoethylidene)cyclohexa-24-dien-1", 0.599184003686 },
	{ "29H34F2N2O5S", 1.93103993212 },
	{ "N=C1C(=[N+](C(=O)N(C1=O)C)CC2=CC=CC=C2)N", 1.02909905974 },
	{ "VHYURLXJOVCFMO-UHFFFAOYSA-N", 1.20642069106 },
	{ "1CC(CN(C1)C(=O)C(CC2=CC=C(C=C2)O)N)C(=O)O", 1.36694481277 },
	{ "12CCC3C(C1CCC2(C(C(F)(F)F)(F)F)O)CCC4=C3C=CC5=C4C=NN5", 0.777570569574 },
	{ "UZHJJAXPARUBEE-UHFFFAOYSA-N", 0.537057107085 },
	{ "107H177N7O66", 1.30385816916 },
	{ "1=CC=C2C=C(C=CC2=C1)C(=O)C(CCCN)(C(=O)O)N", 2.00702836186 },
	{ "YVTXHAKBZHZTE-UHFFFAOYSA-N", 2.26131336204 },
	{ "29H30F3N3O3S", 0.967382789531 },
	{ "VSSMHIALVSERGV-UHFFFAOYSA-N", 0.95198984637 },
	{ "1CN2C(C1OC3=CC(=CC(=C3)[N+](=O)[O-])F)CNS2(=O)=O", 0.105331339465 },
	{ "VEBUQGZTSRAXLN-UHFFFAOYSA-N", 0.474943100149 },
	{ "1-mesyl-4-(2-methoxy-1-methyl-ethyl)peroxy-buta-13-di", 1.28259099017 },
	{ "3-ethoxy-5a9a-dihydrodibenzofuran-2", 0.277508006693 },
	{ "JDSIFCYOJJNCPV-UHFFFAOYSA-N", 2.01202658166 },
	{ "LGHYFKRFXOWCDH-UHFFFAOYSA-N", 1.50448824844 },
	{ "FYPSHGQOYLQV-UHFFFAOYSA-N", 2.00501928304 },
	{ "2-[[(2E)-2-methoxyiminoethyl]thio]-3-pyridi", 1.86968655682 },
	{ "38H36Cl2N2O2", 1.22034890499 },
	{ "RLRNRZGGXMMWFR-UHFFFAOYSA-N", 0.781025835454 },
	{ "RVWORPANOYGGLQ-UHFFFAOYSA-N", 0.480711175312 },
	{ "KFMQHOPOONZQGK-UHFFFAOYSA-N", 0.835242297249 },
	{ "MJFMHFGFAAKKA-UHFFFAOYSA-M", 0.930274642613 },
	{ "1=CC=C(C=C1)C(=O)C(C)CN2CCCCC2.C(C(C(C(C(C(=O)O)O)O)O)O)O", 0.388622008314 },
	{ "RZCUDLRSYITXBH-UHFFFAOYSA-N", 2.11921622798 },
	{ "TXGDWYOMMQGMW-UHFFFAOYSA-N", 2.18143351045 },
	{ "1=C(C(=O)[NH+]2CN(NC2=N1)CC)CC3=CC=C(C=C3)C4=CC=CC=C4C5=NNN=N5", 0.701065845624 },
	{ "OC1=CC=CC=C1C2C3C(NNC3C(=O)N2C4=CC=C(C=C4)F)C5=CN=CC=C5", 1.28205259136 },
	{ "1CCCCC1N2CN(C3=CN=C4C(=C32)C=CN4)CC5=C(ON=C5C6=CC=CC=C6", 1.73135415353 },
	{ "N(C)C(=O)COC1=CC=C(C=C1)C(=O)C=CC2=CSC(=N2)C3=CC=C(C=C3)F", 0.621369580896 },
	{ "rt-butyl 2-methyl-3-phenyl-prop", 0.400548425101 },
	{ "20H16BrClN4O3", 0.328659880611 },
	{ "27H31ClN2O5", 1.78447376046 },
	{ "46H63F2N3O14", 0.896547591596 },
	{ "14-ditert-butyl-3-propylpiperazin-2", 0.665184075529 },
	{ "1(CCCC1)NCC2=NC(=O)C3C(=N2)C=CS3", 0.265901432438 },
	{ "2-(23-dihydro-1H-inden-5-ylthio)-N-(3-pyrazolidinyl)acetamid", 1.43774401636 },
	{ "1C2CC(C2(C)C)C3=C1N=C(C=C3)C4=CC=CC=C4P(C5=CC=CC=C5)C6=CC=CC=C6", 1.71202469638 },
	{ "3-[24-bis(trifluoromethyl)phenyl]-5-(4-fluorophenyl)pyrazolidi", 0.521168353837 },
	{ "1=CC(=C(C(=C1)Cl)F)N2C=NC3C2=NC(=O)C=C3", 2.10764864295 },
	{ "TMZQLQXZRRQSM-UHFFFAOYSA-N", 0.57173362544 },
	{ "ATWHJNANMLFCGH-UHFFFAOYSA-N", 1.70128447517 },
	{ "STASBNNZXDKILK-UHFFFAOYSA-N", 1.53435299282 },
	{ "UVASPWKLVNSJIF-UHFFFAOYSA-N", 0.41157185174 },
	{ "12H18OSi", 0.1587537309 },
	{ "1C(OC(O1)(C)C)CC#CC#CC(C=C)OC(=O)C2=CC=C(C=C2)[N+](=O)[O-]", 1.81497576125 },
	{ "VICBPUZIFZQEFC-UHFFFAOYSA-N", 2.36339753693 },
	{ "VVZFZHROKHLRJT-UHFFFAOYSA-N", 0.912840462667 },
	{ "28H44F3N3O6", 0.243936301837 },
	{ "OC(=O)C1CCCCC=CCCCC(C(=O)N2CC3C(C2C(=O)N1)C3(C)C)NC(=O)OC(C)(", 0.343909849257 },
	{ "benzyl N-[6-(methylamino)cyclohex-3-en-1-yl]carbam", 0.67491032846 },
	{ "ULPBPNNFUIDHC-UHFFFAOYSA-N", 0.458339830173 },
	{ "XLAOOCCCOMWKMP-UHFFFAOYSA-N", 0.390449805742 },
	{ "JYTDPNMLPGZZBW-UHFFFAOYSA-N", 0.910988463744 },
	{ "OC(=O)C1=NN(C=C1C=C(C#N)C(=O)C2=CC(=CC=C2)OC)C3=CC=CC=C3", 1.30076428495 },
	{ "SJLHIMQZYKZPBS-UHFFFAOYSA-N", 1.06361738818 },
	{ "YOLATBRGIYZJGH-UHFFFAOYSA-M", 1.38457051932 },
	{ "16H14ClN5O", 0.181646624728 },
	{ "1COC2(O1)C3CC4CC2CC(C4)(C3)CNC(=O)OCC5=CC=CC=C5", 2.34602146286 },
	{ "(C1=CC=CC=C1)NC2=NC(=NC=C2C(=O)N)CC3=CC(=CC=C3)N(C4CC4)C(=O", 0.79034263305 },
	{ "1C(C(NN1)C)C(=O)N(C)C2CCCC3=CC=CC=C23", 0.252342280364 },
	{ "diphosphanyl-(1-phenylhex-4-en-3-yloxy)-phosphinophosphi", 1.99804054858 },
	{ "(C(=O)N)NCC1=NC2=C(N1CC3CCCO3)C=C(C=C2)OC4=CC(=C(C=C4)F)F", 1.48243514794 },
	{ "4-butyl-7-methylbenzimidazol-1-ide;yttrium", 1.50840854482 },
	{ "UFXKOJAHLHGIQK-UHFFFAOYSA-N", 2.39737087079 },
	{ "RDLCUKUVRCATFO-UHFFFAOYSA-N", 0.526137739029 },
	{ "XLNXTPWDSQRYPJ-UHFFFAOYSA-N", 0.754066911143 },
	{ "NLJXQGWSRFKTIV-UHFFFAOYSA-N", 1.32232199575 },
	{ "RCPLIWNGJWKWGM-UHFFFAOYSA-N", 0.789487626765 },
	{ "1=CC(=C(C(=C1)C)/C=C/C2CCNCC2", 2.33071334967 },
	{ "1CCC(C1)(C(=NO)N)C(=O)NCCC2=CC=CC=N2", 2.03684291835 },
	{ "N-(2-bromo-4-fluoro-phenyl)-3-(26-difluorophenyl)prop-2-enamid", 0.616120294689 },
	{ "19H24Cl2N4O5", 0.886728049642 },
	{ "1-(25-diisoamoxyphenyl)-2-(p-tolyl)ethene-12-dithiolate;nick", 0.649916490148 },
	{ "39H68O6", 1.9706851003 },
	{ "(=O)NCC1=CC=C(O1)COC2C(C(C(C(O2)CO)O)O)O", 1.89255332834 },
	{ "HOIICMGNSWIEOP-UHFFFAOYSA-N", 0.898670191792 },
	{ "(CC)N(C)C(C)C(=NO)N", 0.882544912561 },
	{ "hI=1S/C7H12O5/c1-3-5(10)6(11)7(12-3)4(9)2-8/h13-11H2H2", 0.774565312295 },
	{ "YBMOSCODWYUMJA-UHFFFAOYSA-N", 2.17006884099 },
	{ "16-dimethyl-4-propan-2-yloxy-3-pyrazolo[43-c]pyridinami", 1.38601687044 },
	{ "N-phenylpiperidin-4-ide-1-carboxamide;yttrium", 0.529612662247 },
	{ "rt-butyl N-(5H-pyrrolo[32-d]pyrimidin-4-yl)carbam", 0.345242052848 },
	{ "HNEPZBJFAKGTDZ-UHFFFAOYSA-N", 1.34808623522 },
	{ "NGCTTXTZCJLACA-UHFFFAOYSA-N", 0.614464313227 },
	{ "1CC2C(C(C3(O2)CCC4C5CCC6CC(=O)CCC6(C5CC4=C3C)C)C)NC1", 0.160354608227 },
	{ "3-[[4-(24-difluorophenyl)phenyl]sulfonylmethyl]-1-cyclohep", 1.01931938889 },
	{ "UQZZQFQLYXLIJP-UHFFFAOYSA-N", 0.574213818774 },
	{ "imino-(2-oxo-3-phosphanyloxy-propylidene)ammonium", 2.146228145 },
	{ "25H28Cl3FO8", 0.204040446578 },
	{ "NVIHHPOUPVHICP-UHFFFAOYSA-N", 0.487921898012 },
	{ "2-(imidazo[12-a]pyridin-5-ylthio)acetaldehyd", 1.89241569628 },
	{ "bis(phosphanyl)phosphane;molybdenum", 0.122739212496 },
	{ "JFSWRWSQNPZRJI-UHFFFAOYSA-N", 0.249117711932 },
	{ "RMIZUQBSFFTEM-UHFFFAOYSA-N", 2.12033252786 },
	{ "7H8N3O6+", 1.49977916705 },
	{ "13H15NO2", 1.14756851229 },
	{ "QNTPIUDNXRXINQ-UHFFFAOYSA-N", 0.702541372145 },
	{ "38H64N16O13", 1.41842620556 },
	{ "6-(2-cyclopentyl-1-hydroxy-ethyl)quinoxaline-23-di", 1.67599118575 },
	{ "42H54O4", 0.913379036725 },
	{ "QVGLYXDUIMQELV-UHFFFAOYSA-N", 0.673951812718 },
	{ "1CC(CN(C1)C2=CC=C(C=C2)CC(=O)C3CCC(CC3)CS(=O)(=O)C(", 0.543107490968 },
	{ "QGWGUYCTRCCVMB-UHFFFAOYSA-N", 1.11598475371 },
	{ "HXWOTCWGSFEID-UHFFFAOYSA-N", 1.75064079697 },
	{ "13H16Cl2N4OS", 1.82778267444 },
	{ "QVSRYYYHMIJQAY-UHFFFAOYSA-N", 1.20584853317 },
	{ "OCC(=O)C1=CN=C2C(=C1NC3C4CC5CC3CC(C5)(C4)O)C=CN2", 0.16416749335 },
	{ "19H24FNO4", 0.993906382671 },
	{ "NMYDZQLGDXBBU-UHFFFAOYSA-N", 0.790634248668 },
	{ "4-[3-[2-(1-naphthalenyl)propyl]-1-pyrrolidinyl]benzoic acid", 1.17794941795 },
	{ "31H23N4Pt-3", 0.100786206118 },
	{ "VBCQQEZBRJTJKQ-UHFFFAOYSA-N", 1.57130111415 },
	{ "14H16N2", 1.68921916624 },
	{ "(C)C(=O)NC1=NC=C(N(C1=O)CCOC2CC3=CC=CC=C3C2)C4=CC=C(C=C4)F", 2.27022746455 },
	{ "LCMXNYSAHLOUAW-UHFFFAOYSA-N", 0.125723821035 },
	{ "SZLFCDJMZCUDEQ-UHFFFAOYSA-N", 1.81373178329 },
	{ "5-keto-22-dimethyl-hept-3", 2.17243323079 },
	{ "1COC2=C(O1)C=CC(=C2)CC3=CC=CC(=C3)C4C(C(C(C(O4)CO)O)O)O", 0.907207330005 },
	{ "1CNC(=O)C2C1CC(NC2NC3=CC=C(C=C3)OC4CCOCC4)C5=CN=C(C=C5)N", 2.29605288788 },
	{ "HHPGXGXAWBZXRN-UHFFFAOYSA-N", 1.74987955959 },
	{ "3-anilino-3-(4-bromophenyl)-1-(4-methoxyphenyl)-1-prop", 0.162112087437 },
	{ "38H62N6O7", 0.547363583687 },
	{ "MQGCIGBQIVPAF-UHFFFAOYSA-K", 0.647015121166 },
	{ "27H24FN2OS", 2.29744867524 },
	{ "MJQPANCVAAAFFT-UHFFFAOYSA-N", 0.810185325898 },
	{ "AMIIDLBCFDYWKW-UHFFFAOYSA-N", 2.32454995376 },
	{ "15H14NNa3O11S3", 0.280838577251 },
	{ "13H13BrFN3OS", 2.17378590909 },
	{ "KYWCIZKXFZIUEZ-UHFFFAOYSA-O", 0.868100393212 },
	{ "MKDVWFNMXELQI-UHFFFAOYSA-N", 1.22885047488 },
	{ "KSBFUGYTANXHA-UHFFFAOYSA-N", 1.43970556744 },
	{ "20H20BN2O4", 0.612175762643 },
	{ "EEVTWSHMIUEEJ-UHFFFAOYSA-N", 0.664581551412 },
	{ "1CC1N(CCC(=O)O)C(=O)C=CC2=CN=CC=C2", 2.00835650349 },
	{ "OFQPRRZFFYULAD-UHFFFAOYSA-N", 0.946694028772 },
	{ "ZWAYAVGOHCKOBH-UHFFFAOYSA-N", 1.21019745459 },
	{ "12H16N4O2S2", 0.583318537423 },
	{ "LOWWCWZKNDINRB-UHFFFAOYSA-N", 1.39479682976 },
	{ "1CC2C3CC(=O)C4=CC(=O)C=CC4(C3C(CC2(C1(C(=O)CBr)O)C)O", 1.0444638006 },
	{ "BHHUAKWHCLAQMJ-UHFFFAOYSA-N", 1.3782914008 },
	{ "56-dihydroxy-2-octenoic acid", 1.19662741693 },
	{ "HSDLHZRFPOJFIZ-UHFFFAOYSA-N", 1.74879791483 },
	{ "13H15N2O2Y", 1.34238376127 },
	{ "MZIAMTGWBHVQOR-UHFFFAOYSA-N", 0.44819931095 },
	{ "N1C(=O)C(=CNC2=CC=CC(=C2)C(=O)NCCN3CCOCC3)SC1=C(C#N)C(=O)O", 1.1182788165 },
	{ "N&apos;-oxidanyl-2-(pyridin-2-ylmethylamino)ethanimidamid", 2.29657139519 },
	{ "3-[5-bromo-2-[2-(3-methylbutoxy)ethoxy]phenyl]prop-2-enoic acid", 1.34399441597 },
	{ "BJBWCGBYLTVVFI-UHFFFAOYSA-N", 1.54423368573 },
	{ "3-(3-nitrophenyl)acrylic acid pent-2-enyl ester", 1.88761506844 },
	{ "OCC1CCN(CC1)C(=O)C2C=C3C(=NC(=O)C4=C3N(N=C4)C5CCOC5)C=C2", 1.85624957245 },
	{ "OC(=O)C=CC1=C(C(=C2C(=C1)C=CO2)OC)OCC3=CC=C(C=C3)Br", 0.98363277269 },
	{ "1=CN=C[NH+]1C2=C(C=CCC2(C)F)F.[Cl-]", 2.17634627536 },
	{ "NVSCLIRESMAESE-UHFFFAOYSA-N", 0.714050591821 },
	{ "OZKYKDFQRHTBMM-UHFFFAOYSA-N", 0.108598381916 },
	{ "1=CSC(=NC(=NC#N)SC)N1", 0.155498750929 },
	{ "NOSMBJFRAKKGAE-UHFFFAOYSA-N", 0.454939667831 },
	{ "27H32FN3O", 1.51695006934 },
	{ "3-[amino-(hydroxyamino)methylene]-67-dihydro-5H-1-pyrindin-2", 2.29077225654 },
	{ "45H60N3O4+", 2.13887016639 },
	{ "34H44ClN2O6+", 1.49364418813 },
	{ "22H31ClN4O3", 1.16633208236 },
	{ "=CCN1C(CCC1=O)CC(=O", 1.55980967169 },
	{ "9H11F5O", 0.545141365688 },
	{ "VHJQEITPPVLFA-UHFFFAOYSA-N", 2.14791081151 },
	{ "1=CC2=C(C=C1)N(C3C2CCCCC3)CCC4=CC=NC=C4", 0.376335918324 },
	{ "22H27F2N5S", 2.2611397209 },
	{ "N-[1-[5-(hydroxycarbamoyl)-2-pyridyl]ethyl]-4-phenyl-benzamid", 0.879182242713 },
	{ "iridium;5-methyl-2-[3-(3-methylphenyl)benzene-6-id-1-yl]pyridi", 0.606053166419 },
	{ "(C(=O)NCC)(O)OC(=O)C=", 2.09887119808 },
	{ "31H29N7O10S2", 2.21664630144 },
	{ "VYIYBDZXLACNED-UHFFFAOYSA-N", 0.981087921636 },
	{ "XDQGBHZKRJXZTQ-UHFFFAOYSA-N", 0.21834230247 },
	{ "N-styrylsulfonylchroman-3-carboxamid", 1.93176184943 },
	{ "KQMNEWHVFYDIB-UHFFFAOYSA-N", 2.33639430245 },
	{ "1C2C3(C=CC4C3C(O2)OC=C4C(=O)OC)OC1=O", 0.606488898327 },
	{ "1=CC2=C(C=C1Br)C(=NC=C2F", 0.239054359113 },
	{ "(CO)NC1=NC(=NC2=NC(=O)C(=NCC3=CC=CO3)N=C21)SCC4=C(C(=CC=C4)F)F", 0.355588574825 },
	{ "LHQBKGZPXDLVAY-UHFFFAOYSA-N", 2.25905265184 },
	{ "QTVSLRFVMGXJNA-UHFFFAOYSA-N", 0.292967641097 },
	{ "QRJBXZWEGNNSCF-UHFFFAOYSA-N", 2.34991386088 },
	{ "[4-(2-amino-3-butoxy-3-oxo-propyl)-2-hydroxy-phenyl] 4-aminob", 1.89257354537 },
	{ "(=NN(C1=CC=C(C=C1)N(C)C)C(=O)OC(C)(C)C)C2=CC=CC=N2", 1.68998974978 },
	{ "1CCN(C(C1)CO)CC2=CC=CC(=C2)C=CC(=O)O", 2.05341379145 },
	{ "BHQTXXDIAKWUGN-UHFFFAOYSA-N", 2.22358792375 },
	{ "UHOHSMWMKGYGFZ-UHFFFAOYSA-N", 1.00945945556 },
	{ "UITLBOGGKFVKLX-UHFFFAOYSA-N", 1.24560794341 },
	{ "2-amino-33-dimethyl-N-(2H-tetrazol-5-ylmethyl)butyramid", 2.37010505984 },
	{ "38H35N3O6", 0.515024842696 },
	{ "QWCNACMJFADLY-UHFFFAOYSA-N", 1.8974979882 },
	{ "1=CC(=C(C=C1[Sb])[N+](=O)[O-])C(=O)O.O.O.[OH-]", 0.534786639108 },
	{ "VBRICDQCKOWTFP-UHFFFAOYSA-N", 1.39484115109 },
	{ "18H12N6", 1.34804551416 },
	{ "([CH2-])CC.[Y]", 0.974045112969 },
	{ "6H32N8O8Rh4", 1.88941688428 },
	{ "OC1=CC=CC(=C1)NC(=O)OCC2CC(CN2)NS(=O)(=O)C3=C(C=CC(=C3)Br)Br", 1.11369678674 },
	{ "UXLMLWBMVAZOHM-UHFFFAOYSA-O", 0.654485342239 },
	{ "HBISCANDZRLTD-UHFFFAOYSA-N", 1.35188652465 },
	{ "(C)C1=NOC(=N1)N2CCC(CC2)N3CCC(C3=O)OC4=C(C=C(C=C4)S(=O)(=O)C)F", 1.90687277513 },
	{ "(C)CC1CN(CCN1C2=NC=C(S2)C3=CC(=CC=C3)F)CC4=CC(=CC=C4)OCC(=O)O", 0.331214602169 },
	{ "N1C(CC(=O)N(C1=O)C)CN(CC2CCCO2)CC3=CC=CO3", 0.479001314867 },
	{ "1=CC(=CC(=C1)N(C2=CC=C(C=C2)C=CC3=CC=C(C=C3)Cl)C4=CC(=CC(=C4", 1.68115588949 },
	{ "25H26ClNO2", 2.05054941181 },
	{ "OFXMNBLGRRLVSW-UHFFFAOYSA-N", 2.07773658236 },
	{ "24H44O3Si", 0.969629997881 },
	{ "13H9IMgN2", 0.415258016257 },
	{ "(C)C1=CC(=C2NC(=C3C=CC(=O)C=C3)N=N2)C(=O)C=C1O", 0.798233940068 },
	{ "N1C(=NN=C1OC2=CC=C(C=C2)F)C(C)NS(=O)(=O)C3=C(C=CC=C3F)F", 0.83256932924 },
	{ "(C)(C)OC1=CC=CC=C1C2(C(=O)O2)NC(CCC(=O)ON3C(=CC=C3O)O)C(=O)O", 1.526513044 },
	{ "1(NC2=C(C(=O)N1)N=C[N+]2(C3CC(C(O3)CO)O)C4C(C(C(O4)CO)O)O)N", 1.76250776671 },
	{ "1=C(C(=CC=C1)C)OCC(C)NC(=O)C(CSC)N", 2.24420158934 },
	{ "26H20ClN3O2S", 0.147116529399 },
	{ "1=CC(=CC(=C1)Cl)C(CCC(=O)O)C(C2=CC=C(C=C2)Cl)[NH3+]", 1.93982834887 },
	{ "N-(2-phenylpropyl)-4-pyrazin-2-yloxy-piperidine-1-carboxamid", 1.6847862938 },
	{ "VVRZWBNTTOKVAB-UHFFFAOYSA-N", 2.03142316389 },
	{ "10H13ClO3S", 0.252897771034 },
	{ "N&apos;-hydroxy-2-(2-isopropoxyethoxy)propionamidi", 1.70306537677 },
	{ "VYQARFLWKBPXKK-UHFFFAOYSA-N", 2.14083568173 },
	{ "henolate;rutherfordium", 0.439389417045 },
	{ "YMHCGCBPYVAPTN-UHFFFAOYSA-N", 1.74464683473 },
	{ "YIFZYSSBLYXDIK-UHFFFAOYSA-N", 1.98244983475 },
	{ "OC(=O)C(=CC1=C2CCCCN2C=N1)C3=CC(=NC=C3", 0.423652189183 },
	{ "29H29ClN4O2", 1.01135772908 },
	{ "FOXCFDFJJLUIZ-UHFFFAOYSA-N", 1.89361205001 },
	{ "1C(CNN1)CNC(=O)CSCC2=CC=CC(=C2", 2.36643754067 },
	{ "KJIRBVIYDVEMAU-UHFFFAOYSA-N", 1.08177752392 },
	{ "MYWHZDBRBWHHGX-UHFFFAOYSA-N", 1.21907418023 },
	{ "JRFDGQBNOZDSAH-UHFFFAOYSA-N", 0.95312773942 },
	{ "2-(34-dichlorophenoxy)-N&apos;-hydroxy-6-methyl-nicotinamidi", 1.18152808355 },
	{ "1=CC(=CC=C1)CNCC2CNNC2C3=CC=CC=C3", 1.40466277831 },
	{ "HHSHLFIALMTCFN-UHFFFAOYSA-N", 0.463151713142 },
	{ "VNZAOGOQDDHHFK-UHFFFAOYSA-N", 1.63345697244 },
	{ "NYTSUAXGHOLI-UHFFFAOYSA-N", 0.382017542777 },
	{ "1=CC(=C(C=C1)NCC2C=[N+](C(=O)N(C2=O)C)C)O", 2.06063348507 },
	{ "1(C2CCC(=C)C(C2(C(CC1=O)O)C)CCC3=CCOC3=O)CO", 0.199329211189 },
	{ "S-[4-(difluoromethyl)phenyl] 2-(2-aminopropanoylamino)ethanethi", 0.712585223294 },
	{ "#CCOC1=C(C=CC(=C1)C(C)N)N2C=CC=N2", 1.58353925322 },
	{ "1CN(C(CN1C2=CC=C(C=C2)C(=O)C)C)CC(=O)N3CCN(CC3)C4CCC4", 1.99667400014 },
	{ "H6PV", 1.38066685643 },
	{ "1CN(CCC1C(=O)NCCCO)C(=O)C=CC(=O)O", 1.88982265013 },
	{ "2-(3-azanyl-2-oxidanylidene-butyl)thiophene-3-carboximidamid", 2.37903400893 },
	{ "WRJLQQMBHOTUPV-UHFFFAOYSA-N", 2.18014836928 },
	{ "2-amino-3-[22-bis(1-pyrrolyl)ethylthio]propanoic acid", 1.7739090062 },
	{ "16H30O5PSW", 2.31492861164 },
	{ "N-(2-amino-2-hydroximino-ethyl)-2-iodo-N-propyl-benzamid", 1.4901852441 },
	{ "WMNDDNSBXFXWQY-UHFFFAOYSA-N", 2.29810030901 },
	{ "JLYIDFHIWCGILH-UHFFFAOYSA-N", 0.491491817069 },
	{ "=CCN1C(=O)C(=CC2=CC=C(C=C2)N3C=NC=N3)SC1=S", 2.34250797019 },
	{ "3-bromanyl-5-(2-isocyanoethenyl)pyridi", 0.53665385545 },
	{ "hI=1S/C9H18O3/c1-6(2)11-5-9-8(10)4-7(3)12-9/h6-10H4-5H21-3H3", 2.10149583866 },
	{ "6-[(3-methoxyphenyl)methyl]-8-oxidanyl-1H-quinazolin-4", 0.291400669018 },
	{ "18H14F3NO3", 1.22659280097 },
	{ "QZHCOMZIOHZIEZ-UHFFFAOYSA-N", 1.89554671203 },
	{ "GUKKIXBAXGJEIT-UHFFFAOYSA-N", 2.13018264537 },
	{ "=CCN1CCC23C4C(CCC2(C1CC5=C3C(=C(C=C5)O)O4)O)NC(=O)C6=CC=CC=C6", 1.06779317961 },
	{ "33H53FN2O11", 0.509789615938 },
	{ "13-(34-dibenzoxy-1-benzyl-pyrrolidin-2-yl)tridecan-1", 2.02760467338 },
	{ "2-tetralin-1-yl-1-[2-(thiazolidine-3-carbonyl)pyrrolidino]eth", 2.1025380019 },
	{ "2-aminoethoxy-keto-prolyloxy-phosphonium", 0.546815780705 },
	{ "MJFVLNGEVGRINE-UHFFFAOYSA-N", 0.199063874577 },
	{ "BHBNDRVPCWGKPB-UHFFFAOYSA-N", 1.07857042624 },
	{ "NHDPLQVIYTKOY-UHFFFAOYSA-N", 1.13462282 },
	{ "14H11BrClFN2", 1.88748959475 },
	{ "SPJJTYOLSALPCM-UHFFFAOYSA-N", 1.17733575792 },
	{ "N-(3-phenylbut-2-enyl)-N-prop-2-enyl-propanamid", 0.376528572186 },
	{ "MHFYXKAQZFNSK-MHSSMOFYSA-N", 0.954266697791 },
	{ "(36-diacetyloxy-4-azido-5-phenylmethoxy-oxan-2-yl)methyl benz", 1.70356926598 },
	{ "1-(3-fluorophenyl)-N-(2-morpholinoethyl)triazol-4-ami", 0.730390308581 },
	{ "N-[6-(3-bromophenoxy)-3-pyridyl]-2-keto-6-phenyl-nipecotamid", 2.04741898241 },
	{ "(2-bromanylspiro[2.2]pentan-5-yl)meth", 1.14501455553 },
	{ "2-(3-fluorophenoxy)-N&apos;-hydroxy-propionamidi", 0.786819469116 },
	{ "38H36N6O8", 2.11415460898 },
	{ "2-(3-methyl-124-triazol-4-yl)acetic acid", 1.36264415518 },
	{ "3H2N4O5", 0.478928513689 },
	{ "4-(5-oxo-124-oxadiazolidin-3-yl)benzenesulfinic acid", 2.09674000366 },
	{ "benzyl N-(1-amino-3-hydroxy-1-oxo-2-phenylpropan-2-yl)carbam", 0.502312026551 },
	{ "JDOUDJZRJXNITH-UHFFFAOYSA-N", 0.910207871822 },
	{ "18H32ClN5", 1.34703801486 },
	{ "methyl 4-[3-(3-methoxypropoxy)-3-oxoprop-1-enyl]benz", 1.40577931947 },
	{ "OOGCZQVTGAVCMZ-UHFFFAOYSA-N", 0.131510116108 },
	{ "(2-acetamido-3-sulfanyl-propanoyl) 2-amino-3-sulfanyl-prop", 1.65333955479 },
	{ "N-(4-oxidanylidenequinazolin-6-ylidene)pentanamid", 0.434895474829 },
	{ "27H31ClFN3O5S2", 1.78359353421 },
	{ "1=C(C=CC(=C1)CC(C(=O)O)NC(=O)OC(C)(C)C)O", 2.01247087045 },
	{ "KLMOSJVWUTYBS-UHFFFAOYSA-N", 1.58007197901 },
	{ "YXQOKLLRULBILU-UHFFFAOYSA-N", 2.39017596499 },
	{ "1C(CC(C2C1C(=O)C(CN2)C(=O)NCC3=CC=C(C=C3)Cl)C4=NC=CS4)F", 0.226038750558 },
	{ "S(=O)(=O)NCC(=O)NCC1C2CCC(C1CC=CCCC(CO)O)O2", 1.97564874914 },
	{ "RSXHOZQOKBNZFN-UHFFFAOYSA-N", 0.373831221472 },
	{ "12H16F3N3S", 0.986268680415 },
	{ "1=CC=CC=C1C(C)NCC2CC(=O)N(C(=O)N2", 2.28497186705 },
	{ "1C(ON=C1CNC(=O)NCC2=CC=CS2)COC3=CC=CC=C3C4=CC=CC=C4", 0.804855727934 },
	{ "1C=CC2=C(C(=CN=C21)C(=O)CC3=CC=CC=C3O)NC4C5CC6CC4CC(C6)(C5)O", 2.29902242137 },
	{ "4-chloro-2-[(4-chlorophenoxy)methyl]-3-hex-2-enyl-1-cyclop", 1.10885368586 },
	{ "methyl 2-acetamido-3-(3-iodophenyl)-2-methylprop", 1.44934701901 },
	{ "GDXOCXZSBUZLJW-UHFFFAOYSA-N", 1.86902823756 },
	{ "JKVWZBFLXKAILA-UHFFFAOYSA-N", 1.22274580272 },
	{ "1=CN=CC=C1C2=CC3=C([C-]=C2)C4=NC=C(N4C=C3)C5=CC=NC=C5.[Ir]", 0.598231862511 },
	{ "UQHEKIZQJJDGV-UHFFFAOYSA-N", 1.14319051171 },
	{ "OTOWQSJJBMKQJD-UHFFFAOYSA-N", 2.10783027088 },
	{ "34H36F3N5O5S", 0.88996290648 },
	{ "1=C(C=CC2=C1CCN(CC2)C(=O)OC(C)(C)C)C(=NO)N", 0.387087153513 },
	{ "WIDZVSYVACHEH-UHFFFAOYSA-N", 0.832590592842 },
	{ "VEWWMAATYPOZFP-UHFFFAOYSA-N", 0.437760011866 },
	{ "KORBNIAFPUYWIT-UHFFFAOYSA-N", 0.319679940214 },
	{ "1CN(CC2=CC=CC=C21)C[C@H](CNC(=O)CCC3=NC4=C(S3)C=CC=N4)O", 0.790578232558 },
	{ "32H43N5O5S2", 1.02717835121 },
	{ "LCTHAPBKPPBBKE-UHFFFAOYSA-N", 1.45517832009 },
	{ "3-[ethyl(2-oxanylmethyl)amino]-N&apos;-hydroxypropanimidamid", 0.495483616882 },
	{ "4-(hydroxymethyl)-7-azabicyclo[2.2.1]heptane-235-tri", 2.30385224673 },
	{ "21H23N7O3S", 1.53480735324 },
	{ "XEMQEEXUBFDPNF-UHFFFAOYSA-N", 1.70726419105 },
	{ "KFGXMRHEDKZLTD-UHFFFAOYSA-N", 1.92403938293 },
	{ "hI=1S/C3H4N2S2.K/c1-4-5-3(6)7-2;/h2H3(H56);/q;+1/p-1", 2.03434289791 },
	{ "1=CC(=NN1CC2=C(C=CS2)C=CC(=O)O", 0.119207918504 },
	{ "QNEBVTNFMVWPBR-UHFFFAOYSA-N", 1.21097834253 },
	{ "21H25F6N5O", 0.984790172272 },
	{ "XRCWWHMFNKFDHY-UHFFFAOYSA-N", 1.91829647469 },
	{ "2-cyano-3-(3-fluoro-4-methoxy-phenyl)acrylic acid", 1.77688557941 },
	{ "iron;2-methoxy-6-methyl-5-oxidanyl-hept-4-en-3", 1.90408160787 },
	{ "hyl 3-[4-(oxan-2-yloxy)phenyl]hept-4", 1.27280261058 },
	{ "2-ethynyl-6-propan-2-yloxy-oxane-345-tri", 0.480600710708 },
	{ "1[C@H]2[C@@H](C(=O)O1)NC(=O)NC2=O", 1.31494297148 },
	{ "ENCWWSHNZOKAH-UHFFFAOYSA-N", 0.511822167814 },
	{ "18H21N2NaO4S", 2.35489896859 },
	{ "6-chloro-3-(4-chlorophenyl)-2-methyl-quinazolin-4", 1.26503631091 },
	{ "(C)CCOC1=CC=C(C=C1)C=C(C#N)C2=CC=CC=N2", 0.16881154047 },
	{ "(C)(C)C(C(=O)NC1=CC=C(C=C1)N2CCCC2)N", 1.49855338834 },
	{ "BLEKQQKZENUKDT-UHFFFAOYSA-N", 2.02993934575 },
	{ "1=CC(=C(C2=C1C(=O)OCCC=CCCC=CC(=NOCC(=O)NC)C2", 1.73231189544 },
	{ "(C)CC(C(=O)OCCl)N", 1.16053716684 },
	{ "(C)CNC(=S)CCC(=O)C(CC1=CN=CN1)NC(=O)C(CCCCN)NC(=O)C2CCCC2", 1.66047439823 },
	{ "WDCJVQIVEOAX-UHFFFAOYSA-N", 1.69057086776 },
	{ "(CC1=NC(=O)C(S1)(C)C2=CC(=NC=C2)C#N)C3=CC=C(C=C3)F", 1.45681622945 },
	{ "MEQLBEKLMJOZTG-UHFFFAOYSA-N", 0.116863573307 },
	{ "RPDDPJBIKIKYNB-UHFFFAOYSA-N", 0.17771646538 },
	{ "GWQQQOKPNKOEGC-UHFFFAOYSA-N", 1.72471659152 },
	{ "FJKIQMFJLYLDSI-UHFFFAOYSA-N", 1.75509486499 },
	{ "27H28N10O", 1.74332587867 },
	{ "XTEUSSZNBVYQEB-UHFFFAOYSA-N", 0.366787735979 },
	{ "N(CCC1CCC(CC1)CC(=O)C2=CN(C3=CC=CC=C32)C)C4CCC5=C(C4)SC(=N5)N", 1.00226357491 },
	{ "1=NSC(=C1C=O)NN=C2C3=CC=CC=C3C=C(C2=O)C(=O)C4=CC=CC=C4", 1.26010837934 },
	{ "GPYYHZUZHIBADD-UHFFFAOYSA-N", 1.5159996087 },
	{ "30H60N2O3SSi2", 0.84498092706 },
	{ "29H41FN2O4", 1.86896436958 },
	{ "2-(hydroxymethylidene)-4-oxopentanoic acid methyl ester", 2.22639134632 },
	{ "4-(2-benzylpyrrolidin-1-yl)sulfonyl-3-methyl-pyrazolidi", 1.90077771579 },
	{ "3-(2-furyl)-N-(3-phenoxyphenyl)prop-2-enamid", 0.594165795055 },
	{ "ZWPQSKAOOUYXFQ-UHFFFAOYSA-M", 1.18273908186 },
	{ "(C(=O)NC(CC(=O)O)[P+](C(=N)C)(O)O)NC(=O)CN", 0.214271001251 },
	{ "29H23Cl2F4N3O7", 2.06748026053 },
	{ "JDCGFMGGEHSUFR-UHFFFAOYSA-N", 0.368568062379 },
	{ "1CC(CN(C1)C2CCC3=CC=CC=C3C2OC4=CC(=CC=C4)Cl)N", 1.09250884357 },
	{ "ZFTXCSVYYNNXQX-UHFFFAOYSA-N", 0.907896473286 },
	{ "(222-trifluoroacetyl) 2-amino-3-benzylsulfanylprop", 2.03119591374 },
	{ "1-(1-phenylpropyl)-3-(2-thiazol-4-yl-3H-benzimidazol-5-yl)ur", 1.96583283815 },
	{ "YCPJUOGMPMFHPW-UHFFFAOYSA-N", 0.659971397921 },
	{ "2-(ethylthio)-6-(4-methoxyphenyl)hexahydropyrimidin-4", 0.20490377978 },
	{ "BIJDAPZICKWHQB-UHFFFAOYSA-N", 0.549449369954 },
	{ "methyl 1-benzyl-5-carbamoyl-pyrrolidine-2-carboxy", 0.57306565398 },
	{ "1CCC(CC1)CCC2CCC(CC2)C[Ga].[Ra]", 0.990328307771 },
	{ "AZHNUSICHJBJKD-UHFFFAOYSA-N", 1.57627342832 },
	{ "1CCN(CC1)CCCC(=O)C2C=CC3=NC(=O)NC3=C2", 2.1046224889 },
	{ "NFGFGNFVCWOGGC-UHFFFAOYSA-N", 0.287372870849 },
	{ "VSQQLMZNLWJRJF-UHFFFAOYSA-N", 2.37596224767 },
	{ "WBMKCKVCMNDZTA-UHFFFAOYSA-N", 1.84237296782 },
	{ "SHSZREWRMOJNHY-UHFFFAOYSA-N", 1.936204657 },
	{ "VPHKIAZDBXJIDD-UHFFFAOYSA-N", 1.42564201699 },
	{ "2-oxopentadeca-46-dien-8-yl ac", 2.18303484717 },
	{ "OVZASMXOLIKD-UHFFFAOYSA-N", 1.88978837206 },
	{ "VPSJGIKRPDGVSE-UHFFFAOYSA-N", 1.72606773151 },
	{ "26H34F3N5O4", 1.03128954445 },
	{ "21H23F2N5O5", 1.16542233574 },
	{ "1=C(C(=CC=C1)CCC(C(C)OCC2=CC=CC=C2)O", 2.20949804356 },
	{ "14-dimethyl-5-phenyl-2-imidazolidi", 0.892293096298 },
	{ "(C)SC(=C([S-])SC(C)C)[S-].CC(C)SC(=C([S-])SC(C)C)[S-].[Ni]", 1.71259277313 },
	{ "BXIHTAWDHDHFHG-UHFFFAOYSA-N", 0.484282308745 },
	{ "27H33N5O10", 0.791137276483 },
	{ "4-phenoxy-N-pyrimidin-5-yl-benzamid", 1.60824422734 },
	{ "3-keto-2-morpholin-3-ylidene-3-(p-tolyl)propionitri", 0.816883715843 },
	{ "hepta-135-tri", 0.421313549255 },
	{ "OC(=O)C(C)NP(=O)(COCCN1C=NC2=C1N=C(N=C2OC3CC3)N)NC(C)C(=O)O", 2.19636804595 },
	{ "2-(dimethylamino)ethyl 3-oxidanylprop-1-enyl phosph", 2.34013473288 },
	{ "31H42ClN7", 2.24142959823 },
	{ "hI=1S/C6H10N4O/c1-5(6(7)9-11)10-4-2-3-8-10/h2-511H1H3(H279", 0.432731012367 },
	{ "31H36N2O12", 1.73929153813 },
	{ "FPEGUYMMPJDRMB-UHFFFAOYSA-N", 0.224829666137 },
	{ "FLANOEWBJJJWDS-UHFFFAOYSA-N", 0.828809565849 },
	{ "OYUXOODEDUJFFZ-UHFFFAOYSA-N", 2.30876862628 },
	{ "1=CC(=CC(=C1CC(=C(C2=CC=CC=C2)S)S)C)C(C)(", 0.679679949044 },
	{ "1CNNC1OCN2C(=O)C3=CC=CC=C3S2", 1.6797028208 },
	{ "4H8O3W", 1.51269677396 },
	{ "1(CCN(C(=O)O1)C(C)C(C)(C)C)C2=CC=C(C=C2)F", 2.12848057663 },
	{ "RUYMHRZWFQAFPF-UHFFFAOYSA-N", 1.36102571272 },
	{ "JXEJXLXLMZKJAK-UHFFFAOYSA-N", 1.03383599555 },
	{ "OEWXJIJJPIYDBO-UHFFFAOYSA-N", 1.25451511956 },
	{ "33H28O7", 0.533059618542 },
	{ "SRMMYAJIAQHNTQ-UHFFFAOYSA-N", 2.07400402167 },
	{ "UKSDPUHYLJPBGT-UHFFFAOYSA-N", 1.55321025213 },
	{ "UKFBQTQXGJXMOF-UHFFFAOYSA-N", 2.25782113985 },
	{ "QPHKDNMYAMCP-UHFFFAOYSA-N", 0.938650850544 },
	{ "(=O)OC1CN(CC(C1OCC2=CC=CC=C2)OCC3=CC=CC=C3)C(=O)OC(C)(", 1.10200085953 },
	{ "hI=1S/C10H10BS/c1-7-6-8-4-3-5-9(11-2)10(8)12-7/h3-6H1-2H3", 0.13874420807 },
	{ "1=NC(=C(C(=N1)Cl)NC(=O)C2=CC=CC=C2Cl)NC(C)CO", 0.190039783258 },
	{ "[(6-keto-5-nitroso-cyclohexa-24-dien-1-ylidene)methylamino]ur", 1.23319797149 },
	{ "27H47O5S", 1.96370860023 },
	{ "hI=1S/C7H9N3O/c1-10(2)6-9-4-7(3-8)5-11/h4-6H1-2H3", 0.852855343577 },
	{ "JXADZONNFJEQQK-UHFFFAOYSA-N", 0.440531506613 },
	{ "JKBCEZNLOGBUSF-UHFFFAOYSA-N", 1.1229806261 },
	{ "H4FO2UV", 1.62242250504 },
	{ "[4-(2-chlorophenyl)-5-(4-pyridyl)pyrazolidin-3-yl]ami", 2.37647561044 },
	{ "5-bromo-2-[4-(methylamino)-1-piperidinyl]-3-pyridinecarbonitri", 1.58053032369 },
	{ "EBAJOZYOKATRV-UHFFFAOYSA-N", 1.71215411396 },
	{ "39H47FN4O7S", 1.83216255759 },
	{ "QMBBISKWAVZDNY-UHFFFAOYSA-N", 1.90222392155 },
	{ "1=NC(C(O1)C2=CC=C(C=C2)S(=O)(=O)C)CF", 1.11401241835 },
	{ "22H23N3O7", 0.790973370693 },
	{ "48H44N3O9+", 2.00765021694 },
	{ "(C1=CC=C(C=C1)NC(=O)C2=CC=C(C=C2)F)NC(=O)C=CC3=NC4=CC=CC=C4O3", 1.76646546841 },
	{ "1C(OC(C1OC(=O)CCC(=O)O)C(C)(C)C(C)(C)P(=C)(", 0.996500818658 },
	{ "1=C(OC(=NC1=O)C=C)O", 1.44225742504 },
	{ "12H8N2OY-2", 0.243953915259 },
	{ "38H49F4N5O5", 1.00572842388 },
	{ "39H60N8O14", 2.02423630342 },
	{ "5-azido-6-ethoxy-2-ethyl-4-methyl-3-ox", 0.87174067741 },
	{ "1CC(C(NC1)C2=CC=CC=C2)NCC3=CC4=C(C=C3)OCO4", 1.57698426898 },
	{ "21H26N5O9PS", 0.32691958961 },
	{ "ANRJQSUPWMIAMM-UHFFFAOYSA-N", 1.89496825718 },
	{ "2-fluoro-4-[5-(3-methoxyphenyl)-3-pyrazolidinyl]benzonitri", 0.321668766562 },
	{ "ZNJFPLCUGVYQRO-UHFFFAOYSA-N", 1.28737383079 },
	{ "MNZRLLHNDQTZLJ-UHFFFAOYSA-N", 1.96543084247 },
	{ "KPQGXQGMQYOHY-UHFFFAOYSA-N", 0.209586348275 },
	{ "1CCC(CC1)CNCC(=O)C2=CC3=C(C=C2)NCC3", 0.336352154987 },
	{ "OC(=O)C=C(C(=O)OCC)ON=C(C(C1CCC(C1)CO)N(C)C(=O)OCC2=CC=CC=C2)N", 2.05983563333 },
	{ "1=C(C(=CC=C1)C)NC(=C(C#N)C(=O)C2=C(C=C(C=C2", 1.48448317337 },
	{ "7-amino-4-(3-benzamidophenyl)-2-thieno[23-c]pyridinecarboxamid", 1.20241291731 },
	{ "disodium;2-bromanyl-5-(4-bromanylbenzene-5-id-1-yl)benz", 1.53024691606 },
	{ "ABLGRJMOFONPOP-UHFFFAOYSA-N", 1.09757388479 },
	{ "APCPKKFIOQOABD-UHFFFAOYSA-N", 1.98029030868 },
	{ "HLTVLJKPDYEQGM-UHFFFAOYSA-N", 1.52489005618 },
	{ "3-methyl-N-(4-nitrobenzyl)pyrazolidine-4-carboxamid", 0.829037034039 },
	{ "XKAPNZJMZNYJJL-UHFFFAOYSA-N", 1.25564113564 },
	{ "FAPAMLBAXJCIOB-UHFFFAOYSA-N", 0.203511962264 },
	{ "12H12N3OSW", 2.20478990426 },
	{ "SSUTXWKNEBIKRL-UHFFFAOYSA-N", 1.05587626013 },
	{ "NZWNWSFDDBNRJT-UHFFFAOYSA-N", 1.98659501422 },
	{ "3-cyclohexyl-2-pyrrolidinecarboxylic acid;hydrochlorid", 0.639775606723 },
	{ "33H48BN5O5", 1.98771402895 },
	{ "NPMIOTVTDKNKM-UHFFFAOYSA-N", 0.714975129493 },
	{ "YGLNYSBGLBZFEI-UHFFFAOYSA-N", 1.69521633704 },
	{ "KUNOEKQTGFAFOJ-UHFFFAOYSA-N", 2.18322837817 },
	{ "6-amino-2-(2-amino-3-fluorophenyl)sulfanyl-13-diazinan-4", 0.714341866239 },
	{ "LSANDVOQPROLCN-UHFFFAOYSA-N", 2.23017127995 },
	{ "1(OC2CC3C4CCC5=CC(=O)C=CC5(C4C(CC3(C2(O1)C(=O)CSC6=NC=CC=N6)C)O", 1.11417241945 },
	{ "222-trifluoro-1-naphthalen-2-yl-N-trimethylsilylethanimi", 2.119883 },
	{ "JHFPAYWUSPPGG-UHFFFAOYSA-N", 0.764946461443 },
	{ "13H21B2N3O3", 0.339899522166 },
	{ "SVVQCTVCENSCCC-UHFFFAOYSA-N", 1.66064958042 },
	{ "1-(4-hydroxy-2-methylol-pyrrolidino)-4-mercapto-butan-1", 1.05875615779 },
	{ "[C-]#[N+]C1CN(N(C1=S)COCC2=CC=[C-]O2)C3=CC4=C(C=C3)C(CC4)O.[Y]", 1.13762903634 },
	{ "1C(C(CN1CC2=CN=C3C2N=CNC3=O)O)CSCCC(C(=O)O)N", 2.28965565665 },
	{ "1=CC(=O)OC2=CC3=C(C=CO3)C(=C21)[O-].[Na+]", 2.39348143645 },
	{ "N-allyl-3-(m-tolyl)prop-2-enamid", 1.41248727113 },
	{ "16H14N2", 0.841812013626 },
	{ "OC1(C2N(C1=O)C(=C(CS2)COC(=O)N)C(=O)O)NC(=O)CSCC#", 2.36799409971 },
	{ "1C(C2=CC=CC=C2CC13NC(=O)C(O3)C4=CC=CC=C4)C5=CC=CC=C5", 2.15643834278 },
	{ "43H58N2O14S2", 0.780597490184 },
	{ "MCTYCZSJJBTTD-UHFFFAOYSA-N", 2.34807054075 },
	{ "hyl 5-hydroxy-246-trimethyloct-2", 1.31795723259 },
	{ "1-[1-(2H-isoindol-1-yl)ethylidene]isoind", 1.70165606205 },
	{ "QRISVXPOVNNWNM-UHFFFAOYSA-N", 0.430753001175 },
	{ "1C(C(C(C1NC2=C(C(=NC(=N2)N)Cl)C3=CC4=C(O3)N=CC=C4)O)O)CO", 2.1122950508 },
	{ "NGHRHNGMEBHNLX-UHFFFAOYSA-N", 0.640774942723 },
	{ "10H16FN5O2S", 0.430129979775 },
	{ "(C)CC(C(=O)OC)NC(=O)NC1=NC=C(C=C1)N2CCCC2=O", 0.669536586642 },
	{ "39H67NO4", 1.14409898777 },
	{ "1C(C(C2CNCC2C1(C3=CC=CC=C3)C4=CC=CC=C4)(C5=CC=CC=C5)O)O", 0.4136856615 },
	{ "QENGPYBYVCHJOF-UHFFFAOYSA-N", 1.31700018799 },
	{ "BVKZKFOJACJVPM-UHFFFAOYSA-N", 2.16907465236 },
	{ "UOWBSVYWCGCDJO-UHFFFAOYSA-N", 1.07280067848 },
	{ "XMLMVIQXILVFFG-UHFFFAOYSA-N", 0.376072030701 },
	{ "2345-tetrahydroxy-NN&apos;-bis(2-hydroxyethyl)adipamid", 1.54467413151 },
	{ "6-oxo-2-(sulfamoylamino)-14-dihydropuri", 1.29122924312 },
	{ "4-benzyl-1-[3-(2-naphthyl)but-2-enyl]piperidi", 1.16071669188 },
	{ "TWXKRHYEDOIGMT-UHFFFAOYSA-N", 0.594114081775 },
	{ "GFKTWWDZRHSIIT-UHFFFAOYSA-N", 2.05118968525 },
	{ "1CC1NC2=NC(=NC3=C(C=NN32)C=C4CC(=O)NC4=O)NC5=CC=C(C=C5)F", 0.28013889132 },
	{ "1=CSC(=C1)C(=O)CC2C(=C(CC3=CC(=C(C=C3)F)Br)N=O)NON2", 1.96785014409 },
	{ "BKXNFKCXGNKUAQ-INIZCTEOSA-N", 0.234374447697 },
	{ "1-(1-azanyl-2-methyl-butyl)cyclopentan-1", 2.19541183155 },
	{ "dicupric;dipicolinate;1H-124-triazol-5-ylamine;tetrahydr", 2.06411532959 },
	{ "methyl 2-azanylchloranuidyl-2-cyclopropyl-eth", 0.802610228513 },
	{ "ZXTALMHXOQSRCQ-UHFFFAOYSA-N", 0.276850169899 },
	{ "1C(C(=O)NC(=O)N1)S(=O)(=O)NC2=CC3=C(CNCC3)C=C2", 0.647687767508 },
	{ "[B](N=[CH-])N=[N+]=[N-].[Na+]", 0.833609336399 },
	{ "(3-methyloxetan-3-yl)methyl 3-(5-methyl-2-furyl)prop-2", 1.2561840751 },
	{ "(CC=CC1C(CC(=O)C1CCCCCCC(=O)O)SCCO)(C2CC2)O", 0.823467592768 },
	{ "ZDROPOTXRPQHNK-UHFFFAOYSA-N", 1.33369182103 },
	{ "25H28F3N7", 1.76098365901 },
	{ "34H36O6", 1.00091412953 },
	{ "HEJXCVVXWHTZJN-UHFFFAOYSA-N", 0.902717607824 },
	{ "OZNKATBHHGHPPA-UHFFFAOYSA-N", 1.05034513352 },
	{ "20H21Cl4N3O3S", 0.296824533172 },
	{ "1CCC[N+]1(C(=O)CC(C(=O)C2=CC(=CC=C2)F)SC(=O)C)C(=O)O", 0.183986711676 },
	{ "N-[4-(2-furanyl)butan-2-yl]-6-oxo-3H-pyridine-3-sulfonamid", 0.619895823346 },
	{ "1C(C(=O)N1S(=O)(=O)O)NC(=O)C(=NOCC(=O)O)C2=CSC(=N2)N", 0.201157691088 },
	{ "OC1CNCCC1CNC(=O)C2=CC(=C(C3=C2OCCCO3", 1.89254627172 },
	{ "dodeca-369-trienoic acid 2-methylpropyl ester", 1.72271538312 },
	{ "NC(=O)C(C(C(C(CO)O)O)O)O", 0.989385151036 },
	{ "=CC(CN(CC1=CC=CC=C1)CC2=CC=CC=C2)C(C(=O)O)OCC3=CC=CC=C3", 2.14597241249 },
	{ "6-oxo-N-(2-sulfamoylphenyl)piperidine-3-sulfonamid", 0.287558582279 },
	{ "YAMOGPQGMPZKW-UHFFFAOYSA-N", 0.147684511708 },
	{ "26H37N3O", 0.482126831699 },
	{ "KIRQNGAQNPSNDW-UHFFFAOYSA-M", 1.13482035923 },
	{ "BQBXMLBEEOX-UHFFFAOYSA-N", 0.197597512983 },
	{ "6H6NO4", 1.29702745967 },
	{ "N1C=C(C=N1)CN(C)C(=O)C2(CCCCC2)C(=NO)N", 1.01870488344 },
	{ "1C(=O)N(C2=C(O1)C=C(C(=C2)C(=O)N(C3CCCCC3)C(C)C)Cl)CCNC(=O)O", 1.35036453486 },
	{ "HXTKWSDBGAZCMB-UHFFFAOYSA-M", 0.134361094504 },
	{ "KTHFTWXWTDEVPA-UHFFFAOYSA-N", 0.458753239178 },
	{ "WMGMDNNPWBFKPW-UHFFFAOYSA-N", 1.78046983287 },
	{ "JGSATUBNQVNGGE-UHFFFAOYSA-N", 2.22060034773 },
	{ "BDXHZEIJYPGRNH-UHFFFAOYSA-N", 1.67800073912 },
	{ "40H38F3N2O5+", 0.522517000612 },
	{ "1=CC=C(C=C1)S(=O)(=O)NC(CC2=CC=CC=C2)C(=O)C(=O)C(C(C)C)N", 2.17806762322 },
	{ "N(CC(=NO)N)C(=O)C1=C(C=CC(=C1)Br)F", 0.524834198065 },
	{ "SMEMANFUOWQWRO-UHFFFAOYSA-N", 2.25296702939 },
	{ "RGREDZURWHFRG-UHFFFAOYSA-N", 1.50051863492 },
	{ "zinc;3-[15-(p-tolyl)porphyrin-2122-diid-5-yl]pentane-15-di", 0.300029468938 },
	{ "UCLPXCWSSLBURC-UHFFFAOYSA-N", 1.50071473719 },
	{ "(C)(C)OC(=O)N1CC2CC1CN2C3=NC=C(C=C3)C(=O)O", 2.10024806368 },
	{ "12H11NS", 0.857890473334 },
	{ "ZDIMUSQWLSOAJX-UHFFFAOYSA-N", 0.828336267868 },
	{ "RZKIESUXWCNUFL-UHFFFAOYSA-N", 1.20376894277 },
	{ "hI=1S/C8H14O4/c1-8(23)12-7(11)5-4-6(9)10/h49-10H5H21-3H3", 1.11503952977 },
	{ "21H14F2N3O2", 1.93665094186 },
	{ "23H19ClN4OS2", 0.696659570506 },
	{ "GCVRFBLZGHPUQP-UHFFFAOYSA-N", 1.63981844399 },
	{ "(C)CC(C(CC=C)C(=O)N)C(=O)NC1C(=O)N(C2=CC=CC=C2C(=N1)N3CCCCCC3", 1.79548174648 },
	{ "36H56O6Si", 0.541206005317 },
	{ "N-[1-(phenylmethyl)piperidin-3-yl]pyrazolidine-4-carboxamid", 1.93846490606 },
	{ "(C)(CNC(=O)CCCC1=NC(=O)C2C=CC=CC2=N1)CN(", 2.27094288428 },
	{ "2-azanyl-9-cyclopentyl-8aH-pyrido[45]pyrrolo[12-d]pyrimidin-8", 1.94444773562 },
	{ "MXTCHIRECSCIHV-UHFFFAOYSA-N", 2.0003257487 },
	{ "1CCN(CC1)C2=CC3=C(C=C2)NN=C3C(=O)CC4CN5CCC4CC5", 0.798103357226 },
	{ "2-azanyl-N-ethyl-N-(2-methylpropyl)-4-methylsulfanyl-butanamid", 0.816837957854 },
	{ "29H36N8O3", 0.639481543023 },
	{ "OC1CC(N(C1)C(=CNC(=O)OC(C)(C)C)O)C(=O)NC2(CC2C=C)C(=O)O", 1.35794386052 },
	{ "1=CC=C(C=C1)OCNN=COC2=CC=CC=C2", 1.44759510974 },
	{ "18H24FN3O5", 1.08543665166 },
	{ "JSNPWEFDKIUWHL-UHFFFAOYSA-N", 1.73918466354 },
	{ "[CH3-].[CH3-].[CH3-].[CH3-].C[CH-]C.[V].[Y].[W]", 1.64828509502 },
	{ "29H48O3Si", 1.37222833832 },
	{ "BETZOSGRVRTDA-UHFFFAOYSA-N", 2.17724904981 },
	{ "WPXBVZWRMUROAB-UHFFFAOYSA-O", 1.65474142129 },
	{ "31H44F2N4O3", 2.28737201295 },
	{ "1C(C1C(=O)NC2=C(OC3=C(C=C(C=C23)F)F)C(=O)N)C4=CC=CC=C4", 1.26265507037 },
	{ "WESXVMWDEWAQLQ-UHFFFAOYSA-N", 0.153424093595 },
	{ "1-(3-amino-4-methanimidoyl-phenyl)-2-quinuclidin-3-yl-eth", 1.60347333472 },
	{ "ZCGGODWXRZXHT-UHFFFAOYSA-N", 0.811611893326 },
	{ "[3-(2-methoxyethenyl)azetidin-1-yl]-methyl-borinic acid", 2.09000227719 },
	{ "VGZBLFZGXKKJLC-UHFFFAOYSA-N", 0.350212564159 },
	{ "ZPABLENUOHJZIQ-UHFFFAOYSA-N", 2.15283918169 },
	{ "34-dimethoxy-1-methylpyrrolidi", 0.975711568226 },
	{ "40H49N9O5S", 2.2041769901 },
	{ "(CCC(C)C1CCC2C1(CCC3C2CCC4C3(CCC(C4)C(C(=O)O)Br)C)C)C(", 2.12515801591 },
	{ "37-bis(5-methyl-2-propan-2-yl-cyclohexyl)octa-26-di", 0.52739469779 },
	{ "[4-phenyl-2-(trifluoromethyl)oxazolidin-2-yl]meth", 1.26335544784 },
	{ "1-(4-chlorophenyl)-444-tris(fluoranyl)but-2-en-1", 2.14913414943 },
	{ "4-oxidanyl-6-(4-phenylmethoxyphenyl)-N-(phenylmethyl)hexanamid", 1.43742890027 },
	{ "JODQPEAVWZLPNX-UHFFFAOYSA-N", 0.535619995499 },
	{ "ZJEWORLEKUGWGU-UHFFFAOYSA-N", 0.765159285961 },
	{ "27H20ClF3N4O4S2", 2.14034898291 },
	{ "LKSQZWITRCNTMO-UHFFFAOYSA-N", 0.717687319738 },
	{ "HZYHDHQOFXCABB-UHFFFAOYSA-N", 0.450436621225 },
	{ "ZBJNLGZEGUMWRM-UHFFFAOYSA-N", 0.982861032493 },
	{ "OFINFNSZJKBCIQ-UHFFFAOYSA-N", 0.582677593791 },
	{ "1=C(C=CC(=C1)CCCC(=O)O)C#N", 0.527755566452 },
	{ "XOGKBFHEGIKPAO-UHFFFAOYSA-N", 1.59792316747 },
	{ "ZRKITPMGHUBYMB-UHFFFAOYSA-N", 1.50918239049 },
	{ "1CC2=C(C1)SC3=NC(=NC(=O)C23)CN4CCCN(CC4)C5=CC=CC=C5C#N", 1.57357066327 },
	{ "WQWXUBQHLDATMJ-UHFFFAOYSA-N", 2.18101324857 },
	{ "1=CC2=C(C=C1)C(=NNC3=C(C=C(C=C3C)C)C)C(=O)C=C2", 1.02646668336 },
	{ "1-benzyl-N-(2-nitrovinyl)piperidin-4-ami", 1.70238625049 },
	{ "GCYFXVYJEFBOON-UHFFFAOYSA-N", 0.336552887484 },
	{ "2-[2-mercaptoethyl(propan-2-yl)amino]eth", 1.08178818488 },
	{ "(C=CCC=CCCCCC)O", 2.12253042445 },
	{ "7-methyl-6-(4-piperidyloxy)-8aH-isoquinolin-1", 2.00858007605 },
	{ "(=C)C1CC2=C(O1)C=CC(=C2O)NC(=O)C34CC3COC5=CC(=C(C=C45)OC)O", 0.965179773265 },
	{ "5-[1-(245-triphenylimidazol-1-yl)heptan-3-yl]tetrazolidi", 0.254688404625 },
	{ "OZPQJVXJPYKKKT-UHFFFAOYSA-N", 1.5643571948 },
	{ "1=C(C2=C3C(=NC(=O)C=C3C(F)(F)F)C=CC2N1CCCCCCCCCS(=O)(=O)CCCCO", 0.752371892839 },
	{ "NHPIQRDZTKRHCT-UHFFFAOYSA-N", 0.734342842268 },
	{ "22H21NO5", 2.36700402868 },
	{ "BTAIYIPZGQYIAX-UHFFFAOYSA-N", 1.90770708354 },
	{ "VUAGAYWPNYOSPA-UHFFFAOYSA-N", 0.956862227663 },
	{ "2-methyl-4-(3-oxidanyl-1-adamantyl)but-2", 2.04039345936 },
	{ "N1C(=O)C2=NC(=NC2=[N+](C1=O)C)NCC3COCCO3", 1.30955362797 },
	{ "AJDSHWOFABBQGA-UHFFFAOYSA-N", 1.99691973553 },
	{ "(2R6R8aR)-26-dimethylindolizidi", 1.92728538317 },
	{ "1=C2C(=CC=C1)N(C(=O)N2CCN(C)C)C(=O)NC(C(C)C)C(=O)N", 1.26230777282 },
	{ "KHXQTTKKELXMPV-UHFFFAOYSA-N", 0.658680509196 },
	{ "ORFNYLFWJPBNEW-UHFFFAOYSA-N", 0.485509145044 },
	{ "XNTVSZWHJWVYMO-UHFFFAOYSA-N", 0.777007423698 },
	{ "LLVRHQDZJWNBRV-UHFFFAOYSA-N", 1.05672311582 },
	{ "2-(5-chloro-13-benzoxazol-2-yl)-1-piperidin-2-yleth", 1.3135997665 },
	{ "ZXESWABBEBKZKT-UHFFFAOYSA-N", 1.44698292451 },
	{ "FGMSZHLXKKMQIV-UHFFFAOYSA-N", 1.96840793135 },
	{ "1=CC=C(C=C1)CN(C2=CC=CC=C2)C(=O)C(=C(C3=CC=C(C=C3)Cl)O)[N+]#N", 0.300376287114 },
	{ "2-methoxy-14-bis[2-(4-methoxyphenyl)ethenyl]benz", 2.08006070397 },
	{ "31H45BN6O7S", 1.61056739487 },
	{ "ZKUQGHAQSNAZJE-UHFFFAOYSA-N", 1.75789446876 },
	{ "2-amino-N-(3-fluoro-4-methyl-phenyl)-4-(methylthio)butyramid", 0.577542193972 },
	{ "31H32ClN5OS", 0.354624051501 },
	{ "YFDNXCNVZUELPB-UHFFFAOYSA-N", 2.21025729789 },
	{ "3-[1-(4-methoxyphenyl)-2-nitroethyl]-3-methyl-1-benzofuran-2", 0.383335492718 },
	{ "YNDQIDZLCYONQR-UHFFFAOYSA-N", 0.45133801969 },
	{ "1-(3-benzyl-23-dihydro-1H-indol-2-yl)-22-dimethylpropan-1", 1.94555016113 },
	{ "ARQJLWIUQTZEID-UHFFFAOYSA-O", 1.87666879933 },
	{ "(C(=O)NCC1=CC(=CC=C1)N(C)C)NS(=O)(=O)C2=CC=CC=C2", 1.84549527428 },
	{ "HEDYFVTZJCYABI-UHFFFAOYSA-N", 0.164608055368 },
	{ "SYOFRADKLPPRAG-UHFFFAOYSA-N", 1.38284059525 },
	{ "N(CC)CC1=CC=C(S1)C=CC(=O)O", 0.463320740106 },
	{ "(CCN=C(CC=CC)N)C(C(=O)O)N", 2.20238361497 },
	{ "3-[3-[(N-methylanilino)methyl]phenyl]-2-propenoic acid", 0.892477768295 },
	{ "1C2CC3CC1CC(C2)C3NC(=O)C4C=NN5C4=NC(CC5C(F)(F)F)C6=CC=C(C=C6", 2.09381322619 },
	{ "propan-2-yl N-[3-(N&apos;-oxidanylcarbamimidoyl)phenyl]carbam", 1.49012889406 },
	{ "23H35F2N7O3", 0.563120623793 },
	{ "LETNEFYJNUJFFB-UHFFFAOYSA-N", 0.502478931356 },
	{ "WLILZSGALPSLT-UHFFFAOYSA-N", 0.12634398068 },
	{ "ZWHUYYXYEALXQO-UHFFFAOYSA-N", 1.29326676692 },
	{ "NC(=O)C1=CC2=C(C=C1)C(OCC2)CCN3CCN(CC3)C4=CC=C(C=C4)O", 1.27399852336 },
	{ "JAIHNEWYHJDREE-UHFFFAOYSA-N", 2.27069809174 },
	{ "2-[5-(4-fluorophenyl)pyrazolidin-3-yl]-5-methylanili", 0.972251876439 },
	{ "4-ethyl-N&apos;-[3-(5-methyl-2-furyl)acryloyl]benzohydrazid", 1.72984404297 },
	{ "30H40N2O2P2", 1.79749006736 },
	{ "KOMGOMNNFSONFL-UHFFFAOYSA-N", 1.17275592257 },
	{ "1C=C(C2C1C(=COC2OC3C(C(C(C(O3)CO)O)O)O)C(=O)O)COC=O", 2.38973698182 },
	{ "6-(4-isopropylphenyl)-2-oxo-3H-pyridine-3-carboxylic acid", 0.99744155565 },
	{ "S(=O)(=O)OC1CCN(C1)C(=O)C2=CC(=CC(=C2)[N+](=O)[O-])[N+](=O)[O-]", 1.1473426419 },
	{ "1=CC(=CC(=C1OC2=CC(=C(C=C2CO)N)NC3=CC=C(C=C3)C#N)C)C=CC#N", 2.34357244152 },
	{ "JDZOFADZSCJHDX-UHFFFAOYSA-N", 2.36650000514 },
	{ "3-[4-(2-isopropoxyethoxy)-3-methoxy-phenyl]prop-2-enoic acid", 1.96693452681 },
	{ "(C)(C(=O)O)NS(=O)(=O)C1C=CC2=NC(=O)NC2=C1", 1.284395022 },
	{ "3-methyl-4-oxidanyl-5-(1-oxidanylprop-2-enyl)oxolan-2", 0.258136278367 },
	{ "LEHAHFPKKVHUFE-UHFFFAOYSA-N", 1.51126977076 },
	{ "hI=1S/C6H4N4O/c11-6-5-4(7-3-8-6)1-2-9-10-5/h1-35H", 1.48867834824 },
	{ "KDVVGVIULMIHIH-UHFFFAOYSA-N", 1.43085029138 },
	{ "37H51FN6O4", 2.13348730101 },
	{ "KVJVPWVDPDIC-PDTRTMRVSA-N", 1.8141837509 },
	{ "S(=O)(=O)C1=CC=C(C=C1)OC2C(CC3=C(C=C(C=C23)Cl)Cl)N4CCCN(CC4)CCF", 0.263183800119 },
	{ "1=CN(C(=O)N=C1N)C2C(C(C(O2)OC(=O)P(O)O)N=[N+]=[N-])O", 2.32240996649 },
	{ "40H36N3O4", 0.986694859636 },
	{ "44H35N3O5", 0.913917748198 },
	{ "BPOGQOHRGWTAGT-UHFFFAOYSA-N", 1.84465110636 },
	{ "OC(=O)C(CC1=CC=CC=C1)NC2=NC=NC(=C2)C3=CC=C(C=C3)C=CC4=CC=CC=C4", 1.34609816007 },
	{ "1C(=O)CC2=CN=C(N=C2N1C(", 0.524924934658 },
	{ "SUTNSWPGGKMMMN-UHFFFAOYSA-N", 2.20688638212 },
	{ "(E)-1-(4-benzylpiperidin-1-yl)but-2-en-1", 2.10624147461 },
	{ "OEQWFZOVDXOGCI-UHFFFAOYSA-N", 2.13688777461 },
	{ "BTXGTFYDZBFPAN-UHFFFAOYSA-N", 1.45297392127 },
	{ "49H58N10O7", 1.80436377424 },
	{ "=CCCCCCCCCCC(=O)OCC(COC(=O)CCCCCCCC=CCCCC)OC(=O)CCCCCCCC=CCC=", 1.0755599639 },
	{ "14H21BN2O4S", 1.38210364275 },
	{ "2-amino-45-bis[(4-methoxyphenyl)methoxy]benzoic acid", 1.237934089 },
	{ "OJOBGEJUABHIIB-UHFFFAOYSA-N", 1.92488505484 },
	{ "hyl 3-[4-(oxan-2-yloxy)phenyl]-4-tributylstannylpent-4", 1.87405014263 },
	{ "26H42GdN4O11+3", 0.400947277653 },
	{ "9H13O4", 1.22715571715 },
	{ "N-[1-(2-chlorophenyl)ethyl]-23-dihydro-1H-indole-2-carboxamid", 0.340811621249 },
	{ "HUVQUEMULWRZDN-UHFFFAOYSA-N", 1.78786291362 },
	{ "VMNJWHHMPUFRPQ-UHFFFAOYSA-N", 2.05757414422 },
	{ "39H36N6O4+2", 2.30576269567 },
	{ "1=CC(=C(C2=C1C(=O)OCCC=CCCC=CC(=NOCC(=O)N3CCSCC3)C2", 0.131661282958 },
	{ "N-(2-acetamidoethyl)-3-(p-tolyl)acrylamid", 2.39780701101 },
	{ "KYUTZOBKXJQLV-UHFFFAOYSA-N", 0.388536977047 },
	{ "WMUIHIQCXNJJLB-UHFFFAOYSA-N", 1.57203764107 },
	{ "KKQFMYBNYLDIC-UHFFFAOYSA-N", 2.33489333143 },
	{ "12CCC3C(C1CCC2=O)CC(C4=CC(=O)C=CC34C)C(F)(F)F", 1.85758565767 },
	{ "WTRGQDNYAGWGIP-UHFFFAOYSA-N", 0.280542455289 },
	{ "14H26O8", 2.31269791193 },
	{ "EOHLMIGYXYYJB-UHFFFAOYSA-N", 2.18838993881 },
	{ "N-methyl-N-[2-(4-methylanilino)-2-oxo-1-phenyl-ethyl]benzamid", 1.34582669738 },
	{ "1-[2-(35-dimethyl-1-piperidyl)phenyl]pentan-1-ami", 1.39004051584 },
	{ "(3-but-2-enoxy-4-methoxy-benzyl)-cyclopropyl-ami", 0.990247508275 },
	{ "43H47ClN6O10S", 0.61432419372 },
	{ "OC1=C(C=C(C=C1OC)C=CC(=O)C2=CC(=C(C=C2)Cl)[N+](=O)[O-])O", 1.56012739517 },
	{ "FDWRYVUIVVAB-UHFFFAOYSA-N", 0.910855186549 },
	{ "chloro 2-methyl-2-[4-(4-octoxyphenyl)phenyl]p", 2.39545460005 },
	{ "8H28O4SiTiZr", 0.388624279222 },
	{ "13H16FN", 1.87632909314 },
	{ "OC(=O)C1C2=CC=CC=C2CN1C(=O)OCC3=CC=CC=C3", 2.15870891642 },
	{ "OC(=O)C12C=CC(CO1)C3C2OC(O3)(", 0.984887428063 },
	{ "(=O)OCC1C(C(N(O1)C)C2=CC=CC=C2)C(F)(F)F", 0.613194205381 },
	{ "RCOGTKAXQNZTLS-UHFFFAOYSA-N", 0.925123345044 },
	{ "BRLQEXMCGPODRD-UHFFFAOYSA-N", 0.532423887161 },
	{ "WBDNDZVRFAPBIK-UHFFFAOYSA-N", 1.13172498891 },
	{ "YNAYMRNQFHXHDX-UHFFFAOYSA-N", 2.2052539752 },
	{ "YHAFARAIMOVXTR-UHFFFAOYSA-N", 0.454661048748 },
	{ "33H40N4O6S2", 2.28392659065 },
	{ "5-(2-chloranyl-4-methyl-phenyl)sulfinylthiophene-2-sulfonamid", 1.11541922228 },
	{ "(C)(C)OC(=O)N1C=NC2=C1C=CC(=C2)CN", 1.39595491098 },
	{ "8H6N4O3S", 1.24396549999 },
	{ "24H32FN3O6S", 2.08300906015 },
	{ "GPUACGMOTNYTMN-UHFFFAOYSA-N", 0.631219512791 },
	{ "(C)(C)OC(=O)N1CCN2C(=CC=C2C=C[N+](=O)[O-])C1", 1.74628276146 },
	{ "[(4-propan-2-yloxyphenyl)methylideneamino]ur", 0.649597346907 },
	{ "N-(2-amino-2-hydroximino-ethyl)-N-methyl-3-phenoxy-propionamid", 1.58058884114 },
	{ "55H92O7VW-2", 1.31167953259 },
	{ "19H21F2NO10P2", 1.25951310315 },
	{ "HKGVBOCAQYOWTK-UHFFFAOYSA-N", 0.528644574089 },
	{ "(=O)CN1C2=C(C=C(C=C2)OC)C(C1=CC=NNC(=O)C3=C[N+](=CC=C3)[O-])(", 0.785458246729 },
	{ "(Z)-eicos-13-en-10-one;(Z)-tricos-7", 0.648515069996 },
	{ "1-(35-dichlorophenyl)-5-[3-(2-hydroxyethoxy)benzyl]-2-pyrrolid", 1.85319697219 },
	{ "UIYJIMXTABMKCA-UHFFFAOYSA-N", 0.648674536796 },
	{ "ZDOVCWOGGWZNIG-UHFFFAOYSA-N", 1.12992035455 },
	{ "JWIIVFHJFPGVLA-UHFFFAOYSA-N", 1.85844486056 },
	{ "QMRQISPRKLJAPF-UHFFFAOYSA-N", 0.511094689516 },
	{ "N(CC)C(=O)C(C)C1CCC2C1(CCC3C2CCC4C3(CCC(=O)C4=N", 1.41169367207 },
	{ "25H35N3O11", 0.705236222771 },
	{ "YSNLPNAFAZKFNE-UHFFFAOYSA-N", 2.31119816335 },
	{ "3-[2-[(25-dichlorobenzoyl)amino]thiazol-5-yl]prop-2-enoic acid", 1.7296557729 },
	{ "1=NOC(=C1)C2CCCN2C(=O)C3=C(N=C4CCNN4C3C5=CC(=C(C=C5)Cl)Cl)CO", 0.753889281422 },
	{ "MVCUDZKVIUUZKN-UHFFFAOYSA-N", 1.27310823857 },
	{ "1-(aminocarbohydroximoyl)-NN-diethyl-cycloheptanecarboxamid", 0.498576957207 },
	{ "RRPCBVFBAMCNJJ-UHFFFAOYSA-N", 0.108491466056 },
	{ "23H32FN3O5S2Si", 0.830712445219 },
	{ "pyridin-3-ylmethyl N-(2-oxidanylcyclohexyl)carbam", 2.18078892718 },
	{ "FCVYGZBRQCITIH-UHFFFAOYSA-N", 1.0873463607 },
	{ "BGTUHONOJMJSE-UHFFFAOYSA-N", 0.90627179103 },
	{ "OC(=O)C(=CC(C(C)C)N(C)C(=O)C(C(C)(C)C)NC(=O)C1CCCCN1C2CC2", 0.488582414117 },
	{ "(=C(C1=C(C=CC(=C1)Cl)Cl)Cl)C#N", 1.76626345289 },
	{ "2-[[(4-bromobenzyl)amino]methyl]-4aH-quinazolin-4", 1.36424323214 },
	{ "JVZAMADWGEJQY-UHFFFAOYSA-N", 1.12871569681 },
	{ "ZBXLBKGUZRHJJF-UHFFFAOYSA-K", 0.162139707624 },
	{ "22H16F3N5O4", 1.12167623988 },
	{ "(=O)C1C(C(C(C(O1)OCC)O)O)O", 1.48358478191 },
	{ "2-(2-oxo-5-phenyl-13-oxazolidin-4-yl)propanoic acid", 1.05162514256 },
	{ "(C)(C)C1(CCN2CCCCC2C1C3=CC=CC=C3Cl)O", 2.10487686669 },
	{ "1=CC=C(C=C1)C(CC(=O)OC(C)(C)C)C(C2=CC=CC=C2)F", 1.11243227976 },
	{ "1(C2CCC1(C(C2)OC(=O)C34CCCN3C=C(C=C4)C(=O)O", 1.1950720704 },
	{ "3-(4-cyanophenyl)-N-methyl-prop-2-enamid", 2.35184334217 },
	{ "N-[5-(3-nitrophenyl)pyrazolidin-3-yl]hexanamid", 1.98777248774 },
	{ "34H49N4O6+", 0.855819079819 },
	{ "7-ethyl-68-dihydro-5H-imidazo[15-a]pyrazine-1-carboxylic acid", 2.30460944958 },
	{ "rt-butyl N-[1-(allylsulfanylmethyl)-2-amino-2-oxo-ethyl]carbam", 0.416072211799 },
	{ "JRYMIUUGCMGFQZ-YWIOZPJLSA-N", 2.26738310837 },
	{ "12-dibutoxyvinylbor", 0.972184512313 },
	{ "2-(22-dimethylpropylidene)-3-methylidenethiophene 11-dioxid", 0.263200592618 },
	{ "AIEGNLXEIVDNNZ-UHFFFAOYSA-N", 1.42873624794 },
	{ "ALQNGVBULCXEKI-UHFFFAOYSA-N", 0.48506600488 },
	{ "SXSCAQNZYCLTHT-UHFFFAOYSA-L", 2.25307236589 },
	{ "N1C(=O)CCN(C1=O)CC(=O)NCCCCN2CCN(CC2)CC3=CC=CC=C3", 0.525249811605 },
	{ "methyl 4-chloro-1-(3-pyridylmethyl)pyrrolidine-2-carboxy", 0.284600390593 },
	{ "methyl 2-amino-3-(4-fluorophenyl)-2-methylprop", 0.88520844768 },
	{ "UZDHHNVAACGFA-UHFFFAOYSA-N", 1.15933690893 },
	{ "VUWONDGOQSGGLQ-UHFFFAOYSA-N", 1.25702607693 },
	{ "(C1=NN=C(N1C)C2=CC=CC=C2C(F)(F)F)OC3=CC=C(C=C3)F", 1.7224476688 },
	{ "6-[(5-ethyl-2-thiophenyl)-oxomethyl]-16-dihydrobenzimidazol-2", 0.347747693273 },
	{ "36H42Cl3N3OR", 1.18407350475 },
	{ "4-(4-cyanophenyl)-2-hydroxy-2-methyl-N-phenylbutanamid", 1.66270781192 },
	{ "phosphinophosphine;uranium;yttrium", 0.962237681701 },
	{ "4-(3-methoxypropanoyl)-1-(1-phenylethyl)pyrrolidin-2", 0.265919727301 },
	{ "6-hydroxy-3-methoxy-77-dimethyl-5-octen-4-one;ruthenium", 1.49795381524 },
	{ "KATNVRLLTXPDPT-UHFFFAOYSA-N", 1.49278122884 },
	{ "15H23ClO", 1.61116622601 },
	{ "26H30FN7O4", 0.325891886508 },
	{ "(C(C1CCCN1)OC)C(=O)NC(CC2=CC=CC=C2)C3NNNN3", 1.87088575566 },
	{ "5-(4-chlorophenyl)-4-methylpyrazolidine-3-carboxylic acid", 1.22109512951 },
	{ "JDKHHTUEXDIWFK-UHFFFAOYSA-N", 0.592131800857 },
	{ "N-[1-(3-bromophenyl)ethylidene]-2-methyl-propane-2-sulfinamid", 2.02902523213 },
	{ "175H207N17O11", 0.444471727 },
	{ "3-(24-dimethylphenyl)-NN-dimethyl-2-propenamid", 1.50867543014 },
	{ "18H26NO6+", 1.83465121591 },
	{ "YGWRXOLHQGBXPH-UHFFFAOYSA-N", 0.2908174238 },
	{ "methyl 4-(4-chlorophenyl)-2-methyl-4-oxo-3-phenylbut-2", 0.670101457012 },
	{ "20H34ClNO6", 0.494434543755 },
	{ "(C1=C(C=CC(=C1Cl)F)Cl)OC2=C3C(=CN=C2N)C(=CO3)C4=CC5=NC=CN5C=C4", 2.07175793034 },
	{ "OC(=O)N1C2CCC1CC(C2)NC3=NN=C(C4=CC=CC=C43", 1.11152060911 },
	{ "19H35O5", 2.11601720564 },
	{ "FMOPSBKAWYAZLA-UHFFFAOYSA-N", 1.0350881334 },
	{ "methyl 5-(2-bromo-1-hydroxyethyl)-2-methylbenz", 1.82402945644 },
	{ "N1C=[C-]C2=CN=C3CC=CC3=C21.[Y]", 1.49108037726 },
	{ "3-methyldocosa-24610131619-heptaenoic acid ethyl ester", 0.921455485376 },
	{ "KSLIZHHUURHPSL-UHFFFAOYSA-N", 1.17356629659 },
	{ "5-(2-methoxyethoxymethyl)-2-oxazolidi", 0.555394266177 },
	{ "18H23N7S", 2.31882200608 },
	{ "26H28ClN5O3", 1.01874683754 },
	{ "QEMKIMVVEZDILM-UHFFFAOYSA-N", 2.38113479499 },
	{ "1-(aminocarbohydroximoyl)-N-benzyl-cyclohexanecarboxamid", 0.785441024246 },
	{ "88-dimethyl-5-(phenylmethyl)-6-nonene-1-sulfonic acid", 1.80998156433 },
	{ "GCUKBMVZYFJFBK-UHFFFAOYSA-N", 0.565746838966 },
	{ "1C(CNN1)CNC(=O)CCC2=C(C=C(C=C2", 1.03797095564 },
	{ "cyclohex-2-en-1-ylmethyl(methyl)azanide;yttrium", 0.155434544318 },
	{ "MNEQSPPBZPPHFE-UHFFFAOYSA-N", 0.573797353362 },
	{ "1-(3-methylpyrazolidin-4-yl)-N-(1-naphthylmethyl)methanami", 0.914592572142 },
	{ "XCUTASKNYJEOA-UHFFFAOYSA-N", 1.39312505091 },
	{ "16H15N5O4S4", 2.06102751299 },
	{ "UZTBODCWWHKBMV-UHFFFAOYSA-N", 1.71819594758 },
	{ "27H46N4O7S2", 0.567872372016 },
	{ "(2-acetyloxy-5-methylphenyl)methyl ac", 0.15054000392 },
	{ "2-(2-but-2-enoxyphenyl)ethylami", 1.1025399812 },
	{ "O/N=C/CSC1=C(C=CC=N1)O", 0.85679925625 },
	{ "(=CC1=CC2=C(C(=C1)Cl)OCO2)[N+](=O)[O-]", 0.529333242804 },
	{ "3-[4-(2-thiomorpholin-4-ylethoxy)phenyl]prop-2-enoic acid", 1.98843961373 },
	{ "QVPYGCSSRAMXMW-UHFFFAOYSA-N", 1.63124880565 },
	{ "6-[4-(methylamino)phenoxy]-124-triazinane-35-di", 1.41395490668 },
	{ "SNODPKNDTUIXGE-UHFFFAOYSA-N", 0.718281913178 },
	{ "1=C(C(=C(C2=C1COC2=O)OCC[Si](C)(C)C)CC=C(C)CC(C#N)P(=O)(C)C)O", 1.9540489033 },
	{ "FWEGURPQTXXDFO-UHFFFAOYSA-N", 1.68632229534 },
	{ "#N.C1CCCCCC1.[Mo](", 0.55706974585 },
	{ "24H22FN3O4S2", 1.32498904541 },
	{ "70H42O24", 0.233186493212 },
	{ "SLGYDFXBFOKQTQ-UHFFFAOYSA-N", 1.09130745859 },
	{ "1(OC2C=C(C(C2O1)N3C=NC4=C3C(=CN=C4N)F)F", 1.75289419051 },
	{ "N-(4-methoxyphenyl)-2-[2-(p-tolyl)vinylsulfonylamino]benzamid", 0.589129163166 },
	{ "MXVFZKYKDSSNBP-UHFFFAOYSA-N", 1.13806178842 },
	{ "HFXAFMCXJGQSMJ-UHFFFAOYSA-L", 2.36723342061 },
	{ "(C)(C)C(C(=O)O)NC(=O)OC(C)(C)C(F)(F)F", 0.566648097125 },
	{ "2-(1-hydroxyvinyloxymethyl)-2-methyl-butyric acid", 1.45886019761 },
	{ "37H43N5O5", 0.706367168655 },
	{ "N-benzyloxy-1-(4-dimethylboranylphenyl)ethanimi", 2.35690656177 },
	{ "NSOMCANOMUALRD-UHFFFAOYSA-N", 1.06664226783 },
	{ "(C)(C)OC(=O)N1CCC2=C1C(=C(C=C2)F)C=O", 0.127775363907 },
	{ "23H26N2O6S3", 1.02665538373 },
	{ "1CNC2=C1C=C(C=C2)C(=O)C3=CC=CC=C3F", 1.93817494442 },
	{ "GYKDXWAPYHHIFN-UHFFFAOYSA-N", 0.128289191967 },
	{ "(C)CC(C(=O)ONN)NC(=O)OC(C)(", 2.3816326288 },
	{ "XCTOVKLMGQYVBN-UHFFFAOYSA-N", 1.79134380435 },
	{ "(CC1=CC=CC=C1)C(CC2=CC=CC=C2)NCC(C)(CS(=O)(=O)NC3=CC=CC=C3)O", 0.769981545569 },
	{ "4-bromanyl-15-dimethyl-56-di(propan-2-yl)cyclohex", 1.15458291457 },
	{ "34H44ClN5O6S", 0.230418851601 },
	{ "N(CC1=CSC=N1)C2=C(N3C=CSC3=N2)C=CC(=O)O", 2.32700533279 },
	{ "(C)(C)C(C(=O)NC1=CC=C(C=C1)C(=O)NCC=C)N", 1.40653240207 },
	{ "1=NC=CC(=C1)C2C(NC(=O)N2)C3=CC=CC=C3", 0.944002079043 },
	{ "7H-cinnolin-7-ide-3-carboxylic acid;yttrium", 1.53303345312 },
	{ "ZQRIDZJQZNLHB-UHFFFAOYSA-N", 0.235847029379 },
	{ "BRICUQHPIPFZEP-UHFFFAOYSA-N", 1.15037906783 },
	{ "1-methyl-2-[3-(1-naphthyl)butyl]pyrr", 0.239456317277 },
	{ "GWCRNLKVWQIIQN-UHFFFAOYSA-N", 0.323089578235 },
	{ "hI=1S/C8H10FNO/c1-5(2)6-3-8(11)10-4-7(6)9/h3-57H1-2H3", 2.22833475543 },
	{ "23H21BrClN3O6", 2.17695607895 },
	{ "QZLPZQSZMUGFM-UHFFFAOYSA-N", 1.07205137343 },
	{ "3-chloranyl-1H-quinoline-24-di", 0.973599378328 },
	{ "N-[(345-trihydroxy-6-sulfanyloxan-2-yl)methyl]butanamid", 0.88219113837 },
	{ "hI=1S/C5H8O2/c1-4-2-5(6)3-7-4/h14-6H2-3H2", 0.72562233412 },
	{ "1C2C(N(CN(C2NN1)C3=CC=CC=C3)C4=CC=C(C=C4)C(F)(F)F)N", 2.05106782807 },
	{ "34H53N2O5+", 1.23718862627 },
	{ "QXLEJOEMOJVNH-UHFFFAOYSA-N", 2.24037940075 },
	{ "N-[(3-methylpyrazolidin-4-yl)methyl]-3-phenoxyanili", 0.204432935623 },
	{ "GWMOSCECQUHGO-UHFFFAOYSA-N", 1.88596587584 },
	{ "YTVGAKWVQPTYRQ-UHFFFAOYSA-N", 0.563172842366 },
	{ "1CN(C(=O)NC1=O)C2CC(C(O2)CI)O[Si](C)(C)C(C)(", 0.132882017849 },
	{ "26H42Br2O8", 1.03073157792 },
	{ "FIYUWBZUGUFKGK-UHFFFAOYSA-N", 2.19679929773 },
	{ "16H10F2N2O", 0.20242727276 },
	{ "(C)CN1C(=NC2=C1C3=CC=CC=C3N=C2N)CCC(=NO", 0.228287588889 },
	{ "(CC(=O)N1CC(=O)CCC(C1=O)CC2=C(C=CC(=C2)Cl)OC)C(=O)NC3=CC=NC=C3", 0.282905258105 },
	{ "KCDDXEUXZLVRCV-UHFFFAOYSA-N", 0.519218404179 },
	{ "RXUXDEDPIKDCEQ-UHFFFAOYSA-N", 1.53999905314 },
	{ "XTHNDMSZOXWXLT-UHFFFAOYSA-N", 1.7728384902 },
	{ "[3-methyl-2-[(4-oxocyclohexen-1-yl)methylene]butylidene]tung", 1.58123372878 },
	{ "OP(=O)(O)OCC1CC(C1COP(=O)(O)OCCCCCCCC)N2C=NC3C2=NC(=NC3=O)N", 0.139244156434 },
	{ "SPADDGYDNVWNI-UHFFFAOYSA-N", 1.44941637829 },
	{ "[P+](C(=O)C(CC1CCCCC1)N)(O)O", 0.575088514865 },
	{ "1=CC=C(C=C1)CC=CC2=CN3C=CC=CC3=N2", 1.83091490919 },
	{ "34H70N4O2", 1.75195686342 },
	{ "LEPDEVHDEBZFLB-UHFFFAOYSA-N", 1.72587584429 },
	{ "ZIKMOAHIXFBGQU-UHFFFAOYSA-N", 1.77976164859 },
	{ "2-ethylsulfanyl-[13]oxazolo[54-c]pyridi", 0.214795107746 },
	{ "(=CC1C(C1(C)C)C(=O)OC(C#N)C2=CC=CC=C2", 0.859670318052 },
	{ "VXRKXSQAFNFQHH-UHFFFAOYSA-N", 0.270443358614 },
	{ "10H11BrN2O5S", 1.56149724393 },
	{ "YYYYATNSNILQIA-UHFFFAOYSA-N", 1.30564737141 },
	{ "5-(22-dimethylpropoxy)-3-diazi", 0.907986545848 },
	{ "BYPLQOCLHIYUEH-UHFFFAOYSA-N", 0.151217314049 },
	{ "233a45677a-octahydro-1H-inden-1-yl(3-chloroallyl)ami", 1.79840461173 },
	{ "36H43ClF3N7O7", 2.14594004805 },
	{ "31H52N6O11", 2.16085582676 },
	{ "XNUJJWXBCYYBAM-UHFFFAOYSA-N", 0.817204567232 },
	{ "17H27N2O7S+", 1.3148970138 },
	{ "7H6NOY", 0.330352147784 },
	{ "BIVHILPLPUKDGY-UHFFFAOYSA-N", 0.330623265588 },
	{ "JTQINLUTHJKBJN-UHFFFAOYSA-N", 2.09990927337 },
	{ "BTLTVIWMNJXKTQ-UHFFFAOYSA-N", 1.92698789293 },
	{ "KFHOPBFNCWJHFL-UHFFFAOYSA-N", 2.00342485225 },
	{ "41H76N8O13", 1.72208257378 },
	{ "iridium;2-(3-phenyl-1-benzene-6-idyl)pyridine;2-phenylpyridi", 0.819290973981 },
	{ "LJECKNQPBOQJPW-UHFFFAOYSA-N", 2.16687062273 },
	{ "JNDIYXFJFPKVSV-UHFFFAOYSA-N", 2.17861323742 },
	{ "1CC(C(=O)NC1)C(C2=CC=CC(=C2)C#N)C(C3=CN=CC=C3)C4=CN=CC=C4", 1.05074818066 },
	{ "N-(1-ethyl-2-hydroxy-nonadec-12-enyl)palmitamid", 1.03253561519 },
	{ "WWAGQHFVOEKOO-UHFFFAOYSA-N", 0.938451450099 },
	{ "15H18N2O9", 0.426409707978 },
	{ "XAPWTDCSEMWTPP-UHFFFAOYSA-N", 1.21497040636 },
	{ "(C)(C)OC(=O)NC(CC1=CC=C(C=C1)C2=CC(=CO2)C=O)C(=O)O", 0.433009066263 },
	{ "(CN=[N+]=[N-])C(C(=O)O)O", 1.06091838548 },
	{ "hI=1S/C7H19O4Si2/c1-8-12(5)6-7-13(9-210-3)11-4/h6-7H21-5H3", 2.14616805777 },
	{ "(C)(CN)C(=O)O.[Re]", 1.82891283704 },
	{ "STWYFHYYHXKRP-UHFFFAOYSA-N", 2.23122747766 },
	{ "HNEJITBPRLKHLS-UHFFFAOYSA-N", 1.90938824328 },
	{ "1C(C(NN1)C2=CC=C(C=C2)OC(F)(F)F)N", 1.74649634561 },
	{ "1CC(=O)CC1CNC2=CC3=NC=NC(=O)C3C(=N2)NC4=CC=C(C=C4)N5CCOCC5", 0.579097133624 },
	{ "6-amino-1-[(1-ethylpiperidin-4-yl)methyl]-13-diazinane-24-di", 2.3649065478 },
	{ "ADGPZLMHWZYXAH-UHFFFAOYSA-N", 1.03087420631 },
	{ "N1C2=CC=CC=C2N=C1C3C4CC(CCC4NN3)OC5=CC6=C(C=CC=N6)C=C5", 0.738013943794 },
	{ "4H8O5SY-2", 1.29306125015 },
	{ "ZTQLWJBBEOQXTF-UHFFFAOYSA-N", 2.04516700768 },
	{ "(4-phenoxycyclopent-2-en-1-yl) ac", 0.632660512216 },
	{ "SBGMTBWQMVQBMW-UHFFFAOYSA-N", 1.48245695647 },
	{ "SFBWLDTXANNXFO-UHFFFAOYSA-N", 1.40334469015 },
	{ "OFGURUYNUWGWQQ-UHFFFAOYSA-N", 1.01725618607 },
	{ "1-hydroxy-3-[5-(4-nitroanilino)pentoxy]-9-xanth", 0.541534749772 },
	{ "1CC(C[N+]1(C(=O)C(C)CS)C(=O)O)(C2=CC=CC3=CC=CC=C32)O", 0.713438727145 },
	{ "1=CC(=C(C=C1)OC2C3CCC2CN(C3)C4=NN=C(S4)C(F)(F)F)ON=C(", 0.846258985 },
	{ "QMHXHRVWQNFLHE-UHFFFAOYSA-N", 1.89644009152 },
	{ "(CC)NC(=O)C1=CC2=C(N=NC2N=C1)C3=C(C4=CC=CC=C4N3)C5=NC=CC(=C5)CN", 1.4949394875 },
	{ "=NOC1=NC=C(C(=N1)NCC2=CC=CC=C2F)F", 2.06902892987 },
	{ "dimethyl-(2-prop-1-enylquinazolin-4-yl)ami", 0.455985485655 },
	{ "SHIWVQINCOZTTP-UHFFFAOYSA-N", 0.807044925527 },
	{ "rt-butyl N-[1-[(4-chlorophenyl)methyl]-3-hydroxy-propyl]carbam", 1.77728677129 },
	{ "S(=O)(=O)CCNC1=NC2=C(C=NN2C(=N1)NC3CC3)C=C4CC(=O)NC4=O", 1.40315918806 },
	{ "N1CC(CC2C1CC3=CNC4=CC=CC2=C34)C(=O)OCO", 1.65685404935 },
	{ "10H10N2Rb2", 0.135816471757 },
	{ "RMQCHJKRVCURSG-UHFFFAOYSA-N", 0.471141963611 },
	{ "1C(C(NN1)C2=CN=CC=C2)CNCCCN", 0.727750182227 },
	{ "(C1=CC=C(C=C1)N2C=CC=N2)NC(=O)C(C(C(=O)N3CC4CNCC4C3)O)O", 2.09061090242 },
	{ "28H28N2O11P2-2", 2.04640288199 },
	{ "N1C(=O)C2C(=NC1=O)N=C(N2CC3=CC=CC=C3C#N", 1.77954282666 },
	{ "=CCCCC=CC1CCCC1CC=CCCCC(=O)O", 1.51874107994 },
	{ "1=C(SC2=NC(=NC(=O)C12)CSC(C)C(=O)N3CCCC4=C3C=CC(=C4)Br", 1.56248637717 },
	{ "2-(aminocarbohydroximoyl)-N-(12-dimethylbutyl)valeramid", 0.54827864565 },
	{ "TFAMKVLXDSESN-UHFFFAOYSA-N", 0.147899545429 },
	{ "1=CC=C(C=C1)S(=O)(=O)N(CC2=CN=CC=C2)C(C(C)C)C(=O)N", 0.137840140468 },
	{ "UYAIQNWMRGEUIP-UHFFFAOYSA-N", 1.41557030135 },
	{ "ZRUNXOBPWHQIFC-UHFFFAOYSA-N", 1.72768546372 },
	{ "1-[4-(trifluoromethyl)phenyl]hex-5-en-1", 0.721755017223 },
	{ "4-keto-N-(3-methoxypropyl)-N-methyl-1H-phthalazine-1-carboxamid", 0.413668798108 },
	{ "N1C(C(C(=O)NC1=O)NCCN(C)C2CC2)N", 0.256369152936 },
	{ "1CC2C3CCC(C3(CC=C2C=C1C)C)O[Si](C)(C)C(C)(", 1.90179004232 },
	{ "LUPRCWWDVBIPSU-UHFFFAOYSA-N", 1.46454927871 },
	{ "HAMJFBXZRYGIW-UHFFFAOYSA-N", 0.743682456371 },
	{ "[P+](=O)OCC1CC(C(=C1C)F", 0.688918852197 },
	{ "7H11ClN4O3", 1.98022872888 },
	{ "1=NC(=O)N=C(C1C(=O)N2CCCC2CC(=O)O)S", 0.94023194556 },
	{ "hyl 4-[(2-hydroxyindan-1-yl)carbamoylamino]benz", 1.96325834482 },
	{ "N-benzoxy-3-(26-dichlorophenyl)acrylamid", 1.18225992813 },
	{ "QZHWCBJKDPQUEP-UHFFFAOYSA-N", 1.28643731558 },
	{ "1OC2=C(O1)C=C(C=C2)C=CC(=O)OCCCC3=CC=NC=C3", 1.09211542399 },
	{ "(C1=CC=CS1)N(C)C(=O)C2=CC=C(C=C2)C(=NO)N", 2.08850637923 },
	{ "19H17F3N4O3", 1.10816637676 },
	{ "N-(azetidin-3-yl)-N-phenyl-acetamid", 1.74723607827 },
	{ "3-amino-2-(heptanoylamino)propanoic acid", 0.135786256389 },
	{ "N&apos;-hydroxy-6-methyl-2-(4-methylpiperazino)nicotinamidi", 0.465928897149 },
	{ "28H34ClN6O4S", 1.32760024211 },
	{ "1-(7-ethyl-1H-indol-3-yl)pentylami", 0.659153274827 },
	{ "2-methyl-3456-tetrakis(phenylmethoxy)cyclohexan-1-ol;tung", 1.225612275 },
	{ "2-[3-(5-acetyl-1-cyclobutyl-3-oxo-2-piperazinyl)propyl]guanidi", 0.601690088386 },
	{ "WEEGFBGRJJCWEU-UHFFFAOYSA-N", 1.66943190823 },
	{ "JPEVRBWZFGOQMV-UHFFFAOYSA-N", 0.629591156244 },
	{ "TZYFECAMACULN-UHFFFAOYSA-N", 1.73595187427 },
	{ "OC(=O)C(=C1N(C(=O)CS1)C)C#N", 0.664339711331 },
	{ "48H47NO16", 1.7652400027 },
	{ "1=CC(=C(N=C1)Br)OC(=O)C=CC2=CC=C(C=C2)F", 1.49080392651 },
	{ "1112234-heptakis(fluoranyl)hex-3", 1.96784072776 },
	{ "(C)C(CCC1=CC=C(C=C1)C2=CC=CC=C2C#N)C(=O)O", 1.86805964038 },
	{ "cyclopentyl 2-methyl-3-(4-oxidanylcyclohexyl)prop", 1.51211214343 },
	{ "N(CCC=C1C2=CC=CC=C2COC3=CC=CC=C31)P(=O)(O)OC4=CC=CC=C4", 0.962063916032 },
	{ "YEJLDZKPWVSFLD-UHFFFAOYSA-N", 1.0922794307 },
	{ "27H36FNO4", 0.176020056042 },
	{ "NC1CC2C3CCCN4C3C(CCC4)CN2C(=S)C1.Br", 0.506650497217 },
	{ "(C1=CC=C(C=C1)N2C(COC2=O)COCC3=CC=C(S3)C(=O)O)F.[Y]", 0.89250981033 },
	{ "15H10BrN3", 1.52198038648 },
	{ "4-methyl-5-oxopentanoic acid;yttrium", 2.17471423018 },
	{ "2-[2-(3-methoxypropylamino)-2-oxoethyl]-4-hexadec", 1.5911276637 },
	{ "[5-(azetidin-2-ylmethoxy)isoxazol-3-yl]methyl N-phenylcarbam", 0.698421457657 },
	{ "FXPXUDGEKMTZMG-UHFFFAOYSA-N", 1.47363605581 },
	{ "RMOPVBZFTUMHCE-MKYTWGQBSA-N", 0.96037221566 },
	{ "47H46Cl2N4O8", 0.815993426248 },
	{ "17H16N2O2", 0.743828689079 },
	{ "rt-butyl N-(4-fluoro-2-methyl-3-oxo-butyl)carbam", 1.58125718835 },
	{ "LYIVXZRTMNCRGZ-UHFFFAOYSA-N", 2.25256900105 },
	{ "SYALFGBZHOVPPI-UHFFFAOYSA-N", 2.21507341129 },
	{ "OC1=CC=CC(=C1OCC=CC)CN", 2.31633849891 },
	{ "19H20N4O8S4", 2.27395568922 },
	{ "1CC(N(C1)C(=O)C=CC2=CSC=C2)C3=CC=CC4=CC=CC=C43", 0.126678886495 },
	{ "1CNCC2=NC(N(C21)C(=N)N)C3=CC=CC(=C4NC5=CC(C=CC5=N4)C(=N)N)C3=O", 1.74924552679 },
	{ "1=NC(=NC=C1OCC2(CC2C(=O)NC3=NC=C(C=C3)F)C4=CC(=CC=C4", 0.151556553772 },
	{ "10H16N3O8", 2.26989618539 },
	{ "OFDVCMKCMBWXSZ-UHFFFAOYSA-N", 1.63837450667 },
	{ "KLADLLOXIZFNIB-UHFFFAOYSA-N", 0.720241498885 },
	{ "4-(2-benzyloxy-4-methyl-benzene-5-id-1-yl)piperidine;yttrium", 1.55973819156 },
	{ "57H92O6", 1.82399575612 },
	{ "=CC(=O)NCC1=CNN=C1", 1.02646705474 },
	{ "WURVVMVCQWJO-UHFFFAOYSA-N", 2.27278982129 },
	{ "1=C(C=C(C=C1)C2=NC=C(C=N2)Br)C3=C(C4CCC(C4)C3=O)O", 1.80614339098 },
	{ "[CH3-].C[O-].[U].[U]", 1.51196686737 },
	{ "RCUCYMWKTHZKIX-UHFFFAOYSA-N", 1.20930654719 },
	{ "2-[but-3-enimidoyl(oxepan-3-yl)amino]-6-(methylamino)hexanamid", 0.961188164664 },
	{ "UKUJNRZDPILU-UHFFFAOYSA-N", 1.65327275015 },
	{ "ARQPRAMSSFRIV-UHFFFAOYSA-N", 1.87640442071 },
	{ "ACUMPBWVQLUYNO-UHFFFAOYSA-N", 1.15193771521 },
	{ "YUABVGBUXJWZFS-UHFFFAOYSA-N", 1.75205317542 },
	{ "35H23F8NO", 0.352422074583 },
	{ "MFCZOJUTWCVWDY-UHFFFAOYSA-N", 0.702876858497 },
	{ "XONLAWWQEBZSAD-UHFFFAOYSA-N", 1.74398371942 },
	{ "(C(C1=CC=CC=N1)SC2NC(CC(=O)N2)C)N", 0.255042078592 },
	{ "48H25N8O", 0.313103630958 },
	{ "XNUJYAURPNFBZ-UHFFFAOYSA-N", 1.11332020886 },
	{ "ADKJXGFMQHXTDF-UHFFFAOYSA-N", 0.566415193326 },
	{ "23H21N3O3", 0.794629769122 },
	{ "12H16N2O4S2", 1.56866161939 },
	{ "3-methylbutan-2-yl 11-methylheptadec-9", 0.334933186151 },
	{ "MFIYXHMKDHDFPY-UHFFFAOYSA-N", 1.06723723224 },
	{ "31H32BrNO7", 1.05797889923 },
	{ "2-amino-N-(5-methyl-2-pyridyl)-3-phenyl-propanamid", 0.54776511011 },
	{ "methyl 6-chloranyl-[13]thiazolo[45-c]pyridine-2-carboxy", 0.660598302233 },
	{ "N(C)C1=NC2C(C(C(OC2S1)CF)O)O", 1.98190227023 },
	{ "1C(C=C(C2C1O2)SOOC3=CC=CC=C3)O", 1.8198917439 },
	{ "JXNSQSSGMWYPST-UHFFFAOYSA-N", 0.478661720014 },
	{ "18H20N8O", 1.54564282489 },
	{ "3-(benzenesulfonyl)-N-[1-(tetrazolidin-5-yl)ethyl]prop-2-enamid", 0.192652629981 },
	{ "1CN(CCN1C2=CC(=CC=C2)OC)C[N+]3=CN=C4C3C(=O)N(C(=O)N4", 0.776740847621 },
	{ "TQGWPKHGWXXEQ-UHFFFAOYSA-N", 1.26194719903 },
	{ "32H29F2IN2O3", 1.89655170955 },
	{ "OWEWZSVSVZZALH-UHFFFAOYSA-N", 1.6873363674 },
	{ "UHUYYMMJEODWOV-UHFFFAOYSA-N", 0.985551867957 },
	{ "(E)-(4-methoxy-3-methyl-13-benzothiazol-2-ylidene)hydrazi", 0.450048062827 },
	{ "YQJXBVBYPPJJEZ-UHFFFAOYSA-N", 0.207068688685 },
	{ "5-chloro-3-hydroxyimino-7-methyl-1-(2-pyridinylmethyl)-2-ind", 1.51492261731 },
	{ "N(C1=CC=CC=C1)N=CC2=CC=CC=[N+]2CCCCCCN3C=C[N+](=C3)CCC[Si](C)(", 1.1770218834 },
	{ "MRWKQXZARYJCQB-UHFFFAOYSA-N", 2.00264836254 },
	{ "3-[N-(pyrazolidine-4-carbonyl)anilino]propanoic acid", 0.229973121446 },
	{ "BROXHLBTIRFZKB-UHFFFAOYSA-N", 0.166030702898 },
	{ "XOWCXHOIUMARHZ-UHFFFAOYSA-N", 1.09053363513 },
	{ "2-(6-methoxynaphthalen-2-yl)-7-nitrooxy-heptanoic acid", 0.598984804027 },
	{ "NOODXCLVMNPGAG-UHFFFAOYSA-N", 0.31082315185 },
	{ "MHAXFMIGJBIRLX-UHFFFAOYSA-N", 0.360097357158 },
	{ "(CC(=O)C=C(C)C1=CC=C(C=C1)C)C(=O)N2CC(CC2C(=O)NC(C)CCC(=O)N)O", 1.24614877946 },
	{ "QBLNIWDSGJPI-UHFFFAOYSA-N", 0.28148092586 },
	{ "21H44O2Si2", 2.29258741175 },
	{ "(CC(C)(C(F)(F)F)C(F)(F)F)C1CCCCC1", 1.06759195332 },
	{ "3-[5-[(2-methylindolin-1-yl)methyl]-3-thienyl]prop-2-enoic acid", 1.85132075208 },
	{ "UPMKFJAFROKNA-UHFFFAOYSA-N", 1.12832979265 },
	{ "1(CN(CCO1)CC2=C(C=C(C=C2)C=CC(=O)O)F", 1.82348102315 },
	{ "[CH3-].CC(=O)CCC[PH3+].[Y].[Y].[Y]", 0.675271269753 },
	{ "LXJFFPIPIRRXFZ-UHFFFAOYSA-N", 2.38001783589 },
	{ "36H54O7", 0.38671568333 },
	{ "2-[34-bis(fluoranyl)phenoxy]pyridi", 0.265761999063 },
	{ "16H17", 1.37826915851 },
	{ "6-(p-toluidino)-56-dihydrouraci", 0.750007078845 },
	{ "HEEVGTQIEAEUDJ-UHFFFAOYSA-N", 0.449398457417 },
	{ "XCVTERUECBMSX-UHFFFAOYSA-N", 0.895145388808 },
	{ "5H17O3SiS", 1.96556948089 },
	{ "ZDBWNFISCKKUQA-UHFFFAOYSA-N", 1.94417938836 },
	{ "hI=1S/C6H12.Cr/c1-3-5-6-4-2;/h1-6H2;/q-2;+2", 0.376031254574 },
	{ "(C)CC(COCC(=O)NO)C(=O)NC(C(=O)OC(C)C)C(C)(", 1.95200873238 },
	{ "1=CC=C(C=C1)COC(=O)C(CO)NC(=O)C(CC(=O)N)N", 2.09233058512 },
	{ "=CC(C)(C)CC(=O)[O-]", 1.33740475188 },
	{ "4-ethoxy-3-methylbenzaldehyde oxim", 2.37532549331 },
	{ "1CC(CC2C1NNC2)CC(C(=O)NCC(C)(C)C)NC(=O)OCC3=CC=CC=C3", 0.686018671992 },
	{ "1=CC=C(C=C1)N2C=C(C=N2)C=NNC(=S)N", 1.91216679852 },
	{ "VNZCVOHWSDWZBZ-UHFFFAOYSA-N", 0.183751410104 },
	{ "OC(=O)C(=O)C=C1CC(NC(N1)OCC2=CC=CC=C2)OCC3=CC=CC=C3", 1.69039320599 },
	{ "31H28N6", 1.58024193542 },
	{ "(C1=CC=CC(=C1CCC(=O)OCC)C)OCC(CN2CCCC2CC3=CC(=C(C=C3)C)Cl)O", 1.45196763695 },
	{ "1CC(=NC1)C(=O)C(C(CCCN=C(N)N)N)NNC(=O)N", 1.88853842757 },
	{ "1COC(C(C1OC)O)CSC2=CC=C(C=C2", 0.817081089082 },
	{ "1=CC=C2C(=C1)C3C(=CC=NC3=NC2=O)C4=CC=C(C=C4)F", 0.52099402066 },
	{ "1C(CNC(=O)C1Br)C2=NC=NC=C2", 0.852295586007 },
	{ "2-amino-3-(34-dihydroxyphenyl)-N-propyl-propanamid", 1.21510572459 },
	{ "ASMVHTPAWRFPDF-UHFFFAOYSA-N", 2.11225888948 },
	{ "WRZLVTZRUYJTPQ-UHFFFAOYSA-N", 1.77528196666 },
	{ "42H49N9O3", 1.64154115 },
	{ "2-(4-nitrophenyl)-N-pyrazolidin-3-yl-ethanamid", 2.33024226313 },
	{ "EMJKLZLUOHPM-UHFFFAOYSA-N", 2.28328927825 },
	{ "34H40O3", 1.64110181955 },
	{ "hI=1S/2CH3.Ti.2Y/h2*1H3;;;/q2*-1;+2;;", 1.22424148193 },
	{ "copper;48-dioxo-2-quinolinecarboxylic acid;dihydr", 2.39486986289 },
	{ "WMNVRTVSQDCMJA-UHFFFAOYSA-N", 2.38206474682 },
	{ "1=C(SC2=NC(=NC(=O)C12)SCC(=O)O", 2.06243870728 },
	{ "ZSXZEDGUYFWLET-UHFFFAOYSA-N", 1.39894247977 },
	{ "[CH2-].CC1=CC(=C)[N-]O1.[Y]", 0.976861524383 },
	{ "44H55NO14", 1.27816721682 },
	{ "22H36N8O10", 0.487191697575 },
	{ "RWGYDPFKRSFYEY-UHFFFAOYSA-N", 0.796244661222 },
	{ "YTFLKFUBQNPIPG-UHFFFAOYSA-N", 1.79816713121 },
	{ "1CC(N(C1)C(=O)C2C=C(C=NC2=O)[N+](=O)[O-])CC(=O)O", 1.75718096486 },
	{ "GQGDAGHPMUQUOD-UHFFFAOYSA-N", 1.64279077093 },
	{ "N1C2=CC=CC=C2C3(C1=CC=NNC(=O)C4=CC=CC=C4)CC5CCCC(C5)C3.[Co]", 1.83242283495 },
	{ "UZLRSPVGSRMMMY-UHFFFAOYSA-N", 1.62688861005 },
	{ "4-keto-2-phenyl-hexahydropyrimidine-5-carbonitri", 1.38794575064 },
	{ "69H88N4O6", 1.79917199693 },
	{ "EJKFFFGWZPLLK-UHFFFAOYSA-N", 1.76606253872 },
	{ "UFCIIANFCGSJE-UHFFFAOYSA-N", 2.03118683676 },
	{ "BAQJFUYHNCJEFA-UHFFFAOYSA-N", 0.682731459417 },
	{ "2-(118-dioxononadecylamino)pentanoic acid", 2.38986338555 },
	{ "ZVVDHLSOQSTYIE-UHFFFAOYSA-N", 0.140019895444 },
	{ "MFHQCCUBDXQXGU-UHFFFAOYSA-N", 1.13225920922 },
	{ "12H11BIrN3", 0.657246947928 },
	{ "SC1=NC2=C(N1C3C(C(C(O3)CO)O)O)N=CN=C2N", 0.72835260534 },
	{ "XHTYAQKROMVDS-UHFFFAOYSA-N", 0.526953280744 },
	{ "27H24N6O14S", 2.03398524321 },
	{ "OC1C(C(OC1N2C=C(C3C2=NC(=NC3=O)N)C#N)CO)O", 0.789924093166 },
	{ "1CC(NN1)NC2=NC=C3CN=C(C4=C(C3=N2)C=CC(=C4)Cl)C5=C(C=CC=C5F)F", 0.116572844545 },
	{ "hI=1S/C6H8NS2.Y/c1-2-8-5-6-7-3-4-9-6;/h4H25H21H3;/q-1;", 1.10886051839 },
	{ "1C(C(NN1)C2=CC(=CC=C2)CNS(=O)(=O)C3=C(C=CC(=C3)F)F)C4=CC=NC=C4", 1.66489090945 },
	{ "N1C(=C2CC3=C(N2)C=C(C=C3)CN)C(=O)C(=N1)C4=CC=C(C=C4)CO", 2.19734013882 },
	{ "(C)(C)OC(=O)NCC1CN(CCO1)CC(CO)C2=C3C(=NC=C2F)C=CC(=N3)O", 1.58744643576 },
	{ "JEJHENFOYJGSRC-UHFFFAOYSA-N", 0.602277957003 },
	{ "OFKNXBAHNUWOJ-UHFFFAOYSA-N", 1.71810769092 },
	{ "12H15F", 0.700148474498 },
	{ "XKONOACPFLOMQ-UHFFFAOYSA-N", 2.00324380441 },
	{ "1C2CNCC1N(C2)C(=O)CCF", 0.553582809357 },
	{ "15H21O3SY-3", 0.234835821009 },
	{ "5-azanyl-2-(2-azanylideneethylamino)pentanoic acid", 2.39282936482 },
	{ "HRXYBOYPZXHFJJ-UHFFFAOYSA-L", 0.831608292023 },
	{ "methyl 3-(butylamino)propanoate;vanadium(2+);yttrium", 1.49893477409 },
	{ "LDZHYERTTMJOGD-UHFFFAOYSA-N", 1.72665091037 },
	{ "HJUXDGGZYPUWHO-UHFFFAOYSA-K", 0.227848253054 },
	{ "FRZLCONSXRFOAD-UHFFFAOYSA-N", 1.92248831455 },
	{ "49H63F2N5O10", 0.471661462266 },
	{ "1=C(NC(=C1CNCCCC(=O)CO)C)C=C2C3=C(C=CC(=C3)F)NC2=O", 1.5092149602 },
	{ "OC(=O)C1=CC=CC(=C1)C2=C3C=CCN=C3C(=O)N=N2", 2.15800280098 },
	{ "(C(C(=O)CO)NC(=O)C1=CC=C(C=C1)C#CC2=CC=C(C=C2)CCCCO)O", 2.1297630223 },
	{ "1CCC2C1(CCC3C2CCC4=CC(=O)CC(C34C)O", 0.647472714232 },
	{ "JHGOYPLJZXXRHW-UHFFFAOYSA-N", 1.48216118256 },
	{ "2-sulfanylpentanethi", 0.649770399825 },
	{ "QMKUSCDJJQFWLA-UHFFFAOYSA-N", 1.5675143857 },
	{ "AWGTWUIPYAPKNE-UHFFFAOYSA-N", 0.399593930209 },
	{ "4-[2-(5-bromo-2-thiophenyl)ethenyl]-NN-dibutylanili", 0.902642880619 },
	{ "HUHZDRAMVHEMEY-UHFFFAOYSA-N", 0.720565370208 },
	{ "YQEOKZALGBLASN-UHFFFAOYSA-N", 1.96598068062 },
	{ "2-amino-N-methyl-4-(methylthio)-N-(3-pyridylmethyl)butyramid", 1.85055762639 },
	{ "1=CC2C(=C3C=CC=CC3=COPO2)C=C1", 1.02154351404 },
	{ "1-(4-tert-butylphenyl)-6-hex-2-enyl-2-piperidinone;yttrium", 2.07072583446 },
	{ "MCAFUHPCQQJWFD-UHFFFAOYSA-N", 2.327598847 },
	{ "LYDPRXKLIIBJTH-UHFFFAOYSA-N", 2.14617035 },
	{ "18H15Cl2FN2O3S", 1.69315927451 },
	{ "OC(=O)C1=CN=C(S1)NC(=O)C(=NOC2CCCC2)C3=CC=C(C=C3)S(=O", 0.293104088491 },
	{ "1-[(1R)-1-methoxyethyl]pyrimidine-24-qui", 0.634344279505 },
	{ "KEAQLVMLVPQFX-UHFFFAOYSA-N", 0.872813813951 },
	{ "OC1=CC=C(C=C1)COC(=O)C2=C(CSC3N2C(=O)C3N=[N+]=[N-])C4=CC=CO4", 0.648213860639 },
	{ "(2-methyl-1-oxo-1-phenylpropan-2-yl) carbam", 2.39378491416 },
	{ "1=CC(=C(C=C1CC(C(=O)NCCC(=O)O)N)O)O", 1.32466486304 },
	{ "hyl 3-[2-fluoro-6-[1-(oxiran-2-ylmethoxy)butyl]phenyl]prop-2", 0.383955715045 },
	{ "32H23BeN3O3+2", 0.607055616185 },
	{ "(=CC(C)C(=O)C1=CC=C(C=C1)N(C)C)C=CC(=O)NO", 1.38098536222 },
	{ "1=C(C=CC=C1Cl)NCC(=NO)N", 1.31068143751 },
	{ "QSDKSCLVHMRROG-UHFFFAOYSA-N", 1.17731961887 },
	{ "RECAETINHPXPKE-UHFFFAOYSA-N", 1.65289138987 },
	{ "1COCCN1C2=NC(=NC(=C2)C3(CC3)S(=O)(=O)C4CCOCC4)C5=CC=C(C=C5)N", 0.336493564187 },
	{ "ARAAFNAAYFFSRK-UHFFFAOYSA-N", 1.10553136423 },
	{ "1=CC(=CN=C1)CC(=O)CCC(=NOC)C2=CC(=C(C=C2", 1.45848154433 },
	{ "RDLJRXJOHRTJNE-UNXLUWIOSA-N", 0.862646359797 },
	{ "hI=1S/C6H4N2O2/c1-4(9)5(7-2)6(10)8-3/h9H1H3", 2.30407015752 },
	{ "3-[(3-chloro-5-methyl-phenyl)methoxy]-2-phenyl-piperidi", 1.5461588456 },
	{ "(3-benzoxy-2-iodo-4-methoxy-phenyl)meth", 0.861316095764 },
	{ "N-[1-(aminocarbohydroximoyl)butyl]-25-dimethoxy-benzamid", 1.78490121663 },
	{ "43H41N2+", 0.303260250469 },
	{ "17H28O7", 0.137177106185 },
	{ "GZEXWWLCJWCMTO-UHFFFAOYSA-N", 2.1821714156 },
	{ "[C-]#[N+]C1=C(C=[C-]C2=C1C=CN3C2=NC=C3)F.[Ir]", 0.678069077512 },
	{ "LVOXLJRIIWWCPD-UHFFFAOYSA-N", 1.91771163524 },
	{ "BCTYSFSJTASTJV-UHFFFAOYSA-N", 1.62898196851 },
	{ "9H18OP+", 1.26542930235 },
	{ "7-ethyl-3-iodo-imidazo[12-a]pyridi", 2.32821229961 },
	{ "SCCOC1C(C(C(C(O1)CO)(C2(COC(C(C2O)O)CO)O)O)O)O", 0.278981060239 },
	{ "1=C(NC2=C1C(=C(C=C2C(=O)O)F)Br", 0.736232261737 },
	{ "5-(2-phenoxyphenyl)triazolidine-4-carboxylic acid ethyl ester", 1.65067438622 },
	{ "26H30ClN3O4", 1.73541374913 },
	{ "JXOWXMGCVAGOGM-UHFFFAOYSA-N", 1.17537998761 },
	{ "LBSULSKBZXNTRR-UHFFFAOYSA-N", 0.601902978075 },
	{ "4-(4-tert-butylphenyl)-7-hydroxy-3-hepten-2", 2.0749696971 },
	{ "NVBOAQZUZUVFK-UHFFFAOYSA-N", 0.525881170611 },
	{ "2-pentyne;triiodovanadium", 0.155309842707 },
	{ "XXMLWNLINMOQCL-UHFFFAOYSA-N", 2.13310672961 },
	{ "(C)C[N+]1=C(C(=NCC(C)CO)C(=O)N(C1=O)C)N", 0.311381902077 },
	{ "KUEHFYGMJWBALZ-UHFFFAOYSA-N", 1.19440476911 },
	{ "GSOKGUGERNSSDV-UHFFFAOYSA-N", 0.226950023932 },
	{ "QOCFGASJASSBLG-UHFFFAOYSA-M", 0.118508614505 },
	{ "21H29NO3S2", 0.133099697355 },
	{ "(C(=C1C(=C(CBr)Br)O1", 1.32758008991 },
	{ "(C1=CC=CC=C1Cl)NC(=O)C=CC2=CC=C(C=C2)C(=O)O", 1.66647905338 },
	{ "UMVQZYPINGXJAO-UHFFFAOYSA-N", 0.250681564593 },
	{ "YYEQFDYNBWZEGE-UHFFFAOYSA-N", 1.15346642233 },
	{ "GVKLSQRAAGIC-UHFFFAOYSA-N", 0.626764118795 },
	{ "(C(=O)N(C(C)C(=C)C)O)NC(=O)OC(C)(", 1.22579750666 },
	{ "OC1=CC=C(C=C1)OC2C(C(C(O2)COP(=O)(O)OP(=O)(O)OP(=O)(C)O)O)O", 0.659163826383 },
	{ "26H25FN2O3S2", 1.09818954303 },
	{ "12CC(C3C(C1CCC2(C=CI)O)CCC4=C3C=CC(=C4)O", 0.322291502391 },
	{ "FDRMIBMVGCDSDP-UHFFFAOYSA-N", 1.49535519841 },
	{ "phenylsulfonyl 2-azanyl-4-methylsulfanyl-b", 1.21114467604 },
	{ "NVSZTRRCCVBNJV-UHFFFAOYSA-N", 1.06180801817 },
	{ "ZECGDUGLDFPJPE-UHFFFAOYSA-N", 0.483737002412 },
	{ "(2-ethoxy-1-formyl-2-oxo-ethylidene)-methylimino-ammonium", 0.116043901469 },
	{ "33H33Br2N7O6S2", 0.82801773339 },
	{ "AUQSIGAPZJCXCT-UHFFFAOYSA-N", 1.36118220731 },
	{ "27H47O2PW", 0.436253155289 },
	{ "18H26N8O2", 0.134200351456 },
	{ "XBKKFVNGLLWFQ-UHFFFAOYSA-N", 0.23834002183 },
	{ "30H58N8O14", 0.616325335167 },
	{ "QFQNUMRVMRWSIF-UHFFFAOYSA-N", 1.2346042974 },
	{ "dium;2-aminopentanedioate;hydr", 1.28539589357 },
	{ "13H12Cl2N2O4S", 0.261672419402 },
	{ "4-(5-chlorothiophen-2-yl)-1-ethoxy-3-methyl-14-dioxobut-2-en-2", 1.47352857489 },
	{ "VGIDAXBSGXJHBG-UHFFFAOYSA-N", 0.96092949841 },
	{ "15H9F6N5O2", 1.88044895109 },
	{ "hyl 4-(3-chloroallyloxy)benz", 0.783720300535 },
	{ "VYWROFRUUPRKKB-UHFFFAOYSA-N", 2.00351898273 },
	{ "JAUGJVUCMZKVMX-UHFFFAOYSA-N", 1.85039363452 },
	{ "VMJVTMUCDIBNCN-UHFFFAOYSA-O", 0.77971216595 },
	{ "21H28N5O8Tm-3", 1.47424997368 },
	{ "4-[(4-chlorophenyl)methoxy]piperidin-2", 0.539781609142 },
	{ "2-amino-33-dimethyl-N-(2-methyl-2-morpholino-propyl)butanamid", 0.839727454233 },
	{ "ZDMZJWGCMXXROS-UHFFFAOYSA-N", 2.25877975899 },
	{ "30H24F6N5O4+", 1.83420362735 },
	{ "FPBANOHADWVIS-UHFFFAOYSA-N", 1.28876997353 },
	{ "FXXOYHNIPAAZJX-UHFFFAOYSA-N", 1.7949337423 },
	{ "SLCKRBVBHMLYKY-UHFFFAOYSA-N", 1.09001555094 },
	{ "OC1=C(C=C(C=C1Br)C[C@H](C(=O)O)[NH3+])Br", 1.41829244925 },
	{ "9-[(4-propan-2-ylphenyl)amino]-10bH-benzo[c][18]naphthyridin-6", 0.663346847378 },
	{ "N&apos;-hydroxy-2-(1-keto-14-thiazinan-4-yl)propionamidi", 1.40428592328 },
	{ "28H22ClF3N4O3S", 1.08200394111 },
	{ "11H12N2O4S", 0.136622638978 },
	{ "N-[[ethanoyl(methyl)amino]oxymethoxy]-N-methyl-ethanamid", 2.02779866891 },
	{ "HLCHMXSCFVFGJJ-UHFFFAOYSA-N", 2.27932799948 },
	{ "QLSXECIPKVNLHK-UHFFFAOYSA-O", 1.80449960869 },
	{ "(46-diphenyl-1-naphthoxy)-bis[(2-methyl-8-quinolyl)oxy]alum", 0.287616226435 },
	{ "30H30F6N2O3", 0.398597190562 },
	{ "WBWYKWXNAOMQTR-UHFFFAOYSA-N", 2.26158172888 },
	{ "OWNXZWEALFHQH-UHFFFAOYSA-N", 1.79876822673 },
	{ "YLPIJAPCKXLBTC-UHFFFAOYSA-N", 2.15960270876 },
	{ "(C)(C)OC(=O)N1CC(C(C1)N)C2=NC=CC3=C2C=CN3", 1.41023474009 },
	{ "18H21BrN4O5", 1.26872248784 },
	{ "1=C(N=C[N-]1)CC(C(=O)O)N.[Ni]", 2.3318806949 },
	{ "3-[3-[[(4-fluorophenyl)thio]methyl]phenyl]-2-propenoic acid", 0.564191854511 },
	{ "GMYNEJIGECKZDX-UHFFFAOYSA-N", 1.08134099126 },
	{ "35H43N5O4S", 0.235433777325 },
	{ "4-(25-dimethylphenoxy)-N&apos;-hydroxy-3-methyl-benzamidi", 0.889993168062 },
	{ "4-methoxy-35-dimethyl-6-[(E)-prop-1-enyl]pyran-2", 1.13254893683 },
	{ "USZHXDXNKMRZFN-UHFFFAOYSA-N", 1.00788527174 },
	{ "4-[4-(4-acetylphenyl)anilino]-2-thieno[23-c]pyridinecarboxamid", 1.69771934695 },
	{ "1CCC(CC1)SC2=C(C=CC=N2)C(=NO)N", 1.35817322646 },
	{ "1=CC=CC=C1N2C(=C(C=N2)C(=O)NS(=O)(=O)C=CC3=CC=CC=C3", 1.93055386749 },
	{ "1-(2-naphthalenyl)-3-[(phenylmethyl)amino]-2-propen-1", 1.81844147948 },
	{ "2-(azepan-1-yl)-4a567-tetrahydrocyclopenta[d]pyrimidin-4", 2.16133156748 },
	{ "N1C(C(C(=O)NC1=O)NC(C)CC2=CSC=C2)N", 0.833057744494 },
	{ "42H55ClN4O7", 1.44952159247 },
	{ "1=CC=C(C=C1)S(=O)(=O)C=CC(C)(C(=O)OCC2=CC=CC=C2)N3CCCC3", 0.166569675727 },
	{ "20H26Cl3NO2", 2.37330653599 },
	{ "YLBCLELGTRDSTG-UHFFFAOYSA-N", 0.517217714616 },
	{ "OC(=O)CC1C(=O)N(CCN1C(=O)CNC(=O)CC2=NC3=CC=CC=C3N2)CC(=O)O", 1.14434082336 },
	{ "35H43FN4O10", 1.51581171353 },
	{ "1=CC=C(C=C1)C(=O)C=CC2=CC3=C(C4=CC=CC=C42)OC(=O)C(=C3)C(=O)N(", 2.2692487782 },
	{ "(C)(C)S(=O)NC(C1=CC=CC=C1)C2=C(C=C(C=C2", 1.78969103016 },
	{ "FVWGWHWHUGISEP-UHFFFAOYSA-N", 0.292838641951 },
	{ "#CCC(CC=O)O[Si](C1=CC=CC=C1)(C2=CC=CC=C2)C(C)(", 0.320011698923 },
	{ "NC(CNCC1=CN=CC=C1)C2=CC=CC=C2", 1.15708597771 },
	{ "N(C)C1CCN(C1)C2=CC=CC=C2C=C3C(=O)NC(=O)S3", 0.184845064042 },
	{ "36-bis[3-(ethylamino)propoxy]-8aH-acridin-9", 1.83990617582 },
	{ "RUWYIXKMLYKZIP-UHFFFAOYSA-N", 2.38003889409 },
	{ "WDDRRLKIVQPSFW-UHFFFAOYSA-N", 0.438136642183 },
	{ "6-mercapto-13-dimethyl-5-methylenepyrimidin-1-ium-24-di", 0.741379834395 },
	{ "20H22N3O3PUV-2", 0.762210104502 },
	{ "25H39F3N2O2", 1.6949355178 },
	{ "2-amino-N-(2-hydroxyethyl)-N-isopropyl-33-dimethyl-butanamid", 1.1618799636 },
	{ "HEZOPRTWIXRUOP-UHFFFAOYSA-N", 1.22729882807 },
	{ "UCOLSRALLZBQEF-UHFFFAOYSA-N", 1.23566154975 },
	{ "XBUSYJABPKFHY-UHFFFAOYSA-N", 1.78678429107 },
	{ "3-[4-[[benzyl(methyl)amino]methyl]phenyl]acrylic acid", 0.363542398624 },
	{ "VLDOVBCIXCYLMW-UHFFFAOYSA-N", 0.895689869117 },
	{ "QDBROPIRANGARB-UHFFFAOYSA-N", 2.31344393756 },
	{ "OC1CCC(CC1)OC2=C(C=CC(=C2)C3C(C(C(C(O3)CO)O)O)O)C#", 1.95251413066 },
	{ "62H78N8O8", 0.386459882612 },
	{ "ACSCVFZKCHIBEJ-UHFFFAOYSA-N", 1.88170953244 },
	{ "SFXPPJAHTQKQBP-UHFFFAOYSA-N", 1.2644090459 },
	{ "2-(6-benzyl-4-oxoquinazolin-3-yl)-1-hydroxyethenolate;yttrium", 0.745171961372 },
	{ "(C(C(=O)NC1=C(C=C(C=C1)I)Cl)N2C(=O)C(NC2=O)C3=CC=C(C=C3)OCCO)O", 1.58629350809 },
	{ "27H27N3O7", 1.3921677982 },
	{ "1-methyl-2-(oxidanidylamino)-4-prop-1-en-2-yl-cyclohexan-1", 1.51435357182 },
	{ "7-amino-3-$l^{2}-azanyl-heptan-2", 2.24054035958 },
	{ "GLLDBYUIUDFFMB-UHFFFAOYSA-N", 2.00753841348 },
	{ "(C)(C)C(=O)C1CC(CN1)O", 1.25654174371 },
	{ "=CC1=NC2=CC=CC=C2C(=N1)NCC3=CC=CC=N3", 1.43985371778 },
	{ "S-benzyl 2-amino-444-trifluoro-butanethi", 0.759356215531 },
	{ "42H52FN7O13S2", 1.23880964255 },
	{ "(=O)OC1=CC=C(C=C1)C2=CC=C(C=C2)C(=O)NC(C(C)OC(=O)CC)C(=O)CCC#N", 2.36537583517 },
	{ "HUWMEDBIHVVHLO-UHFFFAOYSA-N", 1.94397378901 },
	{ "N&apos;-hydroxy-3-(2-methylpiperidin-1-yl)butanimidamid", 0.357005247989 },
	{ "2-[[3-(aminomethyl)piperidin-1-yl]methyl]-4aH-quinazolin-4", 1.79989231577 },
	{ "23H14IrN", 0.337478613955 },
	{ "1C(N(C=CC1=O)C2=CC=C(C=C2)I)C3=CC=C(C=C3)C(=O)O", 0.356943739523 },
	{ "WLRQJIPGIWCXTN-UHFFFAOYSA-N", 1.69840052901 },
	{ "YVFGGNYAYDYKRZ-UHFFFAOYSA-N", 0.889992733598 },
	{ "N1C=CC=C1C(=O)NC2=CC=C(C=C2)C=CC(=O)O", 0.497869384129 },
	{ "HJUZXBNRLGKERP-UHFFFAOYSA-N", 1.92677841476 },
	{ "MLTVPPILTSRDMS-UHFFFAOYSA-N", 1.80307738362 },
	{ "USSSCWKDXXPKLQ-UHFFFAOYSA-N", 1.97615061954 },
	{ "ONQCJBEFABMHN-UHFFFAOYSA-N", 1.15748276912 },
	{ "38H50O4S2", 0.459320733293 },
	{ "NNRNPKUFAQOYSP-UHFFFAOYSA-N", 1.98299836107 },
	{ "13-bis(2-pyridinyl)-132$l^{2}-diazaborole;iridium", 1.71664039769 },
	{ "2-[[(5-bromo-2-thiazolyl)amino]methylidene]-1-benzothiophen-3", 1.97719434451 },
	{ "2-methyl-1-[2-(4-methylphenoxy)ethyl]pyrrolidi", 0.871466787866 },
	{ "SUDZXSHVLFBZCO-UHFFFAOYSA-N", 0.744471337376 },
	{ "1=CC=CC=C1OCCN(CCO)C2CCC3=C2C=CC(=C3)C=CC(=O)O", 1.70492856218 },
	{ "GBNZQWKLXFEPI-UHFFFAOYSA-N", 1.58494002752 },
	{ "2-[(3-fluorophenyl)sulfonylamino]-4-keto-4-methoxy-butyric acid", 1.99129415583 },
	{ "RFPHLZBKIUPDFI-UHFFFAOYSA-N", 2.33047810128 },
	{ "(=O)C(C)(CO)COC(=C)O", 1.74779460159 },
	{ "3-[5-(methylsulfamoyl)-3-pyridyl]prop-2-enoic acid", 1.11644677441 },
	{ "2-(35-dimethylpiperidin-1-yl)-N&apos;-hydroxybutanimidamid", 2.1941091178 },
	{ "19H18N+", 2.33514289061 },
	{ "AGPPAPRPDZQQEO-UHFFFAOYSA-N", 0.521845934142 },
	{ "2-amino-1-(2-fluorophenyl)-44-dimethyl-3-p", 1.8235418616 },
	{ "22H26F2", 0.441825130403 },
	{ "UVWAAVNZQNHQGP-UHFFFAOYSA-N", 2.18830380911 },
	{ "[C@@H]1CCN(C[C@@H]1N(C)C2=NC=NC3=C2C(=CN3)C)S(=O)(=O)C(C#N)C(=O", 0.517806193867 },
	{ "55H43N", 0.175610985041 },
	{ "22H50O5Si4", 0.184180482405 },
	{ "OBWKXGVAVTODD-UHFFFAOYSA-N", 0.590228027899 },
	{ "4-fluoranyl-1-methanidyl-2-(methoxymethyl)pyrrolidi", 1.86891777902 },
	{ "OC1=NC2=C(C=C1)N=CC(=O)N2CCN3CCC(C(C3)CNCC4=NC=C5C(=C4)OCCO5)O", 1.93642221819 },
	{ "LUACOURPRLWJLS-UHFFFAOYSA-N", 0.745889688471 },
	{ "14H12IN3S", 2.06968100977 },
	{ "JYJZQOBCDXVLF-UHFFFAOYSA-N", 1.14989014055 },
	{ "FDGHLBKLTMLGBA-UHFFFAOYSA-N", 1.06573094222 },
	{ "30H38N4O2S2", 0.336598810343 },
	{ "50H75NO13", 0.843684482489 },
	{ "YHHWZBOJGVREGX-UHFFFAOYSA-N", 1.14548537697 },
	{ "1=C(C=C(C=C1)CC2=CN=C(S2)NC(=O)C3CC(NN3)C(C)C)F", 1.0995601576 },
	{ "rt-butyl 4-vinylazepane-1-carboxylate;yttrium", 2.20896220618 },
	{ "2-(N-[3-(5-bromo-2-thienyl)acryloyl]anilino)acetic acid", 0.12518473025 },
	{ "2-methyl-1-[2-[(2-methylpropan-2-yl)oxy]morpholin-4-yl]propan-2", 2.30498522839 },
	{ "37H45N4O5SY", 1.90971140346 },
	{ "1=CC=C(C(=C1)NC2=NC=CC(=C2)C(=O)O)O", 0.942625125064 },
	{ "12H19Cl2N", 1.61805342533 },
	{ "YDWHPYMHSBONEC-UHFFFAOYSA-N", 0.142690502844 },
	{ "JWELELHXBNAHCD-UHFFFAOYSA-N", 2.19898065566 },
	{ "GOBJANWSNPLEHA-UHFFFAOYSA-N", 1.95826450312 },
	{ "2-[3-(2-methoxyphenyl)prop-2-enoylamino]oxyacetic acid", 0.416378762428 },
	{ "RPBCDVFGRQJVSR-UHFFFAOYSA-N", 0.64542770471 },
	{ "(1Z)-1-[[2-(1-cyclohexenyl)ethylamino]methylidene]-2-naphth", 0.98234344894 },
	{ "(C(C(=O)[O-])[NH3+])OP(=O)(O)O", 1.83667801338 },
	{ "15H21Cl2NOTi", 1.36295697936 },
	{ "MFBPDXNOJJWXQP-UHFFFAOYSA-N", 2.31049372141 },
	{ "KEJYFDPTRDTTRS-UHFFFAOYSA-N", 0.576069570809 },
	{ "ZJDNJANHYIWKPA-UHFFFAOYSA-N", 1.5264942579 },
	{ "1=CSC(=C1)CCNC(=O)C2=CN=CC(=C2)C=CC(=O)O", 1.6431966162 },
	{ "(C)CC1=C(C(=NN1C(C)C)OC2C(C(C(C(O2)CO)O)O)O)CC3=CC=CC=C3", 1.74006280638 },
	{ "ZEHGPRADAGXYKZ-UHFFFAOYSA-N", 1.88086950623 },
	{ "YJKDXJYCPMPDH-UHFFFAOYSA-N", 0.535967188909 },
	{ "1CC(C1)(C2=NC=C(N2)C3NC(=O)NN3)N", 1.68033453945 },
	{ "[12-diformyloxy-2-(4-methylbenzoyl)oxyethyl] 4-methylbenz", 0.756092015458 },
	{ "(C)C(CC(C(CC1CCCCC1)N)O)O", 1.67741785718 },
	{ "1=CC(=CC=C1)S(=O)(=O)NC2=CC=C(C=C2)C3=CN=CC4=C3C=C(S4)C5=NNN=N5", 1.31079834871 },
	{ "KXPYHNWEYFQGX-UHFFFAOYSA-N", 2.1982533088 },
	{ "ZTBRXJKNVYVREW-UHFFFAOYSA-N", 0.465250393108 },
	{ "39H45N3O3S", 1.38138245251 },
	{ "6-pyrrolidin-1-ylcarbonyl-13-diazinane-24-di", 0.996282268716 },
	{ "3-[4-[2-[methyl(pentyl)amino]ethoxy]phenyl]prop-2-enoic acid", 0.934244393186 },
	{ "MKZVVMBYSCCMNG-UHFFFAOYSA-N", 1.09651266157 },
	{ "3-[2-(3-ethoxy-2-pyrrolidinyl)-2-oxoethyl]-5-fluoro-4-quinazoli", 2.14053462848 },
	{ "LNUFPHQYTLBCJF-UHFFFAOYSA-N", 0.724287235434 },
	{ "35H31IrN4", 0.90505418244 },
	{ "1=C(C(=CC=C1)OC2=C(C=C(C=C2)C(=NO)N", 1.86648431293 },
	{ "2-(4-ethoxypiperidine-1-carbonyl)-N&apos;-hydroxybutanimidamid", 2.01872720361 },
	{ "2-pyrrolidin-3-yloxy-3-tetrahydropyran-4-yl-pyrazi", 1.09292401026 },
	{ "(C(C#N)O)N", 2.06419895488 },
	{ "BENICRKNDJMIEI-UHFFFAOYSA-N", 1.37984318169 },
	{ "methyl 2-benzyloxy-5-[1-hydroxy-7-(4-pentylphenyl)heptyl]benz", 0.694683461317 },
	{ "(=O)NC1=NC(=CS1)C2CCCN2C(=O)OC(C)(", 0.245549302207 },
	{ "[3-keto-1-(1-methoxypropyl)-4-methyl-pent-1-enoxy]lead", 1.31678675999 },
	{ "(C)OC1=NC=NC2=NC=C(N=C21)C3=CC=C(S3", 0.836243741324 },
	{ "1=NSC(=C1C=O)NN=C2C3=C(C=C(C=C3)CO)C=C(C2=O)C(=O)NC4=CC=C(C=C4", 1.3761313363 },
	{ "JYQFTVJUALP-UHFFFAOYSA-N", 0.654616965621 },
	{ "29H35BrN2O2", 1.44255207442 },
	{ "ULRSEMCBTLFFDJ-UHFFFAOYSA-N", 0.235350518358 },
	{ "OUZCNZNXQAZMGY-UHFFFAOYSA-N", 1.88570532763 },
	{ "N-[2-(pentanoylamino)cyclohexyl]pentanamid", 1.82119103146 },
	{ "YAZNWWDSGXAKLJ-UHFFFAOYSA-N", 1.49173970112 },
	{ "2-amino-1-(1-azocanyl)-4-(methylthio)-1-b", 1.34835161025 },
	{ "(=O)C1(CC1)CC=CC(C)C2CCC3C2(CCCC3=CC=C4CC(CC(C4=C)O)O", 2.25928169352 },
	{ "1CC(C(NC1)C2=CC=CC=C2)OCC3=CC(=CC=C3)C(F)(F)F", 1.77369119107 },
	{ "(2-tetradec-9-enoyloxy-3-tridecanoyloxy-propyl) tetradec-9", 0.902027538982 },
	{ "N-(2-aminocyclohexyl)-3-(13-benzodioxol-5-yl)prop-2-enamid", 0.971126221714 },
	{ "OC1C2C(CCN1)NC(=O)C3C2CNCC3", 1.66241119516 },
	{ "hepta-135-triene;yttrium", 2.23386420729 },
	{ "MWORFPIQLNOZJH-UHFFFAOYSA-N", 1.78021704497 },
	{ "robenzene;yttrium", 1.13203940158 },
	{ "QXWJNCYYZDAIIT-UHFFFAOYSA-N", 0.172723476158 },
	{ "OPUGPNWVBNLQN-UHFFFAOYSA-O", 1.72824917811 },
	{ "1=CCCC2=C1C(C=C2)CC(C)[N-][N+]#N", 1.64809124799 },
	{ "(CNC1=C(C=C(C=C1)OC)OC)NC(=O)C2(CCCCC2)NC(C3=CC=CC=C3)C(F)(F)F", 0.705508112941 },
	{ "[C-]#[N+]CCCN1CCC(C(C1)O)CN", 1.03522088892 },
	{ "rt-butyl N-(3-oxidanylspiro[4.5]decan-2-yl)carbam", 1.14082240684 },
	{ "BJVDKMKQVUBWJN-UHFFFAOYSA-N", 1.44690482934 },
	{ "UCAQWNKBEMHMSG-UHFFFAOYSA-N", 1.79873224309 },
	{ "N(C(C)C)C(=O)C1=C(C=CC(=C1)C=CC(=O)O)F", 1.35178567038 },
	{ "[7-(4-bromophenyl)-5-oxidanylidene-3-phenyl-octyl] meth", 2.16383986153 },
	{ "(C)(C)[Si](C)(C)OC1CC23CCCC(C2C4(CC3)OCCO4)C(CC1=C)(O)O", 1.34091981893 },
	{ "N-ethyl-N-(2-hydroxyethyl)-3-(2-nitrophenyl)-2-propenamid", 0.400931361015 },
	{ "VXLKDFKIUHYFGU-UHFFFAOYSA-N", 0.163245718938 },
	{ "KWFHYNFOMZHNC-UHFFFAOYSA-N", 1.07149256954 },
	{ "BFKLOTYQCOTHBI-UHFFFAOYSA-N", 0.525030566654 },
	{ "NC(CN1CC2=C(C1=O)C(=C(C=C2)OC)F)(C#C)C(=O)O", 0.873391803651 },
	{ "(7-chloro-3-methyl-2H-pyrazolo[43-d]pyrimidin-5-yl)methylami", 0.106854177288 },
	{ "6-amino-2-(4-piperidinylthio)-13-diazinan-4", 0.450637708383 },
	{ "XUVNSKUGZLOMO-UHFFFAOYSA-N", 1.46047408739 },
	{ "(3456-tetrahydroxy-2-oxohexyl) 4-oxop", 0.715182825786 },
	{ "42H28", 1.79703971247 },
	{ "carbanide;1-methyl-2-phenylbenzene;yttrium", 0.557469274076 },
	{ "RTPLOVZMMIGKGY-UHFFFAOYSA-N", 0.277794175103 },
	{ "XSQNCCXWNVZKKO-UHFFFAOYSA-N", 2.00519980883 },
	{ "3-[4-(azocan-1-ylmethyl)-3-fluorophenyl]prop-2-enoic acid", 0.507553661606 },
	{ "22H29FN4O7", 2.11178743473 },
	{ "24H30NO4", 2.141700454 },
	{ "31H31FN2O", 0.202264269957 },
	{ "TALCOYFTURMJ-UHFFFAOYSA-N", 0.941275197016 },
	{ "25H25F3N6O5", 0.864982143539 },
	{ "N-[23-bis(4-chlorophenyl)propyl]-2-phenyl-butanamid", 1.47811013096 },
	{ "FNQTXEQFNAQRHV-UHFFFAOYSA-N", 1.9678224553 },
	{ "31H35F2NO5", 2.29164896985 },
	{ "53H65N14OsW+2", 0.274004913706 },
	{ "MQZGVNKGFDOVMQ-UHFFFAOYSA-N", 1.58764324237 },
	{ "1=C(C=CC(=C1C#N)F)C2CN3CCN(CC3CO2)C(=O)CC4=C(C=C(C=C4)N5C=NN=N5", 2.36738155861 },
	{ "QTMDBPWICIVGJY-UHFFFAOYSA-N", 2.32694763422 },
	{ "XQHXVRMVCFQIF-JTQLQIEISA-N", 0.260153603777 },
	{ "20H29BN4O6", 0.753331619548 },
	{ "SNYCNHETNJFGV-UHFFFAOYSA-N", 0.967148716198 },
	{ "hI=1S/C9H17NO2/c1-12-7-3-6-10-5-2-4-9(10)8-11/h1911H2-8H2", 1.19355347241 },
	{ "MKNOBPHSZVQVGD-UHFFFAOYSA-N", 0.184984502944 },
	{ "27H34ClFN3O7", 0.977533675212 },
	{ "hI=1S/C6H5.H2NO.Y/c1-2-4-6-5-3-1;1-2;/h1-5H;1H2;/q2*-1;", 0.967220969896 },
	{ "11H15N5O6S", 0.299432281495 },
	{ "XSOHTVCGYVERDP-UHFFFAOYSA-N", 1.17876069277 },
	{ "1COCCN1CCOC(=O)CCCC=CCC2C(CC(C2CSC3=CC(=CC(=C3)Cl)Cl)O)F", 0.307960683685 },
	{ "HJVKHTPRNVZCDE-UHFFFAOYSA-N", 2.04451501886 },
	{ "37H50N4O9S", 0.16406694296 },
	{ "[B]1OO1.[NH4+].O=[Ti].[K+].[Ga]", 0.126790920331 },
	{ "2-(3-methoxyphenyl)-6-methyl-13-dihydropyrrolo[34-c]pyridin-7", 0.707016771782 },
	{ "2-amino-33-dimethyl-N-(oxan-4-yl)butanamid", 0.875317758433 },
	{ "WHETXQJCSUYTPB-UHFFFAOYSA-N", 1.31056898974 },
	{ "(C)CC1C(=O)NC(C(N1)OC)C(=O)NCC2=NC3=CC=CC=C3N2", 0.208207908598 },
	{ "XDPQURWYMXDFKS-UHFFFAOYSA-N", 2.33683815039 },
	{ "EMLBNOKDXDHKM-UHFFFAOYSA-N", 1.49279475974 },
	{ "RXAQKMYGZDEOBV-UHFFFAOYSA-N", 0.92301161647 },
	{ "KLWPGMTZBKASGO-UHFFFAOYSA-N", 0.234263279645 },
	{ "12CCC3C(C1(CCC2(C4=CC=C(C=C4)OC)O)O)CCC5C3(C=CC(C5)O", 2.3271583522 },
	{ "38H62O14", 1.16108522292 },
	{ "(C)CCC(=O)N(CC(C)C)NC(=O)C1CCCN1C(=O)OC(C)(", 0.303251513757 },
	{ "3-[[[3-(1-methylpyrazol-4-yl)acryloyl]amino]methyl]benzoic acid", 0.214248975999 },
	{ "N1C=CC2=C1/C(=N/O)/CCC2", 0.698736821783 },
	{ "1=CC(=C(N1C2=CC=CC=N2)C)C=C3C(=O)N(C(=NC)S3", 2.30387154435 },
	{ "1CC2CC1CC2NC(=O)C3=CC=C(C=C3)NC4=CC=C(N5C4=NC=C5)C6CCNC(=O)C6", 0.264944566853 },
	{ "MXSHJKLOPMKYOG-UHFFFAOYSA-N", 1.44754929716 },
	{ "5-(4-methoxybutan-2-ylamino)phthalazine-14-di", 1.93087023482 },
	{ "N(C)C(=O)OC1C2C(CC3C2O3)C(=CO1)C(=O)O", 0.381920174222 },
	{ "JVHODUBKWVIXQG-UHFFFAOYSA-N", 2.26028031161 },
	{ "QQVFZIXBAANTNO-UHFFFAOYSA-N", 1.31555744556 },
	{ "N1C2=C(C=C(C=C2)C=NOC(=O)C)C3=C1C=CC(=C3)C(=O)C4=CC=CC=C4", 0.364844960123 },
	{ "OOHVDEHFVNFFOI-UHFFFAOYSA-N", 0.294513109461 },
	{ "XRLJOIATXNEQDT-UHFFFAOYSA-N", 0.381367949386 },
	{ "23-difluoro-1-(4-prop-1-enylcyclohexen-1-yl)-4-propyl-benz", 0.808517469995 },
	{ "OUYYVJODRFGHJK-UHFFFAOYSA-N", 2.27021710087 },
	{ "FDQIOVMCKFNH-UHFFFAOYSA-N", 1.15583460922 },
	{ "7-methoxy-8-methylene-quinolin-4", 0.67834106677 },
	{ "YEFBJXLEOPPRBN-UHFFFAOYSA-N", 0.223048422478 },
	{ "1(C(C(OC(C1O)OP(=O)([O-])[O-])C(=O)O)O)O.[O-]S(=O)(=O)[O-]", 2.07208287791 },
	{ "UJMQNMWKFWGDDY-UHFFFAOYSA-N", 2.23592868212 },
	{ "3-cyclopentyl-2-(4-methylimidazol-1-yl)propionic acid", 0.750416342187 },
	{ "=CCC1CNCCNCCNCCNCCN1.Cl[Mn]", 2.19243529672 },
	{ "(=O)OC(C)OC(=O)C1CCC2C1(CC3C4(C2CC(C5=CC(=O)C=CC54C)F)O3", 0.924884805308 },
	{ "RDLTUTRMVXXYCC-UHFFFAOYSA-O", 0.746728042212 },
	{ "26H32N6O8S6", 2.25843602611 },
	{ "OTSPGVACKWSNY-UHFFFAOYSA-N", 2.37213114917 },
	{ "N1C(C(=CC=CC=C2C(C3=CC=CC=C3N2C)(C)C)C(=O)N(C1=S)CC)O", 1.23872277374 },
	{ "12CCC(=O)CC1CCC3C2CCC4(C3CCC4OC(C)(", 2.03598881436 },
	{ "(CC1=CC=CC=C1)N(C)N2CCN(CC2", 1.90836430682 },
	{ "23H27F2IN2O3", 0.987976215072 },
	{ "SAKVRUWVGAUFJ-UHFFFAOYSA-N", 0.684534688926 },
	{ "2-amino-N-(1-isopropyl-4-piperidyl)-33-dimethyl-butanamid", 1.08755610775 },
	{ "3-(4-hydroxyocta-15-dienyl)-2-(8-hydroxy-7-oxo-octyl)cyclop", 1.72882766906 },
	{ "1=C2C(=NN(C2=NC(=O)C1CCC(=O)NC3=NC4=C(S3)C=C(C=C4)O", 0.328125548814 },
	{ "8-methyl-9-(1-methylallyl)-3-propyl-45-dihydropurine-26-di", 0.791079971626 },
	{ "UFHGNEDHNQQGFI-UHFFFAOYSA-N", 2.17532885448 },
	{ "HQOIVFUUPLNLPD-UHFFFAOYSA-N", 1.90879128699 },
	{ "(C(C(C(C(C=O)N)O)O)O)OS(=O)(=O)[O-].[Na+]", 0.629433110666 },
	{ "UDJVNRAWLDPLJX-UHFFFAOYSA-N", 0.730695459624 },
	{ "1=CC=CC=C1CNC(=O)C2=CC=CC(=C2)C(=NO)N", 1.19200073905 },
	{ "N-[2-(2-hydroxyethoxy)ethyl]-2-methylpent-2-enamid", 1.74961685775 },
	{ "AILLMTOPGGZFGN-UHFFFAOYSA-N", 0.529544871326 },
	{ "xophosphine;triiodovanadium", 0.789652589945 },
	{ "1CCNC(C1)C(=O)CC(CCC2=CC=CC=C2)C3=NN=C4N3C=C(C=C4)C(F)(F)F", 1.42646517742 },
	{ "22H29BrO4", 2.11036163977 },
	{ "RXQOSISJLCVABE-UHFFFAOYSA-N", 0.201216084589 },
	{ "SLZEZSDKDLNFE-UHFFFAOYSA-N", 1.46421351573 },
	{ "2-(1-methoxy-3-pyrrol-1-ylpropan-2-yl)isoindole-13-di", 0.173098506461 },
	{ "1=C(C=C(C=C1)C(=O)NCC(=O)N(C)CCCC2CC(NN2)C3=CC=CC=C3", 1.53112403242 },
	{ "KDDKZIAAWFAWKY-UHFFFAOYSA-N", 2.39582208457 },
	{ "ALBKIHDCDZNBKG-UHFFFAOYSA-N", 0.839652524523 },
	{ "18H21BrN2", 1.57024911812 },
	{ "VBUJSLPFOHSMTG-UHFFFAOYSA-N", 0.72732601202 },
	{ "N(C)C(=CN=O)NC1=CC=CC=C1.C(F)(F)(F)S(=O)(=O)O", 1.09600990091 },
	{ "1CN(CC1NCC2=CC=CC=C2Cl)C3=C(C=C(C(=C3)F)C=NNC(=S)N)F", 0.663138125299 },
	{ "2-propan-2-yl-2-buten-1", 1.33548971895 },
	{ "3-ethylidene-5-methyl-cyclop", 0.509005709518 },
	{ "2-fluoro-N-methylbut-2-enamid", 1.95584750572 },
	{ "YLODAPZIMYJRJE-UHFFFAOYSA-N", 0.545631656666 },
	{ "10H16Cl2O4V", 1.36641153334 },
	{ "7-bromanyl-3-(2-methylsulfanylphenyl)-7H-isoquinolin-1", 1.53714554346 },
	{ "1-(4-benzyloxyphenyl)-3-methoxyimino-propan-2-ami", 2.1329096812 },
	{ "N1C2=CC=CC=C2N=C1C3C4CCCC(C4NN3)OCC5=CC=CC=C5", 1.80473316167 },
	{ "3-acetylimino-2-methyl-but-1-en-1", 2.32423990516 },
	{ "FCUDMKRWRRZUIV-UHFFFAOYSA-N", 1.12287921283 },
	{ "WARMJOOBKSFPEI-UHFFFAOYSA-N", 1.16526876722 },
	{ "21H22N10O3", 0.818740870358 },
	{ "OP(=O)(OCC)OC1=C(C=CC(=C1)F)C=C2C(=O)N=C(S2)N3CCCC3", 0.357367676817 },
	{ "GPMCAAPJCWTIPI-UHFFFAOYSA-N", 1.30126889552 },
	{ "27H27ClN2O4S", 1.0133600891 },
	{ "1CC2=C(C=C(C=C2)O)C3(C1CCCC3)CCNC(C)C=", 1.64690321148 },
	{ "29H53N7O5Si2", 1.4235821738 },
	{ "ARNRBGXCKORCJY-UHFFFAOYSA-N", 0.9466368581 },
	{ "MZCFGCASKKIUBR-UHFFFAOYSA-N", 1.91254815198 },
	{ "OCC(COCC)OC1=C(C(=C(C=C1)C2CCCC3N2CCN(C3)C4=CC(=C(C=C4)Cl)O", 1.58419990247 },
	{ "UOZMBUFVLKSCCX-UHFFFAOYSA-N", 0.871617450331 },
	{ "QWUIOXDITPFZRA-UHFFFAOYSA-N", 0.189440326109 },
	{ "(=O)NC1=CC=C(C=C1)C=CC(=O)C2=CC=C(C=C2)NS(=O)(=O", 1.31597841273 },
	{ "N-cyclohexyl-3-(35-dichlorophenyl)acrylamid", 1.24660775225 },
	{ "1CC(C(C1CCOS(=O)(=O)C)CO[Si](C)(C)C(C)(", 0.879587328079 },
	{ "VYGCJDNJPYWVRI-UHFFFAOYSA-N", 1.75111426128 },
	{ "RLEKPYULRURYBM-UHFFFAOYSA-N", 1.26141471627 },
	{ "XIXCUJOEBAZTCZ-UHFFFAOYSA-N", 1.46067811183 },
	{ "OC1=CC(=C(C=C1)C=CC(=O)NC2=NC(=CS2)C3=CC=CS3)O", 2.15853023148 },
	{ "ONKGUCSWSRJDLD-UHFFFAOYSA-N", 0.579034375424 },
	{ "1=CC=C(C=C1)CC(C2C(CC(O2)O)(CC3=CC=C(C=C3)C)O)O", 1.74577796384 },
	{ "dimethyl 2-(6-methoxy-2-phenyl-oxazinan-3-yl)propanedi", 1.0370881578 },
	{ "2-[15-bis(5-nitro-2-furanyl)pent-4-en-2-ylideneamino]guanidi", 2.34964015191 },
	{ "5-keto-3-phenyl-5-(tetrazolidin-5-ylamino)valeric acid", 1.60197911083 },
	{ "6-[chloro-(5-ethyl-2-thiophenyl)methylidene]-1H-benzimidazol-2", 1.76401311322 },
	{ "N-(3-pyridylmethyl)-1-(tetrazolidin-5-yl)ethanami", 1.69947756995 },
	{ "OC(=O)C(=CC1C(C1(C)C)C(=O)OC2CCC3=C2C=CC(=C3)CC=C)F", 1.66138121157 },
	{ "28H30BrNO3", 2.31519546207 },
	{ "QXNOTHPOIMVHPQ-UHFFFAOYSA-N", 1.94284834513 },
	{ "VISVKKIOHHVROE-UHFFFAOYSA-N", 1.90700479706 },
	{ "20H20ClFN4O3S2", 0.267040068263 },
	{ "18H18", 0.148953341801 },
	{ "1=CN=CC(=C1)C(=O)NC2CCC3C2CCN3CC4=CC=CC5=C4NC=C5", 0.785832027607 },
	{ "bis(non-2-enyl) 9-oxidanylheptadecanedi", 1.67981928591 },
	{ "UDVCACLJVYACLV-UHFFFAOYSA-N", 2.158027072 },
	{ "1-[4-(2-fluoro-22-dimethoxy-ethyl)phenyl]propan-2-ami", 1.93725289632 },
	{ "FVYPWFUTJNHLLH-UHFFFAOYSA-N", 2.37359700969 },
	{ "ithium;1-bromo-3-ethylchloranuidylpropane;zinc;chlorid", 1.26100440403 },
	{ "N-[(3-methyl-4-morpholinyl)-sulfanylidenemethyl]benzamid", 2.0195919752 },
	{ "ZNTHVWRXWLQYTK-UHFFFAOYSA-L", 1.98671504083 },
	{ "VQJIGYATXHDRGB-UHFFFAOYSA-N", 2.05547226077 },
	{ "(2-octadecanoyloxy-3-pentadec-9-enoyloxy-propyl) nonadec-9", 2.20416999753 },
	{ "1=CC2=C(C(=C1)NC(=O)CNC(=O)C3=C(C=C(C=C3)Cl)I)C(=O)N=NC2=O", 1.43879929563 },
	{ "13H12O3", 0.14590045647 },
	{ "3-[1-(phenylmethyl)-3-pyrrolyl]-1-(5-tetrazolidinyl)-2-propen-1", 2.00097832591 },
	{ "OC1C=CC2CC(=O)OC2O1", 0.566954456362 },
	{ "BKHJVTACQPMXDO-UHFFFAOYSA-N", 0.689373132103 },
	{ "[CH3-].CC(C)(C)C(C)(C)OC1=CC2=C(C[N-]C2)C=C1.[Y]", 1.37230310491 },
	{ "1=CC2=C([C-]=C1)C3=NC=CN3C4=CSC=C24.[Ir]", 1.78006878858 },
	{ "23H17Cl3N4O", 2.01440766075 },
	{ "KSHALPQFVXZMBP-UHFFFAOYSA-N", 1.23432339575 },
	{ "8-methoxy-1-octanol;2-propenenitrile;uranium(2+", 0.611618213221 },
	{ "12H15AlN2O", 1.32328865791 },
	{ "3-(2-bromo-4-methyl-phenyl)-N-methoxy-prop-2-enamid", 1.59287925953 },
	{ "GIBBNTFBLODHCL-UHFFFAOYSA-N", 2.04767502349 },
	{ "14H17FO4", 1.0588153157 },
	{ "GHOIGLQJNVPTIE-UHFFFAOYSA-N", 0.451159764189 },
	{ "ZXMOUNIRGPGDM-UHFFFAOYSA-N", 2.05475650845 },
	{ "JTRRQWUYXSGKM-UHFFFAOYSA-N", 0.810862947735 },
	{ "23H34ClN5O2S", 2.05129571323 },
	{ "2-[2-isocyano-1-methyl-2-(p-tolyl)vinyl]thioph", 0.151490043897 },
	{ "2-amino-NN-bis(cyanomethyl)-3-(1H-indol-3-yl)propionamid", 1.82300328194 },
	{ "(=O)NC(CCC(=O)O)C(=O)OCC1C(CC(O1)N2CC(C(=O)NC2=O)C)F", 0.425162677914 },
	{ "1C2C(C(=O)N2C(=C1SC3CC(NC3)C(CCN)O)C(=O)O)C(C)O", 0.495024343632 },
	{ "7-fluoranyldodecan-6", 1.24660603556 },
	{ "NQSXSSTXZLWKC-UHFFFAOYSA-N", 0.951818518217 },
	{ "8H18ClN3O2", 2.26859370962 },
	{ "HJOPNERGNFGNNU-UHFFFAOYSA-N", 0.972512075032 },
	{ "N-butyl-4-[2-(methylamino)ethyl]benzamid", 1.83680406126 },
	{ "28H29FN2O3S-2", 2.21276745912 },
	{ "37-bis(chloranyl)-1H-ind", 1.4425183204 },
	{ "ACIPYEMJIAPMOT-UHFFFAOYSA-N", 1.0432864045 },
	{ "23H30FN3O2S", 1.68588475314 },
	{ "HIRUKYIQXIZKJB-UHFFFAOYSA-N", 2.29339034428 },
	{ "ZMHXYOOONGDABO-UHFFFAOYSA-N", 0.776439585876 },
	{ "JNYBDCNMSBFGD-UHFFFAOYSA-N", 1.18434722047 },
	{ "15H27BrN2O2", 0.852982010263 },
	{ "QYBUGRZCBSTMCA-UHFFFAOYSA-N", 2.3396547912 },
	{ "NZIUMGZJYQTPPY-UHFFFAOYSA-N", 1.01951121457 },
	{ "3-cyano-5-methylhexanoic acid ethyl ester", 0.132689053544 },
	{ "3-(4-nitrophenyl)-N-(3-piperidinylmethyl)-2-propenamid", 1.18619508735 },
	{ "MYOZPLSTPVSIC-UHFFFAOYSA-N", 0.920701774242 },
	{ "3-(13-benzothiazol-2-yl)-N-(3-ethoxypropyl)acrylamid", 0.624935148444 },
	{ "1=CC=C(C=C1)CC(C(=O)OC(=O)C(CC2=CC=C(C=C2)N)N)N", 2.07068671151 },
	{ "MHFFYUKHKOZHAF-UHFFFAOYSA-N", 1.79310903435 },
	{ "SVYFXLZWFKEBAI-UHFFFAOYSA-N", 2.00035228349 },
	{ "UKGSMDLLAOHMFF-UHFFFAOYSA-N", 2.25977723675 },
	{ "33H42ClF2N3O3", 0.214806120373 },
	{ "VIJPWDCBAYKWAZ-UHFFFAOYSA-N", 0.296706215784 },
	{ "FLPHBZOVYVGRCW-UHFFFAOYSA-N", 0.789574887123 },
	{ "(5-ethyl-4-methyl-thiazol-2-yl)sulfamic acid", 0.415669845046 },
	{ "15H16F3N3O3S", 2.1385918167 },
	{ "(2-sec-butylphenyl) 13-dimethyl-24-dioxo-5H-pyrimidin-1-ium-5", 2.1752898574 },
	{ "2-azanyl-3-[4-(trifluoromethyl)phenyl]hex-4-en-1", 1.89453732224 },
	{ "41H47N2O10S3+", 0.591265151737 },
	{ "12H12Cl3NO2S", 0.152083921783 },
	{ "2-(4-chloro-2-nitrophenyl)acetic acid tert-butyl ester", 2.21411324124 },
	{ "HKTZAHATZXAGG-UHFFFAOYSA-N", 1.21368806214 },
	{ "(CO)C(C1=CC(=CC=C1)Br)O[Si](C)(", 0.941865599564 },
	{ "YXYBQRQKWRDVFS-UHFFFAOYSA-N", 1.51425442842 },
	{ "19H18FN5O5", 1.57830299606 },
	{ "OCC1C(CC(C1=C)O)O", 0.238755569049 },
	{ "BKWQPEYLPWUWQE-UHFFFAOYSA-N", 1.10504844749 },
	{ "20H26N8O13P2", 1.73481746194 },
	{ "VOBZATGCMNOBJM-UHFFFAOYSA", 1.23672268256 },
	{ "1C2=C(C(=O)N1)N=C(C=C2)C(F)(F)F", 0.77025227658 },
	{ "24H23N5O15S4", 0.252545076152 },
	{ "hyl 3-(4-oxidanylidene-3aH-indazol-1-yl)prop", 1.12371367902 },
	{ "2-[1-[4-(4-fluorophenyl)piperazin-1-yl]ethyl]-4aH-quinazolin-4", 0.601388128922 },
	{ "VYHDWYHIBWVWNS-UHFFFAOYSA-N", 0.662969358635 },
	{ "ZXUWEJSWMHDBAI-UHFFFAOYSA-N", 2.18260200979 },
	{ "UBGRIBGRAMKOSJ-UHFFFAOYSA-N", 0.667539258906 },
	{ "ALQISPAUCACEER-UHFFFAOYSA-N", 0.201364837008 },
	{ "1-(2-azanyl-2-hydroxyimino-ethyl)-1-methyl-ur", 1.89717891868 },
	{ "NUGMBDNSVOAQDW-UHFFFAOYSA-N", 1.79318149405 },
	{ "KOAVRXJZRFDDI-UHFFFAOYSA-N", 1.87419465358 },
	{ "WFJIBXZJLKDQHG-UHFFFAOYSA-N", 1.84081414019 },
	{ "38H53N4O11S", 1.10649245402 },
	{ "1CN2CCC1CC2C3(C4C5=CC=CC=C5C(O3)NN4)C6=C7C=C(C=CC7=NC=C6)O", 1.32587953215 },
	{ "ANYQAUYAZDJH-UHFFFAOYSA-N", 0.766569442145 },
	{ "LYQMWNMRJPYMHR-UHFFFAOYSA-N", 1.08604104591 },
	{ "OLCCQYTLHZXCF-UHFFFAOYSA-N", 1.29980284516 },
	{ "FVXXTUYUHWDCTI-UHFFFAOYSA-N", 0.716742952798 },
	{ "FZYAYFYQRMZAAW-UHFFFAOYSA-N", 0.84899238762 },
	{ "1-(6-ethyl-4-azaspiro[2.5]octan-4-yl)-222-tris(fluoranyl)eth", 0.915572755413 },
	{ "1-phosphanylpiperidin-3", 0.702323863638 },
	{ "6-[hydroxy-(5-methyl-2-thiophenyl)methyl]quinoxaline-23-di", 0.223559006892 },
	{ "ithium;tetralin-6-sulfinate;hydroxid", 1.76924803935 },
	{ "2-(cyclobutylmethyl)-5-ethyl-4aH-pyrano[23-d]pyrimidine-47-di", 0.231736351436 },
	{ "XMSNSCUOLJFXGI-UHFFFAOYSA-N", 0.999598049146 },
	{ "YJNCAHXPAIBCJY-UHFFFAOYSA-N", 1.66651523289 },
	{ "5H12O3R", 2.01023157737 },
	{ "14H8N2O2S", 2.24638848892 },
	{ "27H34N4O7S2", 0.412550197142 },
	{ "NASQREWLNCFTIG-UHFFFAOYSA-N", 0.987782095418 },
	{ "FKSGYENYQYALOQ-UHFFFAOYSA-L", 0.281385757366 },
	{ "1-(4-chlorophenyl)-N-methoxy-2-propanimi", 0.78291785239 },
	{ "222-trifluoroacetic acid (2-carbamoylpyrrolidino) ester", 1.68418596441 },
	{ "1C(C(C(OC1(C)C)OC2=C(C3=C(C=C2)C=C(C(=O)O3)CC(=O)C)C)O)OC(=O)N", 1.42484442245 },
	{ "OFUEOLVUSPAJD-UHFFFAOYSA-N", 2.18335803501 },
	{ "23H27N9O2", 1.00494934932 },
	{ "1=CC(=CC(=C1)C(=O)N)C=CC(=O)NCCCCCC(=O)NO", 1.3222217926 },
	{ "N(C1CCC2C3CC4=C(C2(C1)CCN3)C=CC(=C4O)O)C(=O)CCC5=CC=CC=C5", 0.115479591922 },
	{ "BVELXMFMOGTRPS-UHFFFAOYSA-N", 0.180636274073 },
	{ "7-fluoro-15-dimethyl-imidaz[21-a]isoindole;yttrium", 1.61552663146 },
	{ "(C)C(C(=O)N1CCC(CC1)C2=CC=C(C=C2)Cl)NC(=O)C3CCC(CC3)C(=O)O", 1.49162406539 },
	{ "59H93N13O18", 0.511652567613 },
	{ "1(C=CN=CCN(C1=", 1.06722702943 },
	{ "22H35ClN4O", 0.926813686619 },
	{ "N1C(=O)C(C=[N+](C1=O)C)S(=O)(=O)NC2=CN=C(C=C2)N", 0.198700033943 },
	{ "QVTFSBWZOOXVBU-UHFFFAOYSA-N", 1.63284280343 },
	{ "29H31FN2O3", 0.175988673506 },
	{ "N-[(5-bromothiophen-3-yl)methyl]-N-methylbut-2-enamid", 1.19485277881 },
	{ "43H54N8O8S2", 0.979261592271 },
	{ "NUCWGNZGJKCFHT-UHFFFAOYSA-N", 2.3867450253 },
	{ "2H4N2OW-2", 0.271821307355 },
	{ "6-keto-N-[1-(1-phenylethyl)pyrrolidin-3-yl]nipecotamid", 1.20841761064 },
	{ "(C)C(C#CC1=CC2=C(S1)C(=O)N(CC2)C3=CC=C(C=C3)N4CCC(C4)N(C)C)O", 0.996400431623 },
	{ "5-chloro-2-cinnamamido-benzoic acid methyl ester", 1.48977828027 },
	{ "GSQDVTWXDUBXBC-UHFFFAOYSA-N", 0.991436064479 },
	{ "RAGKDAMCOZYVFA-UHFFFAOYSA-N", 1.98913347439 },
	{ "25H37N2O8Si", 0.690773099473 },
	{ "BKKADWSYIZGPJJ-UHFFFAOYSA-N", 0.672384486418 },
	{ "VOSAMZFHHQRTJ-UHFFFAOYSA-N", 1.93068865375 },
	{ "rt-butyl 5-(N&apos;-oxidanylcarbamimidoyl)indole-1-carboxy", 0.878277724014 },
	{ "BQRGLNDTGQOWPR-UHFFFAOYSA-N", 1.97830363011 },
	{ "23H35FN6S", 2.08391083819 },
	{ "1=CC2=C([C-]=C1)C3=NC=CN3C=C2.C(#N)C(=C(C#N)[S-])[S-].[Cu]", 1.12635636489 },
	{ "3-(2-furanyl)-34-dihydro-124-triazole-5-thi", 1.3922067158 },
	{ "N-(57-dichloropiazthiol-4-yl)-6-keto-nipecotamid", 0.79507744727 },
	{ "KFOKIVRPOPIWPL-UHFFFAOYSA-N", 0.109196298671 },
	{ "QPPFXPPYCVNF-UHFFFAOYSA-N", 0.471282417684 },
	{ "N-[1-(1-aminoethyl)-3-hydroxy-2-oxo-propyl]-4-styryl-benzamid", 1.34524509692 },
	{ "KKVKZLPULCCNNK-UHFFFAOYSA-N", 1.81479198217 },
	{ "1-(3-chloranylprop-2-enoxy)-2-ethyl-benz", 1.29076734179 },
	{ "1C(N(C2=CC=CC=C21)CC3=CC(=CC=C3)Cl)C=NNC(=S)N", 1.17938104268 },
	{ "(=C)C1CCC2C1(CCC3C2C4C(C5C3(CCC(C5)O)C)OC(O4)(", 0.571567844172 },
	{ "XLDFBXHGPCVUGQ-UHFFFAOYSA-N", 0.125139696003 },
	{ "HGGIUIJGESTBNX-UHFFFAOYSA-N", 1.03077424612 },
	{ "N-butan-2-yl-3-pyridin-3-ylprop-2-enamid", 1.47520538495 },
	{ "FQHKWHKASGAVRY-UHFFFAOYSA-N", 0.606558748155 },
	{ "AGEYDJIXKCNTQZ-UHFFFAOYSA-N", 1.68230790151 },
	{ "62H80ClN9O16S", 0.212826448184 },
	{ "2-(1-benzylpyrrolidin-3-yl)-4-bromo-pyridi", 1.26157885516 },
	{ "5-[(2-fluoro-5-methoxy-phenyl)methylene]thiazolidine-24-di", 0.221267656988 },
	{ "32H44N2O7S", 0.715300859912 },
	{ "N(CC1=CC(=C(C=C1)F)C=CC(=O)O)C2CCCCC2", 1.78908777212 },
	{ "NECGJMWKKLEPPJ-UHFFFAOYSA-N", 0.325267002231 },
	{ "WCAWKSOMLCPLU-UHFFFAOYSA-N", 1.78139754867 },
	{ "hI=1S/CH4O2S.O.Sn/c1-4(2)3;;/h1H3(H23);;/p-1", 1.25304874428 },
	{ "2-[(5-iodo-2-mesyl-6-methyl-pyrimidin-4-yl)amino]cyclop", 0.170513967893 },
	{ "NIJDAWHGFNOVRH-UHFFFAOYSA-N", 0.795826687672 },
	{ "42H45N2O2S+", 0.572834169251 },
	{ "THFDXDKLHUMDW-UHFFFAOYSA-N", 0.822736315234 },
	{ "45H42F15NO14", 2.1994356601 },
	{ "1CC1NC2=NC(=NC3=C(C=NN32)C=C4CC(=O)NC4=O)NC5CCOC5", 1.76796286916 },
	{ "JYMXUBSNSDDCO-UHFFFAOYSA-N", 2.34985194268 },
	{ "MBPRAVRGGIGWKT-UHFFFAOYSA-N", 2.26083184944 },
	{ "1-(1-methyl-2-phenylcyclopropyl)-4-nitrobenz", 1.37712424957 },
	{ "2-bromo-3-ethoxycarbonyl-pentanoic acid", 0.729353907119 },
	{ "(C)C(C(=O)NC1=CC=C(S1)C(=O)OC)NC(=O)C2=CC=C(C=C2", 1.74470974445 },
	{ "N-[1-(4-chlorophenyl)cyclobutyl]-3-(5-methyl-2-furyl)acrylamid", 1.90879180514 },
	{ "1CC2(CC3C1=NC(=S)N=C3C(F)(F)F)OCCO2", 0.21686170544 },
	{ "OC1=CC(=C(C=C1)C2C(CC(=S)N2C3=CC=CC=C3)CCCC4=CC=C(C=C4)F)O", 1.65550419607 },
	{ "7H-quinolin-7-ide;yttrium", 2.19631027137 },
	{ "1CC(NN1)NC2=NC(=NN3C2=CC=C3)N4CCN(CC4)C(=O)CC5=C(C=CC=C5", 2.34942525322 },
	{ "10-methoxy-78910-tetrahydrobenzo[a]pyrene-789-tri", 1.26398975712 },
	{ "26H27N4O", 0.115501967572 },
	{ "XKBDXSAYWWSNAM-UHFFFAOYSA-N", 0.218000527149 },
	{ "(C)OC(=O)C1CCCN1C(=O)C(C2=CC=CC=C2)N(CC3=CC=CC=C3)CC4=CC=CC=C4", 0.377935149493 },
	{ "VWLJMDNPRCZJFY-UHFFFAOYSA-N", 0.280879324626 },
	{ "2-amino-5-guanidino-valeric acid chloromethyl ester", 0.328369638527 },
	{ "NMJHOUGOOAYRNU-UHFFFAOYSA-L", 0.69586985936 },
	{ "4-chloro-3-hex-2-enyl-2-[(3-isopropylphenoxy)methyl]cyclop", 1.78657288828 },
	{ "3-[4-[(dimethylamino)methyl]-3-methyl-phenyl]-3H-pyridazin-6", 1.87900373064 },
	{ "1(CC2CC2(C3CCC(C3C1)(COC(=O)C4=CC(=CC(=C4)Br)F)O", 1.05070877808 },
	{ "SKQCKDFYIDMGEC-UHFFFAOYSA-N", 1.31528366941 },
	{ "MTNSRVRPONBHHW-UHFFFAOYSA-N", 1.282855267 },
	{ "(=NOC(=O)C)C(=O)C1=CC=C(C=C1)SC2=CC=C(C=C2)C(=O)O", 0.47802284319 },
	{ "N-[1-(2-furyl)ethyl]-N-(2-furylmethyl)-4-methoxy-anili", 0.868592850827 },
	{ "NJYHPYPSLYQPPB-UHFFFAOYSA-N", 1.26751469241 },
	{ "MTTYSPKJQZMJQO-UHFFFAOYSA-N", 2.3546621149 },
	{ "OTMDOYAQKBUUCC-UHFFFAOYSA-N", 1.25134283797 },
	{ "OXNCQWHEDHCPPY-UHFFFAOYSA-N", 0.226819271009 },
	{ "1=CC(=C2N=NC(=S)N2C3=C4C=CN(C4=C(C=C3)OC)C(C)C)C(=O)C=C1", 1.35112872451 },
	{ "XISDJOVLUASZQU-UHFFFAOYSA-N", 2.30446028657 },
	{ "JKLWLDFPJIEEEB-UHFFFAOYSA-N", 2.37078879186 },
	{ "MBMUHVGAQREVKF-UHFFFAOYSA-N", 1.64034047764 },
	{ "(C(=O)O)S(=O)(=O)N1CC2=CC=CC=C2CC1C(=O)N", 0.842049825289 },
	{ "OEVPMOJRTDNQGN-UHFFFAOYSA-N", 0.141577268148 },
	{ "2-fluoro-5-[(2-oxopiperidine-3-carbonyl)amino]benzoic acid", 0.143961976072 },
	{ "1=CC=C2C(=C1)C(=CO2)N=C=O", 0.375175949675 },
	{ "(CN1CC2CC1C(=O)N2C(C)(C)C3=CC=C(C=C3)F)C(=O)N4C(CC5C4C5)C#N", 1.99745322814 },
	{ "3-chloro-3-(2-methyl-5-nitro-phenyl)-2-phenyl-acrylonitri", 1.55758437472 },
	{ "ctadecyl 2-amino-2-(3-guanidinopropyl)-3-oxo-tetradec", 0.608750995617 },
	{ "1CNNC1NC(=O)COC2=CC=CC=C2O", 1.6066314243 },
	{ "5-[36-bis(oxidanylidene)pyridazin-1-ium-1-yl]pentanethioamid", 2.26757595057 },
	{ "OMUHPPIZKLRDCB-UHFFFAOYSA-N", 1.72016973469 },
	{ "11H9N5OS", 1.63180532318 },
	{ "42H47N5O8", 0.177225396276 },
	{ "2-amino-3-sulfanylpropanoic acid;mang", 1.45335256922 },
	{ "KIMOAWZDRQHKKE-UHFFFAOYSA-N", 0.199412505241 },
	{ "1(C=CSC(=N1)N)C2=CC(=CC=C2)CC(=O)C3=CN=C(C=N3)O", 1.35722824026 },
	{ "FNUNQYIGTPWWEO-UHFFFAOYSA-N", 0.393040144291 },
	{ "131H196N4O53S4", 1.62968840016 },
	{ "N-cyclopropyl-3-(4-ethylphenyl)acrylamid", 2.31582846543 },
	{ "(CCN)CC(C(=O)O)N.[Cu]", 1.52842558447 },
	{ "6H11Br2NO", 0.600547312732 },
	{ "AWNTNXILJHWOR-UHFFFAOYSA-N", 0.685768570676 },
	{ "YRXYZYCGYFUMQX-UHFFFAOYSA-N", 0.245260274716 },
	{ "21H21F2N6O2+", 2.02035257775 },
	{ "(CC)(COCC(=C)C1CCC2C1(CCCC2=CC=C3CC(CC(C3=C)O)O)C)O", 1.16466551491 },
	{ "1COC2=C(N1C(=O)NC3=CC=CC=C3)N=CC(=C2)C4=CC=N[NH2+]4.[Cl-]", 1.6241475241 },
	{ "LEZCVPISFQCWKT-UHFFFAOYSA-N", 0.776199658496 },
	{ "(C)(CO)C1C2C(C(=O)N(C2C3=CC=CC=C3OC)C4=CC=C(C=C4)C5=COC=C5)NN1", 1.44821312296 },
	{ "NC1=N[C@@H]2[C@H]([C@@H]([C@H](O[C@@H]2S1)CCN)O)O", 2.28497453731 },
	{ "3-chloro-2-methyl-3-(2-methyl-4-nitrophenyl)-2-propenenitri", 0.724534186089 },
	{ "3-(1-methyl-132$l^{2}-benzodiazaborol-3-yl)isoxazole;platinum", 1.12759097549 },
	{ "1=CC=C(C=C1)[I-]C2CC(=O)NC(=O)N2", 1.9655032313 },
	{ "2-[(3-fluoranyl-4-methyl-phenyl)methyl]-4-methyl-pyrrolidi", 2.01570641654 },
	{ "MCPYEGEBAUZDI-UHFFFAOYSA-N", 2.27308702176 },
	{ "SITKBFDCISJDAW-UHFFFAOYSA-N", 2.29140849114 },
	{ "ODYTXFFUDBTJQZ-UHFFFAOYSA-N", 1.44270403679 },
	{ "ZRNZJBKCHJCZQX-UHFFFAOYSA-N", 1.5240880605 },
	{ "N-ethyl-1-cyclohex-3-enamine;yttrium", 2.30719902001 },
	{ "N(CC1=CC(=CC=C1)Br)C2=NC=C(C=C2)C(=NO)N", 0.790620131713 },
	{ "31H36F3N7O4", 1.13651266673 },
	{ "NDCPPXXVFMVXLU-UHFFFAOYSA-M", 0.792150417165 },
	{ "XUBKHXCUUXVJG-UHFFFAOYSA-N", 2.14577959431 },
	{ "1=CN(C=N1)CCCCNC(=O)NC(=O)C=CC(=O)O", 1.75386709915 },
	{ "1=CC=C(C=C1)CNC2=CC3=C(C=C2)C(=O)N=NC3", 1.96195848438 },
	{ "NC(=O)C1=CC(=CC=C1)CNC(=O)C=CC2=CN(N=C2C3=CC=CC=C3)C4=CC=CC=C4", 2.15449123898 },
	{ "5-methyl-2-(6-methylhept-5-en-2-yl)-1-cyclohex", 2.36244388231 },
	{ "WAOIDKLDJMXRNZ-UHFFFAOYSA-N", 2.26712387968 },
	{ "FYOKRUDWPAALIU-UHFFFAOYSA-N", 0.139765034401 },
	{ "=CCN(CC(=O)N1CCCC1C(=O)O)C(=O)OCC2=CC=CC=C2", 1.36284728631 },
	{ "OCPRZSJXCVSKRI-UHFFFAOYSA-N", 2.3539839969 },
	{ "AHLKRGBUFNPYTA-UHFFFAOYSA-N", 1.3382591062 },
	{ "VFEGRKQFXXDKFY-UHFFFAOYSA-N", 2.01405872602 },
	{ "12CCCC1C3CCC4CC5(CCC4(C3CC2)C)CC5", 1.07982843093 },
	{ "UQIVKXBQPUYETR-UHFFFAOYSA-N", 1.31959475639 },
	{ "1=COC(=C1)C(C2=CC3=NC(=O)C(=O)N=C3C=C2", 1.1248104402 },
	{ "1=CC2C(=CC(=NC2=O)NCC3=CC=C(C=C3)Br)C=C1", 1.83185032562 },
	{ "3-[5-(1H-indol-3-yl)-2-methylene-1H-pyrrol-3-ylidene]indolin-2", 0.682491974039 },
	{ "OC1CCC2C(C1)CC(C(=O)N2)C3=CCN(CC3)CC4=CC=CC=C4", 2.05564018222 },
	{ "ZTPGIGBCBGOITA-UHFFFAOYSA-N", 0.173284249193 },
	{ "6-[3-aminopropyl(isopropyl)amino]-124-triazinane-35-qui", 1.45922097992 },
	{ "34H60ClNO6SiS", 1.33802122439 },
	{ "55-dimethyl-3-(2-methylpropanoyl)-4-oxidanyl-hex-2", 0.19147699916 },
	{ "(C)(C)[Si](C)(C)OCC(C(C=C)O)O", 1.89961778455 },
	{ "2-(furan-2-yl)-6-methyl-1-(4-methylphenyl)sulfonyl-piperidi", 0.782180762686 },
	{ "(C)CC(C(=O)C(C(C)C(C(O)S(=O)(=O)O)N)C(=O)OCC1=CC=CC=C1)N", 0.840895711709 },
	{ "1C(NC(CC1=O)C(=O)NC2=CC=CC3=CC=CC=C32)C(=O)NC4=CC=CC5=CC=CC=C54", 2.07235248545 },
	{ "VMEOBIOIEZDDGG-UHFFFAOYSA-N", 1.73068247668 },
	{ "OZFDGXZFUDDYAE-UHFFFAOYSA-N", 0.43394678991 },
	{ "OKKKBMQCJSNLRT-UHFFFAOYSA-N", 0.545407009361 },
	{ "22H27N5O7", 1.93538444713 },
	{ "OP(=O)(OCCCCCCCCCC)OC1C2C(C3C(C1OC(O3)O2)O)O", 1.98775310183 },
	{ "SJFYUZMFXYVIG-UHFFFAOYSA-N", 1.05383736676 },
	{ "2-amino-N-(1-benzyl-2-keto-ethyl)-2-phenyl-acetamid", 1.93636382567 },
	{ "KOUODMWUHLKFA-UHFFFAOYSA-N", 1.00883324952 },
	{ "SNWLDJWXADSXTF-UHFFFAOYSA-N", 1.98056557927 },
	{ "STPGTZYMOOKJS-UHFFFAOYSA-N", 2.35735524249 },
	{ "N&apos;-hydroxy-1-(methylamino)tetralin-5-carboxamidi", 1.28590551082 },
	{ "S(=O)(=O)C1=CC=C(C=C1)OC2C(CC3=C(C=C(C=C23)Cl)Cl)N4CCN5CCCC5C4", 2.09835464764 },
	{ "LDCUVPJFMMEYCM-UHFFFAOYSA-N", 1.06722630081 },
	{ "3-methyl-N-(3-methylsulfonylphenyl)-4-pyrazolidinecarboxamid", 0.129068775084 },
	{ "3-carboxypropyloximino-oxido-pyrrolidino-ammonium", 1.218284113 },
	{ "hI=1S/C6H11NO2/c7-5-1-4(3-8)2-6(5)9/h4-68-9H1-3H2", 2.02037517754 },
	{ "ZEBHJAVZHAHING-UHFFFAOYSA-N", 0.259936977148 },
	{ "hyl 3-methyl-4-oxothieno[23-d]pyrimidine-6-carboxy", 1.96365483447 },
	{ "B1(C2=C(CO1)C=CC(=C2)C=CC(=O)C3=CC=C(C=C3)SC)O", 0.612724669891 },
	{ "18H13N2Na3O6S2", 1.70091155223 },
	{ "1(CCC=C2C1CCC3C2(CCCC3(C)C(=O)N4CCOCC4)C)C=", 1.60953681967 },
	{ "2-benzoxy-7-(2-trimethylsilylethoxymethoxy)-3-vinyl-oct-5", 1.52957276185 },
	{ "LKFNMOWIBPRWKM-UHFFFAOYSA-N", 0.409216563621 },
	{ "SWERFYYTCRGCQP-UHFFFAOYSA-N", 0.854750117545 },
	{ "2-amino-5-thiazolo[45-b]pyridinecarbonitri", 0.151938228747 },
	{ "24H14F5N5", 0.338062834432 },
	{ "S(=O)(=O)OC1CCN(C1)C(=C)O", 1.96789378636 },
	{ "N-tetralin-5-yl-3-(135-trimethylpyrazol-4-yl)acrylamid", 1.90606620671 },
	{ "UBFTNDGBXKSTM-UHFFFAOYSA-N", 0.593393612132 },
	{ "1CC(CC(O1)CO)OC(=O)C(F)(F)F", 0.205252800455 },
	{ "EUSVQRLWLHPE-UHFFFAOYSA-N", 0.202949928588 },
	{ "(35-dimethylphenoxy)-[(2-methyl-8-quinolinyl)oxy]gallium", 1.24020298761 },
	{ "FVLBCVKJFWYFSX-UHFFFAOYSA-N", 1.16438531776 },
	{ "OC1=CC(=CC(=C1)CNC(=O)C2=NC(=O)C3C=CC=CC3=C2)O", 2.28625001168 },
	{ "2-isopropyl-34-dimethyl-pent-2", 1.04893316295 },
	{ "1CC2=C3C(=NC(=O)C=C3Cl)C=CC2N1CC(F)(F)F", 2.16568598924 },
	{ "FKBODYLDYUWOMM-UHFFFAOYSA-N", 0.978314891436 },
	{ "AGIQSOCLYNWVSM-UHFFFAOYSA-N", 0.706730698946 },
	{ "26-bis[2-(4-methoxyphenyl)ethenyl]anthrac", 1.89803397095 },
	{ "24H33F3N4O2", 1.35356710102 },
	{ "SGKQDBDNMZYDV-UHFFFAOYSA-N", 0.86683140407 },
	{ "11H17BFNO4", 1.19564463608 },
	{ "XTFBDQXDBUNOFX-UHFFFAOYSA-N", 0.345398052831 },
	{ "N(CC)C1CC(NC1)C(=O)O", 0.679752995574 },
	{ "ROUUGMAPFXCPQS-UHFFFAOYSA-N", 0.949220484939 },
	{ "(=O)OC(C1C(=O)N(CCO1)C2=CC=CC(=C2)C(=O)N(C)CC(=O)N(C)C)C(=O)O", 2.19793898425 },
	{ "1=C(C=C(C=C1)NC(=O)C2=CC=CC=C2)NC(=O)CCN3C(=O)C4C=CC=CC4=NC3=S", 0.56645575 },
	{ "SCDZVASKBQTQHK-UHFFFAOYSA-N", 1.1308975019 },
	{ "TXZFEOZJYBMHIH-UHFFFAOYSA-N", 1.78925791869 },
	{ "24H14N6PtS", 0.237428229369 },
	{ "RACQKIDJVMBZSR-UHFFFAOYSA-N", 0.624795406233 },
	{ "QIURIDDFVODBL-UHFFFAOYSA-M", 2.36549193296 },
	{ "OC1=CC(=C(C=C1)C(=O)C=CC2=CC(=C(C=C2Br)O)OC)O", 0.979730051865 },
	{ "GMDUMARTULFBPR-UHFFFAOYSA-N", 1.91176788844 },
	{ "1CCCN(C1)C(C)CNC(=O)C=CC2=CC=C(C=C2)OCC3=CSC(=N3", 1.69825528269 },
	{ "YDOJYNNCLOFTGQ-UHFFFAOYSA-N", 0.124299601643 },
	{ "UYBQTXYRBBIZKZ-UHFFFAOYSA-N", 0.552525116124 },
	{ "QLVFVOFJOADKCT-UHFFFAOYSA-N", 0.6117929708 },
	{ "1NOC(=N1)C2=CC3=C(N2)C=CC(=C3)C4=C(CC(=N4)C(=O)CCC5CCCCO5", 0.866878474706 },
	{ "(C)N1CCN(CC1)C(=O)C(CCSC)N", 1.81240891486 },
	{ "YFSZYKXSYWZCBG-UHFFFAOYSA-N", 0.901528926759 },
	{ "43H47N9O5", 2.37436733981 },
	{ "ZSJHVHROGZCKCU-UHFFFAOYSA-N", 2.15264711347 },
	{ "GWHLTNLWQISNY-UHFFFAOYSA-N", 2.38356754425 },
	{ "ASHVRYPJUUPECY-UHFFFAOYSA-N", 0.31005381482 },
	{ "KYVIECXZVKKNEG-UHFFFAOYSA-N", 1.95961033037 },
	{ "JOYOBEHZGAVAOW-UHFFFAOYSA-N", 2.33459778173 },
	{ "(2-nonadec-9-enoyloxy-3-tetradecanoyloxypropyl) docosa-1316-di", 2.23642643103 },
	{ "[2-amino-6-(ethylcarbamoylamino)hexanoyl] pyrrolidine-2-carboxy", 1.48099857085 },
	{ "WPDGLKIHUCUPHV-UHFFFAOYSA-N", 1.76259095872 },
	{ "QUBSPFCXPGMDRA-UHFFFAOYSA-N", 0.851187584914 },
	{ "hI=1S/C5H4F4O4/c6-2(7)4(10)12-1-13-5(11)3(8)9/h10-11H1H2", 2.25267000238 },
	{ "FHFLSYPRIHBTSB-UHFFFAOYSA-N", 1.240596853 },
	{ "1-oxo-N-[2-(2-oxolanyl)ethyl]-8aH-isoquinoline-3-carboxamid", 0.768650902401 },
	{ "OC1=CC=CC=C1C2=CSC(=N2)NC(=O)C=CC3=CC=CC=C3[N+](=O)[O-]", 1.93599804554 },
	{ "26H42N6O8", 0.657844605051 },
	{ "AAJUCOGCFJGYSN-UHFFFAOYSA-N", 0.689359900251 },
	{ "1=C(C(=CC=C1)N2CCN(CC2)CC(COC3=CC=CC4=C3C=CN4)OC(=O)C(=O)O", 0.893856932093 },
	{ "FVDYALKCHXMADO-UHFFFAOYSA-N", 0.528644218446 },
	{ "(C)N1C2C(C(=O)NC1=O)NCCN2", 1.82081347859 },
	{ "H7O2PSU", 1.72474176796 },
	{ "43H64N6O9", 2.30006209029 },
	{ "2-[3-(4-bromophenyl)-5-methyl-4-pyrazolidinyl]pyridi", 1.85459081109 },
	{ "14H26O5PSW", 1.26386689244 },
	{ "disodium;2-methoxy-5-(4-methylbenzene-5-id-1-yl)carbonyl-benz", 1.64981026931 },
	{ "OC1C(=CC=C2C1=NC(=CC(=O)C3=CN=CC=C3)N4C2=NCC4)OCCO", 1.02499572264 },
	{ "1=CC=C2C(=C1)C(=CC(=O)O)C(=O)N2CC3=CC=CC(=C3)C(=O)O", 1.57157414277 },
	{ "GFCBKJCKAZVEHV-UHFFFAOYSA-N", 1.00213122541 },
	{ "20H31N5O7S", 2.39733096473 },
	{ "24H21FIrNO2", 0.270862217387 },
	{ "(C)CN(CC(CCC1=CC=CC=C1)O)S(=O)(=O)C2=CC=CC3=C2N=C(S3)N", 0.427664759288 },
	{ "6-[(isopropylamino)methyl]-56-dihydrouraci", 1.392117877 },
	{ "24H24N12O9S3", 0.214040967535 },
	{ "copper;4-hydroxy-3-methyl-3-buten-2", 2.20516786928 },
	{ "OCNPRPUGMGZQCQ-UHFFFAOYSA-N", 1.59341109275 },
	{ "MYLCYQSWLSQZAL-UHFFFAOYSA-N", 0.423798840012 },
	{ "1C(NC(NC1=O)CC2=CC=C(C=C2)O", 1.91936232619 },
	{ "magnesium;carbanide;ethy", 0.324714954571 },
	{ "22H18F4N4O2", 2.16195866549 },
	{ "46H70O8", 1.28500693865 },
	{ "GVALFBVBLRVZIT-UHFFFAOYSA-N", 1.4663724419 },
	{ "(C)(C)OCC(C(=O)OC)NC(=O)N(CC1=CN=CS1)C2CC2", 0.397972760181 },
	{ "OC1=CC(=C(C(=C1)OC)CNC(=O)C=CC2=NC3=CC=CC=C3O2)O", 0.948074252622 },
	{ "28H23F6N5O4S", 1.73972558217 },
	{ "1=NC2=C(O1)CCCC23CNCC3C(=O)N4CCC(CC4C5CCCCC5)C6=CC=CC=C6", 0.124436700825 },
	{ "GYCRQKGSPUEFLY-UHFFFAOYSA-N", 0.514393602881 },
	{ "26H22FNO2S", 1.71581045683 },
	{ "WNKRAFPWHFRWLC-UHFFFAOYSA-N", 2.29133030626 },
	{ "3-[(1-carbamoylpropylamino)methyl]-2-ethyl-hexanoic acid", 0.736978184655 },
	{ "SMDUSLCCBRSISA-UHFFFAOYSA-N", 1.53651372893 },
	{ "NFKPKMOWSYXLU-UHFFFAOYSA-N", 1.65532003066 },
	{ "1-butyl-4-cyclohexyl-piperazine;yttrium", 1.63634014847 },
	{ "19H21NO9", 0.220811709825 },
	{ "OC1=C(C=CC2=C1C=CN2)C(=O)CCN", 1.31560905617 },
	{ "22H21ClF2N6O3", 0.294846846989 },
	{ "SMCUSAOLJMFCH-UHFFFAOYSA-N", 1.09229698582 },
	{ "RVIGXKIDTDCWBQ-UHFFFAOYSA-N", 0.121172358653 },
	{ "NABMPOXTCLBNMV-UHFFFAOYSA-N", 0.302749349874 },
	{ "1C(CN(C1C=CCCCC(=O)O)CC2=CC=C(C=C2)O)NS(=O)(=O)C3=CC=C(C=C3", 0.161884031507 },
	{ "N(CC1=CC=CC=C1OC)C(=O)C(CCSC)N", 1.38514319037 },
	{ "3-(4-tert-butylphenyl)but-2-enenitri", 2.30138191584 },
	{ "AMCCVQMEYSLBOS-UHFFFAOYSA-N", 1.66144523506 },
	{ "OCDNRLAKVCJYFZ-UHFFFAOYSA-N", 0.715128446348 },
	{ "3-methylol-24-dihydropyrazolo[43-d]pyrimidine-57-qui", 1.87370190592 },
	{ "MNKBTSDDRIXAOY-UHFFFAOYSA-N", 0.18772474081 },
	{ "3-bromo-4-oxo-4-phenyl-2-butenoic acid", 1.29893773805 },
	{ "AWTNNHPWGVWPON-UHFFFAOYSA-N", 2.39211862929 },
	{ "VIFBYLMOUGEQKC-UHFFFAOYSA-N", 1.38556529831 },
	{ "20H21NO5S2", 0.240583848742 },
	{ "OCWSUCQMSKLEN-IJKLMLNUSA-L", 1.45792569526 },
	{ "24-dioxo-3-(6-quinolylhydrazono)-1H-quinoline-6-sulfonic acid", 0.551724283354 },
	{ "1014-bis(bromanyl)tetradecan-7-yloxy-tri(propan-2-yl)si", 2.11964450377 },
	{ "99H87N3", 1.9908118335 },
	{ "LVDRDLDTDQPVNS-UHFFFAOYSA-N", 0.141025130781 },
	{ "N-hydroxy-2-methyl-6-(5-phenylpenta-24-dienoylamino)hexanamid", 0.35116545254 },
	{ "NC(=O)C(=O)C(CCC=C)NC(=O)C1CCCN1C(=O)C(C(C)(C)C)NC(=O)OC(C)(", 2.17565415071 },
	{ "VWIHMICBOVUTJJ-UHFFFAOYSA-M", 2.3927270126 },
	{ "1CC=CCCC=CCC(C1)C(=O", 1.33952310059 },
	{ "52H42N4O4", 0.808921661066 },
	{ "pent-1-en-3-ylboronic acid", 1.87483515614 },
	{ "LQFTWBZOYZUXPP-UHFFFAOYSA-N", 0.15541287138 },
	{ "21H24FO5", 1.97297183667 },
	{ "26H34N8O4S", 0.402704962021 },
	{ "1=CC(=CC=C1NC2=NC=C3C(=N2)C(=O)N=NC3=O", 1.10909115719 },
	{ "(=N)C1=CC2=C(C=C1)N=C(C=C2)C=CC=CC=CC3=CC4=C(O3)C=CC(=C4)C(=N)N", 0.355231002789 },
	{ "1CC23CC(C1C)OC2CCC(O3)CCO", 0.246833274282 },
	{ "ASZJEPZTDBOOGV-UHFFFAOYSA-N", 0.883405826337 },
	{ "diethyl 2-[3-[bis(prop-2-enyl)amino]prop-2-enylidene]propanedi", 2.36222845985 },
	{ "KNXSDHDPWHAOSO-UHFFFAOYSA-N", 0.97186174415 },
	{ "MZAUAPVWXYNFRO-UHFFFAOYSA-M", 0.12980499918 },
	{ "24H19N3O6S", 1.33542555213 },
	{ "6-pentoxy-4a56788a-hexahydro-1H-quinazoline-24-di", 1.1273737389 },
	{ "ZYSDIWYKGBJOPO-UHFFFAOYSA-N", 2.22123309695 },
	{ "30H33ClO4S", 1.88875421764 },
	{ "27H36N2O6S2", 0.726768684833 },
	{ "N(CC1=CC=CS1)CC(=NO)N", 2.16844933704 },
	{ "QXHQVINSJQQLQS-UHFFFAOYSA-N", 1.10350214568 },
	{ "3-[ethyl(isopropylsulfonyl)amino]-N&apos;-hydroxy-propionamidi", 0.437726339677 },
	{ "NKXZNTIYOXDDDZ-UHFFFAOYSA-N", 1.86273149665 },
	{ "QQIGMKPBMXYMII-UHFFFAOYSA-N", 0.858338445014 },
	{ "UXJOPWSLBJAFQ-UHFFFAOYSA-N", 0.387191284579 },
	{ "2-[2-azanyl-1-(3-methoxyphenyl)propyl]sulfanyl-13-diazinan-4", 0.460969866253 },
	{ "N-cyclooct-4-en-1-yl-2-methylprop-2-enamid", 2.10348411542 },
	{ "1CC2=C(C1NCCC3=CN=CC=C3)C=CC(=C2)C=CC(=O)NO", 0.635513483843 },
	{ "31H47KO5", 0.163794997496 },
	{ "UQGSZPDACJPDEJ-UHFFFAOYSA-N", 2.01581337395 },
	{ "OC(=O)CC(C)(C)CC(C)C#CC=CC=CC(C)COC1=CC=C(C=C1)F", 0.341838795502 },
	{ "NTRYCCTWPWWUDH-UHFFFAOYSA-N", 1.94028172742 },
	{ "MANQXFRXFSSGGW-UHFFFAOYSA-N", 1.10949078124 },
	{ "25H20F5N3O2", 2.32058262763 },
	{ "(4-methoxy-2-nitro-phenyl) 3-(4-nitrophenyl)prop-2", 2.34593802579 },
	{ "GLTPOMJYUGSOPR-UHFFFAOYSA-N", 0.626816926185 },
	{ "UYYZSFIGQSSTBZ-UHFFFAOYSA-N", 1.50093097416 },
	{ "BNPQNTLTUMYVDX-UHFFFAOYSA-N", 0.186895411411 },
	{ "VXSUIAOYZVGPB-UHFFFAOYSA-N", 0.810484662233 },
	{ "21H24FNO3", 0.691090198257 },
	{ "KJEKMBCTXFQPOK-UHFFFAOYSA-N", 0.937340798274 },
	{ "WAYJMCYGRSSDZ-UHFFFAOYSA-N", 0.726620212305 },
	{ "UFRYRSYMFOEOQH-UHFFFAOYSA-N", 0.56860837106 },
	{ "rt-butyl 2-amino-3-[(1-methoxy-1-oxopropan-2-yl)amino]prop", 1.17139954753 },
	{ "3-(34-dimethoxyphenyl)-2-(4-nitrophenyl)prop-2", 1.55949907224 },
	{ "17H12F2N2O2S", 2.32148076785 },
	{ "UEYUXNVBOMUIOJ-UHFFFAOYSA-N", 0.991386213699 },
	{ "1=NN(C(=C1C=CC(=O)NC2=C(C=CC(=C2)C(=O)N(C)C)OC)Cl)CC3=CC=C(C=C3", 0.409128070274 },
	{ "USACXZWWNXGAAZ-UHFFFAOYSA-N", 0.810923578529 },
	{ "2-(4-propargylpiperazino)ethylami", 1.70741730562 },
	{ "3-[4-(3-ethylpiperidin-1-yl)carbonylphenyl]prop-2-enoic acid", 1.68613555983 },
	{ "XYOHAWCBKJMAK-UHFFFAOYSA-N", 1.23678882768 },
	{ "N(CCO)CC(C[N+]1=CN=C2C1C(=O)N(C(=O)N2C)C)O", 1.70731160051 },
	{ "OZRVHJBILHAHDQ-UHFFFAOYSA-N", 0.475545795382 },
	{ "7H9F3N4O2", 1.22159790554 },
	{ "1C(=O)CCC2(N1CCC3C2CCC4(C3CCC4COCC(=O)O", 0.410215909829 },
	{ "26H25F3N5O5+", 1.83589313964 },
	{ "LBZFACYGLACDAF-UHFFFAOYSA-N", 2.10578417415 },
	{ "1C2C(C(O1)(CO2)C)(C)OP(N(C(C)C)C(C)C)OCCC#N", 2.21500229937 },
	{ "2-(methylamino)-3-oxidanyl-N-(phenylmethyl)propanamid", 1.23663615608 },
	{ "30H52N2O7", 0.91059366947 },
	{ "OCCOCCOC1C(C(C(C(O1)COC(=O)C)C)C)NC(=O", 1.94080687363 },
	{ "UYPKLAYWANDDSS-IHBYNMNLSA-N", 1.12134657521 },
	{ "21H35BrO3", 2.24358681117 },
	{ "JBFZTPSHLYLOGQ-UHFFFAOYSA-N", 0.753075335456 },
	{ "3-(dimethylamino)-4-hydroxy-5-methyl-deca-79-dien-2", 1.97775719284 },
	{ "methyl-phenyl-(2-quinolylmethyleneamino)ami", 0.21336852843 },
	{ "KAFFMSJAFHVHHC-UHFFFAOYSA-N", 1.04082630293 },
	{ "WJLRUIFNDFQFAJ-UHFFFAOYSA-L", 2.33631308816 },
	{ "25-bis(buta-13-dienoxy)-36-dimethylpyrazi", 1.66772268841 },
	{ "4a9b-dimethyl-35-dihydro-2H-[14]dioxino[23-b]ind", 0.741441331818 },
	{ "RNUKJGGRWQRAML-UHFFFAOYSA-N", 0.395534285371 },
	{ "3-chloro-6-fluoro-9-[(3S4R)-3-fluoro-4-piperidyl]carbaz", 0.563213963954 },
	{ "(C)C(C)NS(=O)(=O)C1=CC(=C(C=C1)F)C(=O)NC2=CC(=C(C=C2)C(F)(F)F)F", 0.962950105185 },
	{ "1=CC=C(C(=C1)CCC(=O)NCCN2C(=O)C(=CC3=CC=CC=C3F)SC2=O)Br", 0.514782397322 },
	{ "ROQJTPHRXQIECE-UHFFFAOYSA-N", 1.84876884776 },
	{ "1CCC(CC1)C=CC2CCC(CC2)CCCCCCC(OC3=CC=CC(=C3)N)OC4=CC=CC(=C4)N", 0.565782099659 },
	{ "ONADDLGSXXIJAT-UHFFFAOYSA-N", 1.23107155657 },
	{ "RUTSDVUJSWMH-UHFFFAOYSA-N", 1.90353171893 },
	{ "2-keto-6-[4-(trifluoromethyl)phenyl]nipecotonitri", 2.38347204483 },
	{ "19H24O7", 1.92116018764 },
	{ "hI=1S/C10H16.W/c1-8-4-2-5-9-6-3-7-10(8)9;/h7-10H1-6H2;/q-2;+2", 0.378211770465 },
	{ "methyl 4-amino-2-ethoxy-thiazole-5-carboxy", 1.08682956479 },
	{ "OC1=CC=CC(=C1)CNC2=NC=NC3=C2N=CN3C4C(C(C(O4)COS(=O)(=O)N)O)O", 1.42280611084 },
	{ "(C)N(C1CCCN(C1)C(=O)OC(C)(C)C)C(=O)C2=CC(=C(C=C2)F)[N+](=O)[O-]", 0.873607672407 },
	{ "18H19F2N3O2", 0.31948383449 },
	{ "N-butyl-N-ethyl-4-(2-phenylethenylsulfonylamino)benzamid", 0.739668173415 },
	{ "methyl 3-cyclohexyl-4-(2-isopropylpyrrolidin-1-yl)-4-oxo-b", 1.79006241263 },
	{ "3-(4-hydroxyphenyl)-24-dioxo-13-thiazolidine-5-carboxamid", 2.04762067255 },
	{ "1-butanol;rutherfordium;yttrium", 1.81305295578 },
	{ "AUMDDXKNWAPCJ-UHFFFAOYSA-N", 1.03599259404 },
	{ "1-(2-aminophenyl)-2-(113-trimethylbenz[e]indol-2-ylidene)eth", 2.238568465 },
	{ "BPVPNFQWUIXVOF-UHFFFAOYSA-N", 1.64874096301 },
	{ "1(C(COCC(=N1)N)(F)F)C2=C(C=CC(=C2)CC(=O)C3=C(C=C(C=N3)C#N)Cl)F", 1.80011640419 },
	{ "1=CC(=CC=C1C2=CC=C(S2)C=CC(=O)NC(C(=O)N)C(=O)N)Br", 0.219630266167 },
	{ "ULDWSQIDPDEKDV-UHFFFAOYSA-N", 1.72808280332 },
	{ "WOPUMXURVQTLD-UHFFFAOYSA-N", 0.718730456387 },
	{ "B1(C(CC2=CC=CC(=C2O1)C(=O)O)NC(=O)CC3(CCC(CC3)NCCN)O)O", 1.07018834779 },
	{ "6-azanyl-2-[bis(phenoxycarbonyl)amino]hexanoic acid", 0.296178270653 },
	{ "OC(=O)C1=C(C(=NC=N1)CCC2CCCC(O2)C3=CC=C(C=C3", 0.338634060255 },
	{ "1C(C(C(O1)N2C=C(C(=NC2=O)CC(=O)OC)F)O)F", 0.605426142751 },
	{ "methyl 3-[(4-methylanilino)-phenyl-methylene]-2-oxo-indoline-5", 1.09353773965 },
	{ "3-[2-[(2-chlorophenyl)carbamoylamino]thiazol-5-yl]acrylic acid", 1.05338893795 },
	{ "LQGQQZROJXUZBC-UHFFFAOYSA-N", 0.801111237042 },
	{ "3-(1-oxohex-5-ynyl)-4-(phenylmethyl)-2-oxazolidi", 1.06109680976 },
	{ "2-(aminocarbohydroximoyl)-N-tert-butyl-2-ethyl-butyramid", 1.25915088885 },
	{ "NC(=O)C(COC)NC1=NC(=NC=C1F)C2=CNC3=NC=C(C=C23", 0.411564649138 },
	{ "1CCC(C(C1)CN)(C)C2CCC3(C(C2O)CCC34OCCO4", 2.37594094825 },
	{ "HAFRENWEAZWSQK-FLFQWRMESA-N", 0.252845324722 },
	{ "1C(C(C(O1)N2C=CC(=NC2=O)NC(=O)C3=CC=CC=C3)OC(=O)C4=CC=CC=C4)O", 0.724467653768 },
	{ "4-ethylidene-33-dimethyl-1-(2-methylpropyl)piperidi", 0.339335673974 },
	{ "RLOKQFFLOZAKDP-UHFFFAOYSA-N", 1.33974512856 },
	{ "HYXRWXBTUMRRSE-UHFFFAOYSA-N", 0.626663184007 },
	{ "1OC2=C(O1)C=C(C(=C2)C=CC(=O)O)OCC3=CSC=C3", 0.815944141809 },
	{ "UNTRHAHOOOZTCW-UHFFFAOYSA-N", 1.57715000796 },
	{ "N-(4-ketoquinazolin-6-ylidene)-35-bis(trifluoromethyl)benzamid", 0.643784264622 },
	{ "NVCHUGOFRYHZAA-UHFFFAOYSA-N", 2.23346901969 },
	{ "NC1CCOC1O", 0.677081765064 },
	{ "25H34N5O3+", 1.6159937833 },
	{ "1(CCCC2(C1CCC3C2=CCC(C3)(C)C=C)C)COC(=O)COCC(=O)O", 1.31382085143 },
	{ "2-phenyl-N-(3-piperidinylmethyl)ethenesulfonamid", 1.33775388733 },
	{ "1-ethylselanyl-4-methoxybenz", 1.7116447133 },
	{ "RIGOOZMDCBCFDS-UHFFFAOYSA-N", 1.92423433199 },
	{ "N1C(=O)C2C(N=C(N2CC3=CC=CC=C3)N4CCCC(C4)N)N(C1=O)CCO", 1.74056538561 },
	{ "3-(35-dichlorophenyl)-N-[3-(hydroxymethyl)phenyl]prop-2-enamid", 2.1121817757 },
	{ "(C)C1=CC=CC=C1N2C3NN=C(N3C4CCCCC4C2=O)SC(C)C(=O)O", 1.59360828042 },
	{ "34H37Cl3N4O6", 0.982783905666 },
	{ "27H32FN4O3S+", 0.771087938808 },
	{ "N-[2-(2-hydroxyethoxy)ethyl]-3-(p-tolyl)acrylamid", 2.36112975166 },
	{ "AWNYJPSFBJHWDG-UHFFFAOYSA-N", 1.75862920485 },
	{ "1(COCCN1C(=O)C(CC2=CC(=C(C=C2)O)O)N", 1.51061369001 },
	{ "(C)C(=O)CC=C(O)O", 1.06341495422 },
	{ "VNGXBBKEAHPKLE-UHFFFAOYSA-N", 1.33737768839 },
	{ "2-(4-chloro-3-methyl-phenoxy)-N&apos;-hydroxy-butanamidi", 1.29871930074 },
	{ "1=C(C=C(O1)CSC2NC(CC(=O)N2)N)C(=O)O", 2.31400004754 },
	{ "3-[4-[(4-ethylthiadiazole-5-carbonyl)amino]phenyl]acrylic acid", 0.661099600479 },
	{ "5-bromanyl-2-[1-(4-fluorophenyl)ethyl]-7-methyl-3H-isoindol-1", 0.232959630624 },
	{ "AOXLWRXIJPYWMB-UHFFFAOYSA-N", 0.375108104099 },
	{ "AMGKELIJHFJULN-UHFFFAOYSA-N", 0.757381625476 },
	{ "zanide;carbanide;(5-hydroxy-4-keto-pentyl)azanide;platinum(4+", 0.786365105531 },
	{ "32H26ClNO4S2", 0.963034123962 },
	{ "OGRSPEGVEKHZEK-SNAWJCMRSA-N", 1.47395925951 },
	{ "SKKVSEFNAIVQSI-UHFFFAOYSA-N", 1.98610001372 },
	{ "BGMJGPOXDFJPKE-UHFFFAOYSA-N", 2.14098581786 },
	{ "2-hydroxyimino-5-methoxy-4-methyl-indan-1", 1.53369767154 },
	{ "1=CC(C2=NC3=NC(=O)N(C=C3OC2=C1)CCC[NH+]=C(N)N)OCC[NH+]=C(N)N", 0.445405170507 },
	{ "14H25NO3S", 0.953077600515 },
	{ "[CH3-].[CH2-]CN([CH2-])C[CH2-].[Y].[Y].[Y].[Y].[W+2]", 2.02621252635 },
	{ "6-keto-N-[4-(134-oxadiazol-2-yl)phenyl]nipecotamid", 0.436334072839 },
	{ "AJJGLIYRBSQLSB-UHFFFAOYSA-N", 0.363572804345 },
	{ "1C(C(C(C1F)CC=CCCCC(=O)OCCO)CSC2=CC(=CC(=C2)Cl)Cl)O", 0.896288530998 },
	{ "17H24N7O14P3S", 0.821763093687 },
	{ "(CC)NC(=O)C1=C(C2C34CCN(C(C3(C1)O)CC5=C4C(=C(C=C5)O)O2)CC6CC6)O", 2.06624656047 },
	{ "KSRXTCLTZOFAPL-UHFFFAOYSA-N", 2.26358515312 },
	{ "51H72N4+2", 1.59721161807 },
	{ "QWAPRRRCESYECV-UHFFFAOYSA-N", 0.369346051657 },
	{ "1=CC=C(C=C1)C2=CC=C(C=C2)CN3C=NC4C3NC(NC4=O)N", 0.314283393284 },
	{ "3-nitrobutylbenzene;yttrium", 0.285788515545 },
	{ "29H33F2N3O2", 1.4372846961 },
	{ "OJQWCWFGPJRF-UHFFFAOYSA-N", 1.24960563755 },
	{ "HYTRVLUBMBSNIH-UHFFFAOYSA-N", 1.68355651326 },
	{ "RRONJSBARJWIK-UHFFFAOYSA-N", 0.740502025337 },
	{ "14H14Br2N2OS", 0.985330682689 },
	{ "YVHHAFATCGAXBO-UHFFFAOYSA-N", 1.09381301202 },
	{ "KEQUAYMLRCZSPR-UHFFFAOYSA-N", 1.81937655975 },
	{ "5-methylquinoxaline-23-di", 1.91721735365 },
	{ "KWQFFCBEIRYGEC-UHFFFAOYSA-L", 0.808534393679 },
	{ "22H30N6OS2", 1.48978598759 },
	{ "carbanide;pentane;tungsten;yttrium", 2.24434077955 },
	{ "WTJBCECFVFKOCU-UHFFFAOYSA-N", 0.705955717933 },
	{ "WWIWBDJZTVAKRI-UHFFFAOYSA-N", 0.755048185299 },
	{ "methanediol;ruthenium", 0.712439043528 },
	{ "(C)CCC(C)NCC1=NC(=O)C2C=CC=CC2=N1", 2.34669246333 },
	{ "WQPWZQXDKYHNDE-UHFFFAOYSA-N", 0.408598795457 },
	{ "WJYBXGDJXBQVMF-UHFFFAOYSA-N", 1.82047610958 },
	{ "1C2CNC1CN(C2)C(=O)NC3=CC=C(C=C3)F", 1.97570327882 },
	{ "UWHUKCVWQASMSK-UHFFFAOYSA-N", 0.809852900063 },
	{ "YUHWEKQOXIYNBU-UHFFFAOYSA-N", 0.750529562495 },
	{ "1-(1-hydroxyethenyl)-3-pyrrolidi", 2.3166988905 },
	{ "SIRCCOIJQOYRKA-UHFFFAOYSA-N", 0.841088764956 },
	{ "1=CC(=CC(=C1)C(=O)OC)C=NO", 2.28425276836 },
	{ "6H10O3S", 0.134071042238 },
	{ "HHEZMGQNHYQIDE-UHFFFAOYSA-N", 0.750705996842 },
	{ "HLJPVMVUBWSG-UHFFFAOYSA-N", 1.07228690504 },
	{ "2-amino-3-(1H-benzimidazol-2-yl)propionaldehyd", 1.13594502535 },
	{ "ZRAGQFEJOBXSV-UHFFFAOYSA-N", 0.917207360092 },
	{ "XTUXSKOUONZGFS-UHFFFAOYSA-N", 2.36268100394 },
	{ "(C(C)(C)C)NC(=O)OCC1=NC(=CC=C1)CO", 1.05652721393 },
	{ "1CC2C(CC1C(=O)NC3=CC=CC=C3)NC(=O)C4C2SC=C4", 1.9992777273 },
	{ "1(OCC(O1)C2C(C(=O)N2C3=CC=C(C=C3)OC)NC(=O)CC4=CC=CC=C4", 0.635385029098 },
	{ "UDGBLPGLFVKWGZ-UHFFFAOYSA-N", 1.87013579239 },
	{ "1=CC=CC2=C1CC(C2OC3=CC=C(C=C3)S(=O)(=O)N)N4CCCC4", 2.36003431818 },
	{ "[CH2-]CC1=CC(=CC=[C-]1)C2=NC=C3C=C(C=CC3=N2)N.[Y]", 1.13197740597 },
	{ "AIWCJQGHRBJHGS-UHFFFAOYSA-N", 2.13047184985 },
	{ "NUOOXQCOFKON-UHFFFAOYSA-N", 0.589506011077 },
	{ "5-[256-tris(oxidanylidene)pyridin-3-yl]pyridine-236-tri", 0.87398313593 },
	{ "methyl 2-[3-(5-bromo-2-methoxy-phenyl)prop-2-enoylamino]prop", 0.983811343787 },
	{ "HYDFEEMMGNZDFD-UHFFFAOYSA-N", 1.45327051238 },
	{ "3-(2-norbornyl)-2-thioxo-4aH-thieno[23-d]pyrimidin-4", 1.92748911404 },
	{ "N-(7-amino-3-oxo-23a-dihydroindazol-4-yl)acetamid", 1.85613507981 },
	{ "MWLHZKKFGYYANK-UHFFFAOYSA-N", 1.2655031343 },
	{ "MCTYRVIRQFVCL-UHFFFAOYSA-N", 1.52822466469 },
	{ "OPQYOAHMYQMPEA-UHFFFAOYSA-N", 1.45329487177 },
	{ "FKENTXFPMOBZHH-UHFFFAOYSA-N", 1.18528357558 },
	{ "UIMFFIUABUAHQH-UHFFFAOYSA-N", 0.203848115477 },
	{ "66H93NO9", 1.35094251273 },
	{ "SYWBCKWHCGBQBW-UHFFFAOYSA-N", 0.376498757942 },
	{ "MDUPPIVEPMOLP-UHFFFAOYSA-N", 1.93655182299 },
	{ "hI=1S/C7H12O4/c1-4-7(210)6(9)5(3-8)11-4/h14-68-10H3H22H3", 1.87215966414 },
	{ "(C)CC1=NC(=CS1)C2=CC(=O)C3=CC=C(C(C3=N2)Cl)OCC(C)O", 1.77107004178 },
	{ "(C)(C)CNC1=CC2=NC(=O)N=C2C=C1Br", 0.237897461376 },
	{ "31H61NO", 0.464261004852 },
	{ "SC1=NC(=NC2=C1NN=C2)S", 2.07969210869 },
	{ "BMUPBOKSBBFPTA-UHFFFAOYSA-N", 1.07444978208 },
	{ "bis(chloranyl)platinum;8-(1-butylbenzimidazol-2-yl)quinoli", 2.00339464731 },
	{ "(C)N(CC1=CC=C(C=C1)C(=NO)N)C(", 2.36801893001 },
	{ "32H36ClF3N2O4SSi", 1.2239170084 },
	{ "SC1=NC=CC(=C1)C(=O)N2CCN(CC2)CC=CC3=CC=CC=C3", 1.2656688209 },
	{ "38H38N4O2", 1.08539466215 },
	{ "HWPDWJGHBQKYQU-UHFFFAOYSA-N", 1.00071778272 },
	{ "HLUJMHOJEJEXTN-UHFFFAOYSA-N", 1.49006311604 },
	{ "zanide;uranium;vanadium;dihydr", 0.701458359492 },
	{ "1=CC(=CC=C1C(=NO)N)C(=O)NCCC2=CC=NC=C2", 1.81376839335 },
	{ "[Se]C1=C(C=C(C=C1)OC)O", 0.249054177892 },
	{ "NPVSIBYCEHXOHU-UHFFFAOYSA-N", 1.65154510597 },
	{ "23H17Li4N5O13S4", 0.689522480135 },
	{ "XAYUFQJHPEKDT-UHFFFAOYSA-N", 2.38175056934 },
	{ "(C)(C)C(C(=O)NC1=CC(=C(C=C1)F)C(=O)OC)N", 0.322345696882 },
	{ "[B]C1CC(C(O1)CC)OP(OCC[N+]#[C-])OCCOP(N(C(C)C)C(C)C)OCCC#N", 2.11100393797 },
	{ "SZJMLOVUXQRVAK-UHFFFAOYSA-N", 0.582667905214 },
	{ "LEBQWPANVJZKQE-UHFFFAOYSA-N", 0.253463615357 },
	{ "95H120N2O24", 0.730062539279 },
	{ "HTMMCGMSEMOECN-UHFFFAOYSA-N", 1.72347021873 },
	{ "1-(1-butylsulfonylpropan-2-yl)-2-ethynylbenz", 0.615841928987 },
	{ "methyl 4-[3-(heptan-2-ylamino)-3-oxoprop-1-enyl]benz", 2.02712911241 },
	{ "7H13N3OS", 1.42502061159 },
	{ "(3-bromo-4-but-2-enoxy-5-ethoxy-benzyl)-isopropyl-ami", 0.91347853011 },
	{ "VOWLGOKHXDSTGG-UHFFFAOYSA-N", 0.331418292359 },
	{ "MZEDEPHLLQRYRD-UHFFFAOYSA-N", 0.819594993771 },
	{ "(C=CC1C(CC(C1CCCCCCC(=O)OC)Cl)OC2CCCCO2)OC3CCCCO3", 0.332355959638 },
	{ "1CC(OC1)CN(CC2=CC=CC=C2)C(=O)C=CC3=CC=CC=C3", 0.224486282884 },
	{ "MPXCSGXLIPSZNF-UHFFFAOYSA-N", 1.03314298685 },
	{ "4-iminophosphinylbut-2-enoic acid ethyl ester", 0.493611922791 },
	{ "19H17N6O3+", 0.33642082738 },
	{ "(CCC1=CC=CC=C1)C2CC=C3C2(CCC4=C3CCC5C4(CCC(C5(C)C)(CCO)O", 0.981681323577 },
	{ "28H26N2+2", 2.16782209114 },
	{ "2-(2-fluoranyl-4-octan-2-yloxy-phenyl)-5-hexyl-pyridi", 1.94209287696 },
	{ "JRXZMAKJAMMTOD-UHFFFAOYSA-N", 1.75235087347 },
	{ "21H24F2N2O3S", 1.60087331132 },
	{ "1=CC=C(C(=C1)CNC(=O)C2C=NC(=O)C=N2)C#CCO", 1.832207559 },
	{ "NTLQWXJIFWGXGT-UHFFFAOYSA-N", 1.64359784757 },
	{ "9-[(3-methoxyphenyl)amino]-3aH-furo[32-c][18]naphthyridin-4", 2.22364708714 },
	{ "RQWYLPGGSKOF-UHFFFAOYSA-N", 1.11665637956 },
	{ "KABSHPGELIXHEV-UHFFFAOYSA-N", 1.96689764072 },
	{ "HIOCBQEYIMIIV-UHFFFAOYSA-N", 0.257509412717 },
	{ "[N]", 1.01052583943 },
	{ "VCQBPEHDXIFH-UHFFFAOYSA-N", 0.678606718734 },
	{ "XWUDBHOGFMNMI-UHFFFAOYSA-N", 0.298405962255 },
	{ "OMRIENGHWZWSEU-UHFFFAOYSA-N", 1.34434081512 },
	{ "JYWWLKLDGCZHGK-UHFFFAOYSA-N", 2.0238461655 },
	{ "ZEXUHICIZHQZCZ-UHFFFAOYSA-N", 1.02278851777 },
	{ "76H103N21O24S", 2.20109718237 },
	{ "=CCOCC1CC2=C(C3N1C(=O)CC(C3)C=C)N(C4=CC=CC=C24)CC5=CC=CC=C5", 0.89565368936 },
	{ "4-(N-pyrazolidin-4-ylsulfonylanilino)butyric acid", 0.158714155994 },
	{ "21H22F3NO5", 0.559079694685 },
	{ "NWADNIVWEDNDLL-UHFFFAOYSA-N", 1.62421480227 },
	{ "[N+]1=C2C=CC=C(C2=[N+](C3=CC=CC=C31)CC4=CC=CC=C4)C5=CC=CC=C5", 2.13586703358 },
	{ "44H51NO11", 2.2979516943 },
	{ "1-(2-methanoylpyrrolidin-1-yl)piperidine-4-carboxylic acid", 2.14160770587 },
	{ "FRJCGPBONCASO-UHFFFAOYSA-N", 1.37764140435 },
	{ "(C1C(NC1=O)C(C)C(=C(C(=O)OCC=C)[N+]#N)O)OC(=O)OCC=", 1.04912924821 },
	{ "14H23NO4S", 0.685804740173 },
	{ "N(C)C=CNC1=NC=CC(=C1)C(=O)N(C)O", 0.330720466808 },
	{ "4-chloro-2-ethyl-6-methyl-thiazolo[45-c]pyridi", 0.987761300443 },
	{ "1=CC(=CNC2=CC=CC=C2)C(=O)C(=C1)C3CCCCCCCCCCC3.Cl[Ti]", 2.14827873121 },
	{ "QIUKRPAMRKUFLL-UHFFFAOYSA-N", 1.7403178731 },
	{ "9H13N5OS2", 1.54884840937 },
	{ "3-amino-N-butyl-3-hydroxyimino-2-methylpropanamid", 1.86756689293 },
	{ "ZGXPDOYBMDTNDH-UHFFFAOYSA-N", 2.22384979513 },
	{ "QQRLRDFHINTCFL-UHFFFAOYSA-N", 1.42865075118 },
	{ "23H32Cl2N2O", 1.70205805141 },
	{ "YLWVWACYCVFODW-UHFFFAOYSA-N", 2.25441038371 },
	{ "cta-13-di", 2.36837257726 },
	{ "26H25ClF3N7O2", 1.50019115961 },
	{ "1=CC(=CC=C1C2=CC=C(O2)C=NN(CCO)C(=S)N)OC(F)(F)F", 0.118852288699 },
	{ "OC1CNCC1NC(=O)NC2=CC(=CC=C2)O", 1.44323242125 },
	{ "WFYCCTPKNAKQSJ-UHFFFAOYSA-N", 1.34324889867 },
	{ "FDPHTNAXUNFUFV-UHFFFAOYSA-N", 2.2872389553 },
	{ "QPVYTSLCCTSMX-UHFFFAOYSA-N", 1.85681303474 },
	{ "5-ethyl-1-fluoranyl-67-dihydrocyclopenta[c]pyridine-57-di", 0.984965120144 },
	{ "1CC(CN(C1)CC2=CC=CC(=C2)C=CC(=O)O)CO", 0.301079860487 },
	{ "ZXJIGFVZVHOKMM-UHFFFAOYSA-N", 1.15725100356 },
	{ "5-chloranyl-3-phenyl-1H-indole-2-carbaldehyd", 2.23770489663 },
	{ "BRNWRMYPQLLLLM-UHFFFAOYSA-N", 0.46463595551 },
	{ "15H22N2O3", 0.8219987923 },
	{ "6-(6-keto-2-naphthylidene)pipecolinic acid methyl ester", 0.687811192638 },
	{ "29H34O3", 1.9996658865 },
	{ "ZOEHPCCYASHQQF-UHFFFAOYSA-N", 2.24240006202 },
	{ "1=C(NC(=C1C(=O)NCCCCCCCC(=O)CO)C)C=C2C3=C(C=CC(=C3)F)NC2=O", 1.32939589381 },
	{ "YVWKEFMOSCSPBE-UHFFFAOYSA-N", 2.32336181607 },
	{ "ALBKDFOFWPJMEN-UHFFFAOYSA-N", 0.696348194241 },
	{ "22H22ClFN2O7S", 0.298584439901 },
	{ "1CC(C(O1)COC)OP(=O)(O)OCC2C(CC(O2)C)OP(=O)(O)O", 0.809941205956 },
	{ "JIQGPOZXYKOGGE-UHFFFAOYSA-N", 0.438701782671 },
	{ "5-(2-iodanylprop-1-enyl)-123-trimethoxy-benz", 2.14200885052 },
	{ "24H24N2O3", 1.55176769058 },
	{ "29H32N6O4S", 0.723350173641 },
	{ "ZIBBQXKOPKKXOC-UHFFFAOYSA-N", 0.825871880176 },
	{ "2-amino-3-(3-methylimidazol-4-yl)propanoyl chlorid", 1.67875000496 },
	{ "48H74N10O13", 1.66643007522 },
	{ "(C)CC(=O)CC1C(=C(CC2=CC(=C(C=C2)F)Br)N=O)NON1", 1.22421932428 },
	{ "(C)C(=O)OC1(CCC2C1(CC(C3(C2CC(C4=CC(=O)C=CC43C)F)F)O)C)C(=O)CO", 1.72418755731 },
	{ "31H48FN3O6", 0.872126536636 },
	{ "S(=O)(=O)NC1CC2CCC(C1)N2CCC3=CC=C(C=C3)OC4=NC5=C(S4)C=CC=C5", 1.98286633189 },
	{ "OC(=O)CCCCC1CC2C(O1)CC(C2C=CC(CCC3CCCCC3)O)O", 1.20379239988 },
	{ "31H51N7O4", 1.01756735556 },
	{ "BRAEXQRLRHOFR-UHFFFAOYSA-N", 0.156567598669 },
	{ "1CCN(C1)C(=O)C2CSCN2C(=O)C=CC3=CC=C(C=C3", 2.14331970726 },
	{ "4-(4-chlorobutyl)-133a466a-hexahydrothien[34-d]imidazol-2", 1.52996782417 },
	{ "OMMQIBWUDIJUOL-UHFFFAOYSA-N", 0.193544247518 },
	{ "2-(5-methyl-24-dioxo-13-diazinan-1-yl)-4-pentenoic acid", 1.11757649535 },
	{ "GXDXADKSIBLBOJ-UHFFFAOYSA-N", 1.61637379506 },
	{ "23-dihydro-1H-indol-5-yl meth", 1.95613976966 },
	{ "1=C(C=CC(=C1)C2=NC(=O)C3C=CC=NC3=C2)N4CCCCC4", 1.91812990184 },
	{ "24H36NO2+", 2.11162567932 },
	{ "(2-phenyl-6-vinyl-4-piperidyl)meth", 2.11136202625 },
	{ "KTMKPDJTHVXTG-UHFFFAOYSA-N", 0.293089471144 },
	{ "WHBSATDWYBWIDO-UHFFFAOYSA-N", 0.285199082519 },
	{ "FIESKNOKHUILKJ-UHFFFAOYSA-O", 0.509794101562 },
	{ "FJJQBASGHGROEI-UHFFFAOYSA-N", 0.140316592937 },
	{ "5-but-2-enyl-4-hydroxy-6-(3-hydroxybut-1-enyl)-2-ox", 2.26811643971 },
	{ "N-nonyl-4-oxidanylidene-1H-phthalazine-1-carboxamid", 2.10636942888 },
	{ "16-cyclohexyl-613-dihydroxyhexadeca-911-dien-715-diyn-2", 1.15186552205 },
	{ "1CCN(CC1)S(=O)(=O)C(=CNC2=NC=CS2)C(=O)C3=C(N=C(C=C3", 0.416696870966 },
	{ "2-[chloro-(2-fluoro-4-nitrophenyl)methylidene]butanenitri", 1.82472186992 },
	{ "4-acetyloxy-3-amino-2-butenoic acid ethyl ester", 1.10363505659 },
	{ "40H59BrN4O6", 2.07425894793 },
	{ "QPUXOZBOXLKAPG-UHFFFAOYSA-L", 1.98276599457 },
	{ "1-butyryl-N-(2-pyrrol-1-ylphenyl)pyrrolidine-2-carboxamid", 2.25650362236 },
	{ "MUXNKEZMSHYHII-UHFFFAOYSA-N", 1.28715872781 },
	{ "GFVIVJCTQDYDK-UHFFFAOYSA-N", 0.193039769392 },
	{ "SBNSPFGSMMMXSG-UHFFFAOYSA-N", 0.558617321008 },
	{ "OCCN(CCOC)C1=CC=CC=C1C=CC(=O)O", 0.107493670004 },
	{ "1=NC(=C(C=C1)C(=NO)N)N(C)CC(COC)O", 2.31124500241 },
	{ "methyl 5-cyclohexyl-3-ethyl-5-oxo-pent-2", 1.78323781324 },
	{ "2-[bis(phenylmethyl)amino]-3-(3-isocyanophenyl)-1-prop", 1.11360363365 },
	{ "OIJKTXQXZFTFF-UHFFFAOYSA-N", 0.793323183277 },
	{ "methyl 2-[5-chloro-2-(oxiran-2-ylmethoxy)phenyl]ac", 0.792751982508 },
	{ "GHGUPSVORLURQV-UHFFFAOYSA-N", 2.0809467356 },
	{ "11H16FNO", 2.26616908342 },
	{ "34H36ClF2IrN4", 1.00131900029 },
	{ "2-morpholin-4-yl-N-(3H-pyrazolo[34-b]pyridin-3-yl)ethanamid", 2.06376714892 },
	{ "WIMISFLAIVQMKS-UHFFFAOYSA-N", 1.12260409902 },
	{ "1CCC(C(C1)OC(=O)C23CCCCCN2C=C(C=C3)C(=O)OC)C(C)(C)C4=CC=CC=C4", 1.58805693505 },
	{ "2-(2-piperidylmethylsulfanyl)hexahydropyrimidin-4", 0.518888333404 },
	{ "26H24ClN5O6", 1.22267074534 },
	{ "OC1=CC(=O)C(=CNNC(=O)C2=CC(=CC(=C2)C(F)(F)F)C(F)(F)F)C=C1", 1.17937545024 },
	{ "HNCDUZKOQCZJNB-UHFFFAOYSA-N", 0.141491420072 },
	{ "FNRAQTWFIOEOLK-UHFFFAOYSA-N", 0.534572169568 },
	{ "FEEHKKPCOGNDJE-UHFFFAOYSA-N", 1.21692557846 },
	{ "hyl 4-(4-amino-2-formyl-33-dimethyl-5-oxopentyl)benz", 1.55775624126 },
	{ "3-hydroxyimino-1-(2-phenoxyethyl)-6-thiophen-3-ylindol-2", 1.84908607442 },
	{ "MGUALGQCSWBCCR-UHFFFAOYSA-N", 2.26121835125 },
	{ "1-(difluoromethoxy)-2-ethoxy-4-(2-nitrobut-1-enyl)benz", 0.705442877758 },
	{ "ZRRFXVXNQNALIE-UHFFFAOYSA-N", 1.26481737875 },
	{ "22H23FN2O2", 1.06074431806 },
	{ "LHHDDZXCLGWOLO-UHFFFAOYSA-N", 0.977520460539 },
	{ "KFBHZIJETSXZDZ-UHFFFAOYSA-N", 0.206881927537 },
	{ "N-(2-azanyl-2-hydroxyimino-ethyl)-N-propyl-butanamid", 1.65713164807 },
	{ "N-(1133-tetramethylbutyl)indoline-2-carboxamid", 1.44766361471 },
	{ "KRWRVRFLODGUCT-UHFFFAOYSA-N", 2.16409473552 },
	{ "WOPVXGLZXFYDTG-UHFFFAOYSA-N", 1.64195092125 },
	{ "23H25N4O2+", 0.842105608863 },
	{ "QKCLAFLJOISDII-UHFFFAOYSA-O", 0.22172557434 },
	{ "JTJUOHCLANMBST-UHFFFAOYSA-N", 1.7539241664 },
	{ "(C)C1C(N(C1=O)CC(=O)OCOC(=O)C)C(C)C(=O)SC2=CC=C(C=C2", 2.17952292328 },
	{ "BTBUXRYRWIVKI-UHFFFAOYSA-N", 0.314996794016 },
	{ "43H73N11O9S", 0.783679016705 },
	{ "NC1=NC(=CC(=N1)N2CCCC2C(F)(F)F)C3=CC4=C(C=C3)C(=NN4)N", 0.35695954145 },
	{ "RBOQZZYGDMLJOG-UHFFFAOYSA-N", 0.571440323777 },
	{ "NCCPBIKAUHYMTR-UHFFFAOYSA-M", 2.29992909659 },
	{ "3-methoxy-4-piperidin-4-ylidene-cyclohexa-25-dien-1", 2.27245204216 },
	{ "MBXXFOOGOVVRIG-UHFFFAOYSA-N", 1.94808278791 },
	{ "VOKJZADZBOTFCL-UHFFFAOYSA-N", 1.77570048322 },
	{ "3-[3-fluoro-4-(furan-2-carbonylamino)phenyl]prop-2-enoic acid", 1.25078906329 },
	{ "TWVMKJPTGOMFLK-UHFFFAOYSA-N", 0.332106524161 },
	{ "SHVRBWHLQZSIF-UHFFFAOYSA-N", 2.15412199331 },
	{ "KJALDVXKXAMWTP-UHFFFAOYSA-N", 1.46963726207 },
	{ "BJTXGBSWJIZLBQ-UHFFFAOYSA-N", 1.93457641493 },
	{ "OIOJSZFIAGVALK-UHFFFAOYSA-N", 0.284194821885 },
	{ "WXOJVQLVVRFMIU-UHFFFAOYSA-N", 1.851982318 },
	{ "BJNJBOLCNOUGHV-UHFFFAOYSA-N", 1.96727009872 },
	{ "1=NC(=NC=C1C(=O)N(CC(C)C)C2CC(CNC2)C(=O)N3CCOCC3)C(C)(", 0.44872994032 },
	{ "4-pentynoic acid (1-aminoethylideneamino) ester", 0.801324769336 },
	{ "1CC(=O)NC(N1)SC(C)C(=O)NC2=C(N(N=C2", 1.95658841656 },
	{ "3-[2-[(N-methylanilino)methyl]phenyl]prop-2-enoic acid", 1.80774451886 },
	{ "1CN(CCC1C2=CC=CC=C2)C3CNCC34CNCC5=CC=CC=C45", 0.274442410454 },
	{ "2-(4-methoxyphenyl)-3-methyl-chroman-4", 0.140310619952 },
	{ "FVQQLPPZUIJOBT-UHFFFAOYSA-M", 1.66735127367 },
	{ "2-(4-methoxy-1H-indol-5-yl)ethylami", 2.38693550747 },
	{ "UNQNQDBUZLGRB-UHFFFAOYSA-N", 1.54520028104 },
	{ "1C2CC3=C(C1(CCN2CC4CC4)C)C=C(C=C3)C(=NO)N", 1.08558916917 },
	{ "(C)CN(C(C1=CC=CC=C1)C(=O)OC2CCCC2)C(=O)CC(CC=C)C(=O)NO", 2.0601598334 },
	{ "2-decenoic acid 2-(dimethylamino)ethyl ester", 0.829157014206 },
	{ "N&apos;-hydroxy-5-[4-(2-pyridyl)piperazin-1-yl]pentanamidi", 2.07347124553 },
	{ "2-amino-1-(4-methoxy-1-piperidinyl)-33-dimethyl-1-b", 2.05464420078 },
	{ "1-phenylhexa-135-trienylphosphonic acid", 1.28069827879 },
	{ "JTSHVVJDGOZTA-UHFFFAOYSA-N", 0.783024858502 },
	{ "23H32ClN3O5", 2.38737150775 },
	{ "(=O)OC(COC(=O)CCCCCCCC=CCCCCCCCCC)COC(=O)CCCCCCC=CCC=CCC=", 2.23807302187 },
	{ "15-bis[4-(trifluoromethyl)phenyl]penta-14-dien-3-ylidenediaz", 2.31929266285 },
	{ "methyl 4-chloro-3-[3-(5-methylfuran-2-yl)prop-2-enoylamino]benz", 0.274642612709 },
	{ "GJFUTLFZMGBIU-PJDGMUFKSA-N", 1.95680796018 },
	{ "4-[3-(4-mesylphenoxy)-5-methyl-phenyl]-2-piperid", 1.76660520947 },
	{ "28H40N4O4", 1.82814620028 },
	{ "3-(2-methylpyrazol-3-yl)-N-propylprop-2-enamid", 1.05924587878 },
	{ "N(CC#N)C(=O)N1CCC(C(C1)F)NC2=CC(=NC=C2C(=O)N)CC3=CC=CC=N3", 2.36380702077 },
	{ "EYKEINRFTZZGT-UHFFFAOYSA-N", 1.31758016636 },
	{ "16H12F3NO2S", 1.88354292281 },
	{ "1=C(C=C(C(=C1Br)O)I)C2=C(C(=C(C=C2CC(C(=O)O)N)I)O", 2.00055696646 },
	{ "1=CC(=C(C=C1C=CC(=O)O)F)NC(=O)CN2C=NN=N2", 1.8035964695 },
	{ "UBMNYXYUGUBVIP-UHFFFAOYSA-N", 1.73221664122 },
	{ "4-fluoro-3-hexyl-2-(phenoxymethyl)cyclopentanone;yttrium", 2.17504144632 },
	{ "N-(9-methyl-6-oxo-5H-purin-2-yl)-N-nitroso-nitramid", 0.104149724185 },
	{ "FMBQEOYMKIHGBD-UHFFFAOYSA-N", 1.638368988 },
	{ "YXLDTTXECQHOS-UHFFFAOYSA-N", 1.68652117437 },
	{ "1CN(CCN1CC2=CC=CC=C2Cl)C3=CC=C(C=C3)C=NNC(=S)N", 1.85920371217 },
	{ "3-oxidanyl-2-[tri(propan-2-yl)silyloxymethyl]cyclobutan-1", 1.45557564151 },
	{ "23H20Ir-2", 2.12938605594 },
	{ "LSVOYSVXYIDFAS-UHFFFAOYSA-N", 1.18378405011 },
	{ "QITXKIWNALQXQY-UHFFFAOYSA-N", 1.4871102296 },
	{ "AUWBXAMGSJMWIO-UHFFFAOYSA-N", 0.578183627882 },
	{ "15H18BrN3O2S", 1.29603229094 },
	{ "HUMOAXFGCGHDL-UHFFFAOYSA-N", 0.707008416709 },
	{ "WULJIJDFRXVHOB-UHFFFAOYSA-N", 2.20750482628 },
	{ "27H25N6O14S+", 0.975350347229 },
	{ "3-amino-3-phenyl-N-pyrazolidin-3-yl-propionamid", 1.09733454694 },
	{ "copper;O1-ethyl O4-ethynyl 3-methyl-2-oxidanyl-but-2-enedi", 1.3920386017 },
	{ "17H18ClF3N2O3", 1.82002619384 },
	{ "2-(3-bromobenzene-6-id-1-yl)-4-propyl-pyridine;palladium", 0.142597995582 },
	{ "23H26ClN9O4S", 2.12156626396 },
	{ "1=CC(=C(C(=C1)C)S(=O)(=O)NC(C)COC2=C(C=CC(=C2)F)C(=O)N", 0.341856566141 },
	{ "YYYXIZVKSZLEEX-UHFFFAOYSA-N", 2.01137708127 },
	{ "MJVYNVNWUWDBDS-UHFFFAOYSA-N", 1.63796798606 },
	{ "SIDUUUCWDVASQI-UHFFFAOYSA-N", 0.357764829801 },
	{ "15H14F3N3O2", 1.60670236845 },
	{ "20H24F2N4O7", 1.27713873895 },
	{ "TUPZROSXWXWXRL-UHFFFAOYSA-N", 2.06629652145 },
	{ "KQGCUNKVTNXVJV-UHFFFAOYSA-N", 0.889584725548 },
	{ "VTQQYITWBIHBPG-UHFFFAOYSA-N", 0.363800808718 },
	{ "SYKKOSGXZYHI-UHFFFAOYSA-N", 1.74134109046 },
	{ "YSSOVNJVXMUAR-UHFFFAOYSA-N", 1.82396002063 },
	{ "RWCXOUULXHNJMZ-UHFFFAOYSA-N", 1.98371128296 },
	{ "N-[3-hydroxy-1-(1-hydroxyethyl)-2-oxo-propyl]-4-propyl-benzamid", 1.06416224264 },
	{ "1N=C2C(=C(N1C)SCC(=O)NC3=CC(=CC=C3)[N+](=O)[O-])C=C(S2", 2.36365127746 },
	{ "(C)(C)C1=NC=C2C(=C1)C3=C(CO2)N=C(S3)CC(=O)N4CCC(C4C(=O)N)CN(", 1.26446515269 },
	{ "YGGMVPXLBIMHJ-UHFFFAOYSA-N", 1.13689621496 },
	{ "OKSURNYFSLHZPN-UHFFFAOYSA-M", 2.309296595 },
	{ "OWHIYOUYPAQABF-UHFFFAOYSA-N", 1.68799043503 },
	{ "N1C(=NN=C1OC2=CC=CC(=C2)N(C)C)C(C)NS(=O)(=O)C3=CC(=C(C=C3", 0.398911318164 },
	{ "1=NN(C(=C1C=CC(=O)O)N(C)C(C)CC(", 0.958163887858 },
	{ "3-[[but-2-enyl(methyl)amino]methyl]benzonitri", 2.11697209761 },
	{ "36H46O6", 2.27222834486 },
	{ "(C1=NNN=N1)NC(=O)C2=CN=CC(=C2)C=CC(=O)O", 0.488222858299 },
	{ "30H34O5", 1.19006953464 },
	{ "MDSNJNUGLGASC-UHFFFAOYSA-N", 1.05118980604 },
	{ "(=CCCC(=CCCC(=CCSCC(C1CNNN1)NC2=NC=NC=C2", 1.84558844997 },
	{ "GXOCAIHEGSFKS-UHFFFAOYSA-N", 2.11149785407 },
	{ "HBZYGCFQYNVQJ-UHFFFAOYSA-N", 1.07592454051 },
	{ "29H46ClN3O3", 1.92054188469 },
	{ "17H20ClFN2O3", 1.83205518148 },
	{ "FKVGIYMKUMZCV-UHFFFAOYSA-N", 2.10268880416 },
	{ "2-chloranyl-7-ethyl-5-methyl-8-(oxan-4-yl)-7H-pteridin-6", 2.31156973 },
	{ "rt-butyl 2-(2-acetamidothiazol-4-yl)pyrrolidine-1-carboxy", 1.538707397 },
	{ "20H26ClN5O4", 1.48025508812 },
	{ "GTDJMLODZDUWSZ-UHFFFAOYSA-N", 2.04087292094 },
	{ "QMEDDXYQMIUCAN-UHFFFAOYSA-N", 2.35615479684 },
	{ "4-bromo-N-(2-oxo-5-benzimidazolyl)-1H-pyrrole-2-carboxamid", 1.78124909355 },
	{ "1=CC=CC=C1CC(CC(=O)C2=CC=C(S2)C3=CC=NN3C)CN", 1.11741589307 },
	{ "SJSTWVKSIGKWNI-UHFFFAOYSA-N", 1.52541645626 },
	{ "RQEQRYYCMVZZGM-UHFFFAOYSA-N", 2.3085838139 },
	{ "1CCN(C1)C(=O)C=CC2=CC=C(C=C2)O", 1.66141598205 },
	{ "(C)(C)NC(=O)N1CCC(C1)ON=[N+](N(C)C(C)(C)C)[O-]", 0.284524751445 },
	{ "FSJQVJLWSOAGKA-UHFFFAOYSA-N", 0.57420666781 },
	{ "AQXDBJDCORAURP-UHFFFAOYSA-N", 2.10491791841 },
	{ "15H18O2", 1.52277365118 },
	{ "40H58N2NiO2", 2.20200891694 },
	{ "RPHJETWQHGLZRF-UHFFFAOYSA-N", 0.170470273553 },
	{ "ZRWCFLWRKGGMPY-UHFFFAOYSA-N", 1.84567779259 },
	{ "33H49F3N6O4SSi2", 1.31721917425 },
	{ "(3-dodecanoyloxy-2-pentadec-9-enoyloxy-propyl) nonadec-9", 1.07968915573 },
	{ "2-amino-N-[4-(cyanomethyl)phenyl]-33-dimethyl-butanamid", 1.54415873013 },
	{ "2-[esyl(methyl)amino]-N&apos;-hydroxy-acetamidi", 1.26589323804 },
	{ "24H32ClN3O7", 1.6589253218 },
	{ "hI=1S/C6H6N4O/c1-4-2-5-6(3-7-8-5)9-10(4)11/h2-311H1H3", 0.98592704964 },
	{ "25H24Cl2F5NO3", 1.87920927625 },
	{ "YWJXNNGGIFJXHI-UHFFFAOYSA-N", 1.46159489422 },
	{ "RDBDZSBJQUPVJO-UHFFFAOYSA-N", 1.772456729 },
	{ "4-[3-(2-oxazolyl)propyl]-3-(4-phenylphenyl)-2-thiazolidi", 0.18345197159 },
	{ "OC1=NC(=O)C2C1CCC2", 1.58159635771 },
	{ "RPJOUWPMKSEMLV-UHFFFAOYSA-N", 2.29772001952 },
	{ "1-(4-methyl-3-oxo-pent-4-enyl)-3-(6-methyl-4-oxo-2-piperidyl)ur", 1.96407841661 },
	{ "NQZPJNMNQIHNEA-UHFFFAOYSA-N", 2.04782755727 },
	{ "1=C(C=CC(=C1)C(=NO)N)N(C)CC2=CC=CC=C2F", 1.62486355171 },
	{ "7-bromo-9aH-[12]oxazolo[34-c]quinolin-4", 0.829301831385 },
	{ "2-(aminocarbohydroximoyl)-N-(23-dichlorophenyl)butyramid", 0.130183942478 },
	{ "TTUWZDLVWKTG-UHFFFAOYSA-N", 0.830645625993 },
	{ "rt-butyl N-allyl-N-[3-(3-hydroxypropyl)cyclohexyl]carbam", 1.6793495278 },
	{ "OVKBVTGGFLXDGH-UHFFFAOYSA-N", 2.1587730566 },
	{ "MMNVBKSFKZWEW-UHFFFAOYSA-N", 1.01882076129 },
	{ "HQHKCYQAKIHFNI-UHFFFAOYSA-N", 0.925463962302 },
	{ "N-pentylindoline-2-carboxamid", 1.96937484772 },
	{ "NBINJHSWCHBMGI-UHFFFAOYSA-N", 1.47857736627 },
	{ "RXBIZKBKYORCJV-UHFFFAOYSA-N", 1.91284758357 },
	{ "SMSKYSIBSAPCKG-UHFFFAOYSA-N", 1.52111490426 },
	{ "27H26Cl3F3N4O5", 0.697831448564 },
	{ "2-butan-2-ylthiophen-3-ami", 0.962444835406 },
	{ "2-chloranyl-111-tris(fluoranyl)but-2", 0.317351735564 },
	{ "26H30ClN7O4", 1.11499037968 },
	{ "HEHSSTXFCPUMR-UHFFFAOYSA-N", 1.50854406374 },
	{ "1CCC[N+]1(C(=O)CC(C(F)(F)F)SC(=O)C)C(=O)O", 1.61521686329 },
	{ "22H22FN2O2S+", 1.72089736234 },
	{ "35H45N5O2", 1.72266718211 },
	{ "3-(3-chloro-4-isopropyl-5-methyl-phenyl)acrylonitri", 2.07499991697 },
	{ "1=C(C=CC(=C1)NC(=O)C(C)(COC2=CC(=C(C=C2)C#N)F)O)[N+]#[C-]", 0.830929274533 },
	{ "ACJVKZZMZWFAE-UHFFFAOYSA-N", 2.14082405911 },
	{ "1=CC(=C(C=C1C#CCO)CNC(=O)C2C=NC(=O)N2)F", 2.06429964886 },
	{ "JDKGFXRNSOGFQZ-UHFFFAOYSA-N", 1.90764363136 },
	{ "6-chloranyl-4-ethoxy-1H-imidazo[45-c]pyridi", 1.26484744988 },
	{ "OC(=O)C1CSCCN1C(=O)C2C=CC(=O)N=C2", 0.870686136876 },
	{ "7-iodo-4a56788a-hexahydro-1H-quinazoline-24-qui", 1.22978083534 },
	{ "41H61F3N4O6S", 1.39520250673 },
	{ "hI=1S/C2H5.4CH3.2Y/c1-2;;;;;;/h1H22H3;4*1H3;;/q5*-1;;+3", 2.25267785587 },
	{ "VOBKNBIKPALJ-UHFFFAOYSA-N", 1.57136821103 },
	{ "1-butyl-3-(6-oxo-1233a4577a-octahydroindazol-3-yl)ur", 2.18626146152 },
	{ "WKBBJZCTVLRYQV-UHFFFAOYSA-N", 0.609758930373 },
	{ "ZKVCQHWMJWLWFP-UHFFFAOYSA-N", 1.56175860197 },
	{ "1CC2=CC=CC=C2CC1=NN=C(N)N", 1.80526447949 },
	{ "1-(3-azanylcyclopentyl)-22-dimethyl-propan-1", 0.452208004873 },
	{ "hI=1S/C7H11FO/c1-9-7-4-2-6(8)3-5-7/h27H3-5H21H3", 0.19032256061 },
	{ "WYTISXSBNVUFDJ-UHFFFAOYSA-N", 0.519164484037 },
	{ "KXTCDRFJWOBXKH-UHFFFAOYSA-N", 0.386016467854 },
	{ "XQQMKKREWAHFON-UHFFFAOYSA-N", 0.991321522341 },
	{ "3-chloro-3-(25-dichlorophenyl)-2-ethyl-acrylonitri", 1.93383466573 },
	{ "JKXCETDPRRPBDO-UHFFFAOYSA-N", 1.51879153407 },
	{ "5-benzoxy-610-dimethyl-11-tetrahydropyran-2-yloxy-undec-2-en-6", 0.507893378097 },
	{ "AAAYRHLEKBZLST-UHFFFAOYSA-L", 0.488445128949 },
	{ "FENPMXQWOHVLHP-UHFFFAOYSA-N", 0.436922249959 },
	{ "34H38F3N7O5S", 0.234305493799 },
	{ "25H27ClN4O6", 1.22094573245 },
	{ "11H15ClN5O4", 0.887937123093 },
	{ "37H56N8O4", 1.92770293355 },
	{ "N-[2-methyl-5-(oxiran-2-yl)phenyl]ethanamid", 2.18130340177 },
	{ "1=CC(=C(C=C1)[N+](=O)[O-])C(C)(C)C(=O)O", 1.27131166022 },
	{ "(2-nonadecanoyloxy-3-octadeca-6912-trienoyloxy-propyl) ic", 0.333781391232 },
	{ "(C)(C)C(CC(=O)C1=C2CN(CCN2C(=N1)C3=CC=CC=C3)C4CC4)C(=O)N", 1.14690102193 },
	{ "GRAQVEBGNURPIW-UHFFFAOYSA-N", 0.493811051743 },
	{ "5-(2-aminoacetyl)oxy-2-(dodecanoylamino)-5-oxo-pentanoic acid", 2.25626271733 },
	{ "1=CC=CC=C1CC2NC(=C3C=CC(=CC3=O)N)ON2", 0.970188785397 },
	{ "VECKVVYHELFKEW-UHFFFAOYSA-N", 1.343316034 },
	{ "LQSLAIWQQXRMA-UHFFFAOYSA-N", 2.17936888514 },
	{ "AVVMLVITIZCGL-UHFFFAOYSA-N", 0.685837592365 },
	{ "MCZCXVSGFOFIEQ-UHFFFAOYSA-O", 0.495668699387 },
	{ "OLBBSMYCISWXQC-UHFFFAOYSA-N", 0.473430648153 },
	{ "33H31N5O6", 0.534384311326 },
	{ "KGVQTGVFONALR-UHFFFAOYSA-N", 0.837062345223 },
	{ "2-[benzoyl-[(3-oxooxolane-2-carbonyl)amino]amino]propanoic acid", 0.274806941512 },
	{ "18H23N4O4+", 1.399044311 },
	{ "7-(4-azanylpiperidin-1-yl)carbonyl-9aH-thieno[32-c]quinolin-4", 2.16381252379 },
	{ "(C)(C)N1CC(C2=C3C(=C(C=C21)O)C=CC=C3[N+](=O)[O-]", 2.19706999368 },
	{ "1=CC=C(C=C1)CCCC(C(C(=O)OCC2=CC=CC=C2)C(=O)OCC3=CC=CC=C3)C(=O)O", 0.402312788248 },
	{ "(C)(C)C1=NOC(=C1)CC(=O)C2CCCCN2C(=O)OC(C)(", 1.38148024108 },
	{ "FTZJENMBJYNJDJ-UHFFFAOYSA-N", 2.10777520151 },
	{ "1CCCC(CC1)(C(=NO)N)C(=O)N2CCCC2C(=O)N", 0.391518940172 },
	{ "(=CCC=C(C)C1(C(C(=C)CCC12CO2)Br)O", 1.49341230631 },
	{ "OXYUSJFEUNZDFU-UHFFFAOYSA-N", 1.591388556 },
	{ "ROLMYTQJWVXNRU-UHFFFAOYSA-N", 1.01627155682 },
	{ "(C)(C)OC(=O)NCC1CCN(CC1)C2C3=NC(=O)NC3=NC=N2", 2.31156849298 },
	{ "NVAQRAQIIMROCN-UHFFFAOYSA-N", 1.26168927699 },
	{ "1C(C(=O)NC2=C(C1=O)C(=CC=C2)N)CC3=CC=CC(=C3)C#N", 0.594577838658 },
	{ "[B].C1CN(CCC2=C1C=CC(=C2)CCN3C=C[C-]=CC3=O)C(=O)C(F)(F)F.[W]", 0.92045528792 },
	{ "30H30ClFN6O2S", 2.07458879781 },
	{ "QKWXTNGATVYZSD-UHFFFAOYSA-N", 1.25983497227 },
	{ "SAHVSCOCIKZLEN-UHFFFAOYSA-N", 2.37327102011 },
	{ "1CN(C(=O)C1(C(=O)NCC2=C(C(=C(C=C2)F)Cl)F)O)C3=CC=CC=C3", 0.371848765495 },
	{ "(6E)-4-nitro-6-[(2-nitroanilino)methylidene]-1-cyclohexa-24-di", 0.266398490672 },
	{ "ZEHSBWQBXDDLIU-UHFFFAOYSA-N", 1.17688542414 },
	{ "56-dinitrooxyhexanoic acid (4-nitrophenyl) ester", 0.604109093079 },
	{ "SMBBQPLBYBZODP-UHFFFAOYSA-N", 1.86404012387 },
	{ "MXVDSOXHOQOMOP-UHFFFAOYSA-N", 0.558274369781 },
	{ "OC1=CC2=C(C=C1)NC3=C2CC4(C(=O)N(C(=O)N4C3C5=CC=CC=C5)CCNCC6CC6", 1.28050598784 },
	{ "(phenylmethyl) 4-ethyl-2-oxidanylidene-3H-quinoline-3-carboxy", 1.75892808715 },
	{ "brassylic acid dihex-2-enoyl ester", 0.871541196289 },
	{ "[CH-]C[CH-]C.[Y].[Y].[Y]", 2.33854631354 },
	{ "NNGOPOAFEGYTAZ-UHFFFAOYSA-N", 2.02583782447 },
	{ "15H11Cl2N3O3S", 0.524971943819 },
	{ "hI=1S/C7H10NO.W/c1-3-6-4-5-8(2)7(6)9;/h4-5H21-2H3;/q-1;", 0.829221885366 },
	{ "55H86N20O25S3", 1.01714893902 },
	{ "4-(1-benzofuran-3-yl)-3-hydroxybutanamid", 2.39227296862 },
	{ "FBZRXUZYDABKNW-UHFFFAOYSA-N", 0.413877080303 },
	{ "SJWWAAGBXWYYJE-UHFFFAOYSA-N", 1.79616913393 },
	{ "JHDLDHCEZDWZFI-UHFFFAOYSA-N", 1.03573808964 },
	{ "28H26ClF6N5O6", 0.409480030666 },
	{ "S(=O)(=O)NC(CS(=O)(=O)C)(C1CCN1C(=O)O)C(=O)O", 0.880166843929 },
	{ "O[P+](=O)OP(=O)(O)O.CO[P+](=O)OP(=O)(O)O.C(C(=O)O)O.[Ti]", 0.580270507688 },
	{ "JQOQAHNDKROBIL-UHFFFAOYSA-N", 1.92568878416 },
	{ "23H28BrNO3", 0.478795958843 },
	{ "BWGMKCWOPPHENR-UHFFFAOYSA-N", 0.920380902693 },
	{ "cetic acid 2-ketobutyl ester;yttrium", 2.30326840577 },
	{ "2-[(2-aminoquinoxalin-6-yl)methylene]thiazolidin-4", 1.40534621568 },
	{ "19H25F3O10", 1.30503109798 },
	{ "1=C(C(=NN1CC2=C(C=C(C=C2)NC(=O)C=CC3=CC=CC=C3)F)C)CC(=O)O", 0.753286920436 },
	{ "20H20IrN4O-2", 1.11488551922 },
	{ "3-[4-[methyl(222-trifluoroethyl)amino]phenyl]acrylic acid", 1.50505013366 },
	{ "N-[4-(furan-2-yl)butan-2-yl]-6-oxo-3H-pyridine-3-sulfonamid", 2.39640779074 },
	{ "N&apos;-hydroxy-2-(propan-2-ylthio)propanimidamid", 0.617379707752 },
	{ "15H36O16Si3", 0.484160163203 },
	{ "29H54N2O7S", 1.33853179238 },
	{ "JZWQZCHUXWRAT-UHFFFAOYSA-N", 2.24367143763 },
	{ "MGGLNFSJPTWNF-UHFFFAOYSA-N", 0.515318521511 },
	{ "21H26N4S", 2.36741076509 },
	{ "2-methylpent-2-enoic acid [4-(2-thenoylamino)phenyl] ester", 1.7877641195 },
	{ "3-(2-butoxyethylamino)-8aH-isoquinolin-1", 1.2041718964 },
	{ "1=CC(=NN1C2=CC=C(C=C2)COC(=O)C=CC3=CC=C(C=C3)C(=O)O", 0.290770731257 },
	{ "N-(2-acetamidoethyl)-3-[1-(2-cyanoethyl)indol-3-yl]acrylamid", 0.442264964609 },
	{ "KBOJJUUHFMLLSP-UHFFFAOYSA-N", 0.334110173742 },
	{ "1CCC(C(=O)N1)C(=O)NC2=NC(=CS2)C3=CC4=C(C=C3)OCO4", 0.808398241036 },
	{ "1=NC(=C(C=C1)NC(=O)N2CCCC2C3=NN=C(N3CC4=CC=C(C=C4", 1.01781063735 },
	{ "NGHHAEDPIDZLGV-MQWKRIRWSA-N", 1.63916557966 },
	{ "44H45N5O4", 0.384123623429 },
	{ "3-[2-(4-cinnamylpiperazino)ethyl]-55-dimethyl-hydantoi", 1.80170755994 },
	{ "MLPWKQDHDFKCFG-UHFFFAOYSA-N", 2.21583975569 },
	{ "14H17N4O10Re-4", 1.47012013087 },
	{ "(C)C(=O)N(C)CC1=CC=CC(=C1)C=CC(=O)O", 1.70945465215 },
	{ "STFNMFIKUPCPS-UHFFFAOYSA-N", 2.24324969743 },
	{ "2-[(4-amino-6-keto-hexahydropyrimidin-2-yl)thio]nicotinonitri", 0.924070501697 },
	{ "NAKNOTNFWVBZAL-UHFFFAOYSA-N", 1.39907823974 },
	{ "XYNUWIKQRZQEBQ-UHFFFAOYSA-N", 0.108347427522 },
	{ "76H94F6N22O22S", 1.2743220025 },
	{ "6-amino-2-[(5-amino-8-quinolyl)sulfanyl]hexahydropyrimidin-4", 2.20505714604 },
	{ "4-azido-5-butan-2-yloxolan-3", 2.13897448678 },
	{ "4-[(6-chloro-8-methyl-5-quinolyl)amino]-4-keto-but-2-enoic acid", 0.941622420954 },
	{ "29H39N5O5", 2.06620272524 },
	{ "methyl 2-ethylpyrrolidin-4-ide-2-carboxylate;yttrium", 1.1254328267 },
	{ "1CC(C[N+]1(C(=O)O)C(=O)OC(C)(C)C)N(C(=O)CN)C(=O)OC(C)(", 1.22047575697 },
	{ "#CC1CNC(=O)NC1=O", 1.48126357514 },
	{ "N1CCC2=C(C1)C=C(C=C2)NC3NCC4C(N3)N(C(=O)N(C4=O)C5=C(C=CC=C5", 1.36191920697 },
	{ "16H10BrN3O5S", 0.634119600518 },
	{ "41H56N4O11", 1.55650511885 },
	{ "47H66Li2N2OSi2", 1.4927643425 },
	{ "3-[6-(2-oxo-6H-quinolin-6-yl)pyridin-2-yl]prop", 1.52911193916 },
	{ "24H27F3O4S", 0.818134059333 },
	{ "ODSRQVLMKAHZAW-UHFFFAOYSA-N", 0.635671592279 },
	{ "BZNNEGJMWWNDSU-UHFFFAOYSA-N", 0.553236465516 },
	{ "(C)C(C(=O)N1CCC(CC1)C2=CC(=C(C=C2)Cl)Cl)NC(=O)C3=CC(=CC=C3)O", 0.394657905551 },
	{ "RWKQKYLLKYKDN-UHFFFAOYSA-N", 1.81172980865 },
	{ "RNOMVYQIFJTADJ-UHFFFAOYSA-N", 0.660006465188 },
	{ "LHXDVTHVUTZIOV-UHFFFAOYSA-N", 2.30578534224 },
	{ "(C)(C)C(=O)NC1=CC(=C(C=C1)N2CCNCC2)C#N.C(C(C(=O)O)O)(C(=O)O)O", 1.07057891569 },
	{ "1C(N(C1=O)CCCC2=CC=[C-]S2)COC3=CC(=CC(=C3)Cl)Cl.[Y]", 1.0175183989 },
	{ "5-methyl-2-(1-propan-2-yloxypropan-2-yl)cyclohexan-1", 2.0066032824 },
	{ "MZIQSSJYTQWSBI-UHFFFAOYSA-N", 0.712328765884 },
	{ "4-[(4-amino-6-oxo-13-diazinan-2-yl)sulfanyl]benzonitri", 1.7889527703 },
	{ "3-benzoyl-N-propyl-2-pyridinecarboxamid", 1.3909465417 },
	{ "(=CC1=CC=CC=C1OCCO)F", 1.5126755257 },
	{ "9-eicosen-1", 1.98519548149 },
	{ "3-amino-N-cyclohexyl-N-ethyl-3-hydroxyimino-2-methylpropanamid", 0.132250533827 },
	{ "2-[(4-hydroxypyrrolidin-3-yl)-methylsulfamoyl]acetic acid", 1.57783941068 },
	{ "HVJCWOXRHVNMMM-UHFFFAOYSA-N", 2.20192525019 },
	{ "1CC(CC1C2=CN=C3N2C4=C(NC=C4)N=C3)CS(=O)(=O)N5CC(C5)(F)F", 1.54190115855 },
	{ "49H62F2N12O6", 0.100174627226 },
	{ "UOXFOOHOAAFGIT-UHFFFAOYSA-N", 1.42671561481 },
	{ "29H22FN5O", 1.58252039075 },
	{ "1CC(=O)C2C34C1C(CC5=C3C(=C(C=C5)OC)O2)N(CC4)CC6CCC6", 2.07443108928 },
	{ "WJVFDNEXTCDR-UHFFFAOYSA-N", 0.413890409946 },
	{ "MINXZBXOLYOJOY-UHFFFAOYSA-N", 0.142014667766 },
	{ "QLCAXSWSKHHXKN-UHFFFAOYSA-N", 1.03589538314 },
	{ "BJOPRYGYKOXDKE-UHFFFAOYSA-N", 2.22182628315 },
	{ "FMHHPDNSBDRW-UHFFFAOYSA-N", 0.701996605641 },
	{ "OC1=CC=CC2=C1OC(=C2)C=C3C4=C(C=C(C=C4)Cl)NC3=O", 1.64953373586 },
	{ "bis(oxidanidyl)phosphane;rhodium", 1.93739009276 },
	{ "2-(3-butylpiperazin-1-yl)-6-(trifluoromethyl)-13-benzothiaz", 0.3423889367 },
	{ "19H21BrN9O2+", 1.81230045672 },
	{ "hI=1S/C4H9ClNO2S/c1-8-4(7)3(2-9)5-6/h39H26H21H3/q-1", 2.15284063874 },
	{ "1-pyrrolidinopyrrolidine-2-carbaldehyd", 1.31068217158 },
	{ "1(OC2C(COC(C2O1)OCC=C)N", 0.160549944169 },
	{ "FXXMCRAQLWNRGX-UHFFFAOYSA-N", 0.83067873785 },
	{ "BXQQQURABWEJLK-UHFFFAOYSA-N", 0.510602352251 },
	{ "(C)(C)OC(=O)NC(CC1=CC=CC=C1)C(=O)N2CCCC2CNC3=CC=C(C=C3)C(=N)N", 1.88859948896 },
	{ "1=C(C=CC(=C1Cl)[N+]#[C-])NC(CO)C(=O)O", 2.16709758887 },
	{ "ZJOYNNSSRBIMLE-UHFFFAOYSA-N", 2.39286548266 },
	{ "2-(phosphinoamino)ethanone;yttrium", 1.81951980532 },
	{ "HMTQVUGVTQWUIK-UHFFFAOYSA-M", 1.52233449987 },
	{ "KSAYYFOQQVNT-UHFFFAOYSA-N", 1.51976686804 },
	{ "4-[tert-butyl(dimethyl)silyl]oxy-N-hydroxy-benzimidoyl chlorid", 1.2579781889 },
	{ "rimethoxymethane;yttrium", 0.837307487757 },
	{ "3-methyl-2-prop-1-enyl-quinoli", 1.55039817374 },
	{ "38H41FN6O3", 0.645678063173 },
	{ "OGNYVPWOZRDXLB-UHFFFAOYSA-N", 0.310669001151 },
	{ "HJVHVIMJTDRFKG-UHFFFAOYSA-N", 2.25413445246 },
	{ "GSVOAJILMQAKJU-UHFFFAOYSA-N", 0.623380020196 },
	{ "4-amino-8-methyl-N-(4-pyridylthio)chroman-7-carboxamid", 1.4649448082 },
	{ "2-(2-keto-3H-pyridin-3-yl)-3-oxa-1-azaspiro[4.5]dec-1-en-4", 1.65571739618 },
	{ "QUBLFNZZFSPSH-UHFFFAOYSA-N", 1.43530063512 },
	{ "3-(13-benzodioxol-5-yloxy)propoxy-(4-fluorobenzylidene)ami", 1.55285254095 },
	{ "FLCJWEJXHUUKHE-UHFFFAOYSA-N", 0.312817954766 },
	{ "VNJSRGNBIAKQOG-UHFFFAOYSA-N", 0.248987429618 },
	{ "(E)-4-phenyl-4-tributylstannyl-but-3-en-1", 1.42320792438 },
	{ "GPGYHFVJIWSVPA-UHFFFAOYSA-N", 1.32547182 },
	{ "3-[4-morpholinyl(oxo)methyl]-5-oxohexanoic acid", 1.17925203717 },
	{ "(=O)NC1CCC(CC1)NC2=NC3=C(C=NN3C(=C2)NC4CC4)C=C5CC(=O)NC5=O", 2.3297574189 },
	{ "XOFFBTRAYMRAEJ-UHFFFAOYSA-N", 0.88013852009 },
	{ "OC1=CC=C(C=C1)N2C(C(C2=O)OCC3=CC=CC=C3)CF", 1.60752931661 },
	{ "LXTADKQUVQFITQ-UHFFFAOYSA-N", 2.22853888134 },
	{ "27H28Cl3N3O5", 0.805479753343 },
	{ "FSFZLAVLSPHCG-UHFFFAOYSA-N", 0.571298506235 },
	{ "WAACEXUDWHTNPS-UHFFFAOYSA-N", 0.640301586865 },
	{ "JIDXTRYXZJATJC-UHFFFAOYSA-N", 1.57660595222 },
	{ "FeH4NO4P-2", 2.24376352448 },
	{ "(C)OCC1CC(CN1)NC2=NC(=CN=C2)C3=CN=C4N3C=CC(=C4)O", 1.79857572649 },
	{ "QLHXETJIMAVLOQ-UHFFFAOYSA-N", 0.699374769095 },
	{ "1C2CC(C1C=C2)O[C-]=O.[Rf]", 1.63723615991 },
	{ "SIUNARRYLXPSRF-UHFFFAOYSA-N", 0.545377422238 },
	{ "3-(22-dimethylbutanoyloxy)propyl-ethylcarbamic acid", 1.62840656063 },
	{ "LVGYRLTWKIMYJX-UHFFFAOYSA-O", 1.12961328559 },
	{ "AUHIQPGTPVYGPR-UHFFFAOYSA-N", 0.181534127281 },
	{ "3-[3-(norbornane-2-carbonylamino)phenyl]acrylic acid", 1.14131849628 },
	{ "benzyl 3-[6-(dimethylamino)-3-pyridyl]prop-2", 1.51781251205 },
	{ "1=C(N=NC(C1=C(N)N=O)OCC(", 2.00851016855 },
	{ "7H7N5OS", 0.754976984938 },
	{ "6-[(2-hydroxyanilino)methylidene]cyclohexa-24-dien-1-one;mang", 0.730693225492 },
	{ "2-[3-ethyl-26-bis(oxidanylidene)-13-diazinan-1-yl]ethanamid", 1.37172004734 },
	{ "46H36F7IrN4P2+2", 1.57498393545 },
	{ "27H29N3O5Si", 0.944073854847 },
	{ "bis-m-tolyl-[1-[2-(2-phosphanylphenyl)cyclopentyl]ethyl]phosph", 0.447942270675 },
	{ "44H42Cl2N4O7", 2.28252121954 },
	{ "1CN(CC1S(=O)(=O)C2=CC=C(C=C2)CN3CCCCC3)S(=O)(=O", 0.711052825511 },
	{ "GAVHDXSKCGIWST-UHFFFAOYSA-N", 2.20707976547 },
	{ "23H24N4O5", 0.970157382908 },
	{ "WJKWTHYIOPILCM-UHFFFAOYSA-N", 0.628768723785 },
	{ "21H26N2O8S2", 0.111799041171 },
	{ "1=CC(=CC=C1)NC(=O)N2CCC(C2)CC(C)C3=CC=CC4=CC=CC=C43", 1.11119431019 },
	{ "(=O)COCCOC1=CC=C(C=C1)C(=C(CCCl)C2=CC=CC=C2)C3=CC=CC=C3", 0.441388990169 },
	{ "JGGPPGBJMBRLPK-UHFFFAOYSA-N", 0.386975016659 },
	{ "1CN(CCN1C2=CC=CC=N2)S(=O)(=O)CCNC(=O)C=CC3=CC=C(C=C3", 0.743635454084 },
	{ "cetimidoylazanide;yttrium", 1.64096252169 },
	{ "MBHIUPCAJGMIN-UHFFFAOYSA-N", 0.442260763563 },
	{ "UPDMTLJAQIQFCX-UHFFFAOYSA-N", 1.5202154293 },
	{ "69H98O13", 1.47559088428 },
	{ "hI=1S/C10H7F3/c11-10(1213)9-5-4-7-2-1-3-8(7)6-9/h13-6H2H2", 0.250142190721 },
	{ "6-[amino(3-bicyclo[2.2.1]heptanyl)methyl]quinoxaline-23-di", 1.39747388547 },
	{ "2-(aminocarbohydroximoyl)-N-(14-dioxan-2-ylmethyl)butyramid", 1.75388504009 },
	{ "28H22F2N4O4", 1.36753800561 },
	{ "3-methylidene-6-methylsulfanyl-7H-pyrazolo[34-d]pyrimidin-4", 1.88790676047 },
	{ "WLUVAFLXGXBEEY-UHFFFAOYSA-N", 0.130685745476 },
	{ "40H35N7O6S3", 1.47503654236 },
	{ "1C=NC(=O)C2=CC(=C(C=C12)OC3CCN(CC3)C(C)C)F", 1.57899936627 },
	{ "NHLPVXKJGDWTBS-UHFFFAOYSA-N", 1.12607714742 },
	{ "N-[1-(2-bromophenyl)ethyl]-3-(2-methoxyphenyl)acrylamid", 2.06831261255 },
	{ "XFOTIMBDAOSOZ-UHFFFAOYSA-N", 1.97738221708 },
	{ "1=CC=C(C2=NC=CN12)C#N", 0.433477508365 },
	{ "GABZPMKAAVEBDR-UHFFFAOYSA-N", 0.580110497619 },
	{ "N1C(C(C(=O)NC1=O)NCC2=CC=CC=N2)N", 0.880461803689 },
	{ "(=O)OC(C#C[Si](C)(C)C)C1=CC=C(C=C1)C#CC2=CC=CC=C2", 1.65311274152 },
	{ "4-[amyl(isopropyl)amino]-N&apos;-hydroxy-3-methyl-benzamidi", 0.427815435116 },
	{ "OCCN1C(C(C(=O)NC1=O)NCCN2CCCC2)N", 0.151573583141 },
	{ "36H44N2O11S", 1.54180063213 },
	{ "UEDPXOQOVLLPO-UHFFFAOYSA-N", 0.161187285844 },
	{ "(4-styrylsulfanylphenyl) ac", 0.888259489825 },
	{ "1CNCCC2=C1C=C(C=C2)Cl.OP(=O)(O)O", 1.91340427238 },
	{ "(6-ethylpyridine-2-carbothioyl)sulfanylgallium", 1.01527807105 },
	{ "3-(aminocarbohydroximoyl)-N-[2-(2-methoxyethoxy)ethyl]benzamid", 1.37575232355 },
	{ "36H40NO", 2.26479267292 },
	{ "2-azanyl-34-dimethyl-5-(phenylmethoxymethyl)cyclopentan-1", 0.151471831134 },
	{ "1CCC(C1)N(CC(=O)O)C(=O)C=CC2=CC(=CC=C2)F", 0.667322450844 },
	{ "[2-(4-aminophenoxy)-3-pyridyl]ami", 1.56019760986 },
	{ "24H33ClN2O6S", 1.02567206711 },
	{ "34-difluoro-N&apos;-(3-nitrobenzyl)oxy-benzamidi", 0.673658919384 },
	{ "hI=1S/C6H5N4.Ir/c1-3-7-9(5-1)10-6-2-4-8-10;/h1-5H;/q-1;", 0.425956446859 },
	{ "QKJZISVTFPJIQK-UHFFFAOYSA-N", 2.07329292003 },
	{ "(C1CCCCC1)NC(=O)C2=C(C=CC(=C2)OC3=C(C=CC=N3)C4=NC(=NC=C4)NC)F", 2.10319368579 },
	{ "2-butenedioic acid O4-phenyl ester O1-prop-2-enyl ester", 0.965963915556 },
	{ "=CC1CCC2C1OCC=C2", 0.430312003876 },
	{ "(C)(C)OC(=O)N1CC2(C1)CN(C/C2=NOC)CC3=CC=CC=C3", 1.17209144245 },
	{ "1=CC=C(C=C1)N(C2=CC=CC=C2)C3=CC=C(C=C3)C=CC4=CC5=CC=CC=C5C=C4", 0.177167477078 },
	{ "(C)(C)OC(=O)N1CCC(CC1)NC(=O)C=CC2=CC=CO2", 2.00608994853 },
	{ "RHWHQWLKMNVXJZ-UHFFFAOYSA-N", 1.63988730988 },
	{ "SUZPHKXJPACDU-UHFFFAOYSA-N", 1.18839827021 },
	{ "39H54N2O5", 0.359912532127 },
	{ "HFVBYPPAFQTZRH-UHFFFAOYSA-N", 0.636625748511 },
	{ "1=CC2=NC(=C3C4C=CC=NC4=NN3)N=C2C=C1", 1.91354192542 },
	{ "UNLHYLJKTSGDRT-UHFFFAOYSA-N", 0.636406095748 },
	{ "19H16ClO4PS", 0.106895254334 },
	{ "VWLFQJCHJPSIF-UHFFFAOYSA-N", 1.78555506456 },
	{ "ZKNOJUQNCHIPET-UHFFFAOYSA-N", 1.54469204577 },
	{ "2-amino-N-(2-bromobenzyl)-33-dimethyl-butyramid", 1.44364060416 },
	{ "HHDPPTFWJICQQR-UHFFFAOYSA-M", 0.733260829739 },
	{ "TVBHCBQZISVNG-UHFFFAOYSA-N", 0.375303780635 },
	{ "(=O)SCCOP(=O)(N(C)C)OCC1C(C(C(O1)N2C=CC(=NC2=C)N)(C)O)O", 0.430733559639 },
	{ "3-(35-dichloro-4-isopropyl-phenyl)but-2-enenitri", 0.59298106275 },
	{ "phenacyl 2-(2-methylbutanoylamino)ac", 1.60497618226 },
	{ "N1NC(=C2C=C(N=C2C)C)CC(=N1", 0.153372271219 },
	{ "1=C(N2C(=CC(=N2)C3CCCCN3C(=O)OC(C)(C)C)N=C1C)C4=CC=CC=C4", 0.116457697882 },
	{ "=CC(=O)NCC1CCCCN1", 1.37499214455 },
	{ "2-amino-6-[[3-(3-pyridyl)acryloyl]amino]hexanoic acid", 0.574578989372 },
	{ "1-hydroxy-3-(3-oxolanylamino)-2-p", 1.06940721462 },
	{ "phosphono 2-azanyl-4-methylsulfanyl-b", 1.65248002882 },
	{ "GKANGMJEGDFSKW-UHFFFAOYSA-N", 2.27566252812 },
	{ "SBHRNSPQCRZIK-UHFFFAOYSA-N", 2.04933385564 },
	{ "KMHFNGUPMMASPQ-UHFFFAOYSA-N", 2.04586202981 },
	{ "28H35ClN4O3", 0.966859084157 },
	{ "ZPMAATPXKFIZRL-UHFFFAOYSA-N", 0.277800580543 },
	{ "32H54N2O2", 1.63773786103 },
	{ "YAMJCPQENZRLQX-UHFFFAOYSA-N", 1.98572823475 },
	{ "BUDQGIIQIUFAEF-UHFFFAOYSA-N", 2.12529952096 },
	{ "RUGYSJXRGYQE-UHFFFAOYSA-N", 0.824935542404 },
	{ "UHJULZLLHXSTIW-UHFFFAOYSA-N", 0.638501293443 },
	{ "3-(ethoxymethyl)-4-methylene-1H-triazol-3-ium-5-thi", 0.656526252263 },
	{ "HEVXAXIHTUFFOG-UHFFFAOYSA-N", 1.25091191656 },
	{ "(=C1C=NC2C1C(=O)NC(=N2)N)N", 0.133558416389 },
	{ "(C(=NO)N)OC1=CC2=C(C=C1)NC(=O)CC2", 1.74487975535 },
	{ "18H29NO3S", 0.457656234786 },
	{ "N-(1H-imidazol-2-yl)-2-keto-nipecotamid", 0.801849028128 },
	{ "3-[2-(hydroxymethyl)pyrrolidin-1-yl]carbonyl-3H-quinolin-4", 0.800420621896 },
	{ "FUUPUGXOBQCCOH-UHFFFAOYSA-N", 0.689313663106 },
	{ "3-(23-dichlorophenyl)prop-2-enoyl azid", 1.55616573649 },
	{ "JUZZKRAENRWPBC-UHFFFAOYSA-N", 0.791719004249 },
	{ "AJCUGXSEVQIPG-UHFFFAOYSA-N", 1.4981635245 },
	{ "NHIUABLWAPZSCK-UHFFFAOYSA-N", 2.24895452007 },
	{ "XWDBPFPIIUIHLC-UHFFFAOYSA-N", 0.839948650136 },
	{ "15H17ClFNO2", 1.1440590059 },
	{ "26H35ClN2O6S", 0.515940520049 },
	{ "[1-(2-azanylbutanoyl)pyrrolidin-2-yl]boronic acid", 0.769932329457 },
	{ "(CC1=CC=CC=C1)(CO)NC(=O)CCCN2C(=O)C(=CC3=CC=CS3)SC2=S", 0.686844351886 },
	{ "1(CCC(C2=C1C=C(C=C2)CO[Si](C)(C)C(C)(C)C)CC=", 0.628732251496 },
	

};

#endif
